@@ -26,15 +26,22 @@ LOG = logging.getLogger(CYLC_LOG)
 LOG.addHandler(logging.NullHandler())
 
 LOG_LEVELS = {
+    "DEBUG": logging.DEBUG,
     "INFO": logging.INFO,
     "NORMAL": logging.INFO,
     "WARNING": logging.WARNING,
     "ERROR": logging.ERROR,
     "CRITICAL": logging.CRITICAL,
-    "DEBUG": logging.DEBUG,
 }
 
 
+class LoggerAdaptor(logging.LoggerAdapter):
+    """Adds a prefix to log messages."""
+    def process(self, msg, kwargs):
+        ret = f"[{self.extra['prefix']}] {msg}" if self.extra else msg
+        return ret, kwargs
+
+
 def environ_init():
     """Initialise cylc environment."""
     # Python output buffering delays appearance of stdout and stderr
@@ -46,7 +53,7 @@ def environ_init():
 
 environ_init()
 
-__version__ = '8.0.2'
+__version__ = '8.2.3'
 
 
 def iter_entry_points(entry_point_name):
