@@ -263,6 +263,13 @@ EVENTS_SETTINGS = {  # workflow events
         .. versionchanged:: 8.0.0
 
            {REPLACES}``abort on inactivity``.
+    ''',
+    'restart timeout': '''
+        How long to wait for intervention on restarting a completed workflow.
+        The timer stops if any task is triggered.
+
+        .. versionadded:: 8.2.0
+
     '''
 }
 
@@ -355,7 +362,7 @@ Supported for use with job runners:
 - slurm_packjob
 - moab
 
-Directives are written to the top of the task job script in the correct format
+Directives are written to the top of the job script in the correct format
 for the job runner.
 
 Specifying directives individually like this allows use of default directives
@@ -395,8 +402,14 @@ so you may want to configure more frequent polling.
 SUBMISSION_RETY_DESCR = f'''
 Cylc can automatically resubmit jobs after submission failures.
 
-A list of intervals which define when the scheduler will resubmit jobs if
-submission fails.
+Submission retry delays is a list of ISO 8601 durations which tell Cylc
+how long to wait before the next try.
+
+The job environment variable ``$CYLC_TASK_SUBMIT_NUMBER`` increments with each
+job submission attempt.
+
+Tasks only go to the ``submit-failed`` state if job submission fails with no
+retries left.
 
 .. versionchanged:: 8.0.0
 
@@ -439,7 +452,8 @@ Event handlers can be stored in the workflow ``bin/`` directory, or
 anywhere the scheduler environment ``$PATH``. They should return quickly.
 
 Multiple event handlers can be specified as a list of command line templates.
-For supported template variables see :ref:`task_event_template_variables`.
+For supported template variables see :ref:`user_guide.runtime.\
+event_handlers.task_event_handling.template_variables`.
 Python template substitution syntax is used:
 `String Formatting Operations in the Python documentation
 <https://docs.python.org/3/library/stdtypes.html
@@ -448,16 +462,38 @@ Python template substitution syntax is used:
 
 TASK_EVENTS_SETTINGS = {
     'handlers': '''
-        Specify a list of command lines or command line templates as task
-        event handlers.
+        Commands to run on task :cylc:conf:`[..]handler events`.
+
+        A command or list of commands to run for each task event handler
+        set in
+        :cylc:conf:`flow.cylc[runtime][<namespace>][events]handler events`.
+
+        Information about the event can be provided to the command
+        using :ref:`user_guide.runtime.event_handlers.\
+task_event_handling.template_variables`.
+        For more information, see
+        :ref:`user_guide.runtime.task_event_handling`.
+
+        For workflow events, see
+        :ref:`user_guide.scheduler.workflow_event_handling`.
+
+        Example::
+
+           echo %(event)s occurred in %(workflow)s >> my-log-file
+
     ''',
     'execution timeout': '''
         If a task has not finished after the specified interval, the execution
         timeout event handler(s) will be called.
     ''',
     'handler events': '''
-        Specify the events for which the general task event handlers should
-        be invoked.
+        A list of events for which :cylc:conf:`[..]handlers` are run.
+
+        Specify the events for which the general task event handlers
+        :cylc:conf:`flow.cylc[runtime][<namespace>][events]handlers`
+        should be invoked.
+
+        See :ref:`user_guide.runtime.task_event_handling` for more information.
 
         Example::
 
@@ -831,6 +867,8 @@ with Conf('global.cylc', desc='''
                     vdr_type = VDR.V_INTERVAL
                     if item == "stall timeout":
                         default = DurationFloat(3600)
+                    elif item == "restart timeout":
+                        default = DurationFloat(120)
                     else:
                         default = None
                 Conf(item, vdr_type, default, desc=desc)
@@ -1016,11 +1054,11 @@ with Conf('global.cylc', desc='''
                     Alternative location for the run dir.
 
                     If specified, the workflow run directory will
-                    be created in ``<this-path>/cylc-run/<workflow-name>``
+                    be created in ``<this-path>/cylc-run/<workflow-id>``
                     and a symbolic link will be created from
-                    ``$HOME/cylc-run/<workflow-name>``.
+                    ``$HOME/cylc-run/<workflow-id>``.
                     If not specified the workflow run directory will be created
-                    in ``$HOME/cylc-run/<workflow-name>``.
+                    in ``$HOME/cylc-run/<workflow-id>``.
                     All the workflow files and the ``.service`` directory get
                     installed into this directory.
 
@@ -1030,11 +1068,11 @@ with Conf('global.cylc', desc='''
                     Alternative location for the log dir.
 
                     If specified the workflow log directory will be created in
-                    ``<this-path>/cylc-run/<workflow-name>/log`` and a
+                    ``<this-path>/cylc-run/<workflow-id>/log`` and a
                     symbolic link will be created from
-                    ``$HOME/cylc-run/<workflow-name>/log``. If not specified
+                    ``$HOME/cylc-run/<workflow-id>/log``. If not specified
                     the workflow log directory will be created in
-                    ``$HOME/cylc-run/<workflow-name>/log``.
+                    ``$HOME/cylc-run/<workflow-id>/log``.
 
                     .. versionadded:: 8.0.0
                 """)
@@ -1042,11 +1080,11 @@ with Conf('global.cylc', desc='''
                     Alternative location for the share dir.
 
                     If specified the workflow share directory will be
-                    created in ``<this-path>/cylc-run/<workflow-name>/share``
+                    created in ``<this-path>/cylc-run/<workflow-id>/share``
                     and a symbolic link will be created from
-                    ``<$HOME/cylc-run/<workflow-name>/share``. If not specified
+                    ``<$HOME/cylc-run/<workflow-id>/share``. If not specified
                     the workflow share directory will be created in
-                    ``$HOME/cylc-run/<workflow-name>/share``.
+                    ``$HOME/cylc-run/<workflow-id>/share``.
 
                     .. versionadded:: 8.0.0
                 """)
@@ -1055,11 +1093,11 @@ with Conf('global.cylc', desc='''
 
                     If specified the workflow share/cycle directory
                     will be created in
-                    ``<this-path>/cylc-run/<workflow-name>/share/cycle``
+                    ``<this-path>/cylc-run/<workflow-id>/share/cycle``
                     and a symbolic link will be created from
-                    ``$HOME/cylc-run/<workflow-name>/share/cycle``. If not
+                    ``$HOME/cylc-run/<workflow-id>/share/cycle``. If not
                     specified the workflow share/cycle directory will be
-                    created in ``$HOME/cylc-run/<workflow-name>/share/cycle``.
+                    created in ``$HOME/cylc-run/<workflow-id>/share/cycle``.
 
                     .. versionadded:: 8.0.0
                 """)
@@ -1067,11 +1105,11 @@ with Conf('global.cylc', desc='''
                     Alternative directory for the work dir.
 
                     If specified the workflow work directory will be created in
-                    ``<this-path>/cylc-run/<workflow-name>/work`` and a
+                    ``<this-path>/cylc-run/<workflow-id>/work`` and a
                     symbolic link will be created from
-                    ``$HOME/cylc-run/<workflow-name>/work``. If not specified
+                    ``$HOME/cylc-run/<workflow-id>/work``. If not specified
                     the workflow work directory will be created in
-                    ``$HOME/cylc-run/<workflow-name>/work``.
+                    ``$HOME/cylc-run/<workflow-id>/work``.
 
                     .. versionadded:: 8.0.0
                 """)
@@ -1082,7 +1120,7 @@ with Conf('global.cylc', desc='''
         A platform consists of a group of one or more hosts which share a
         file system and a job runner (batch system).
 
-        A platform must allow interaction with the same task job from *any*
+        A platform must allow interaction with the same job from *any*
         of its hosts.
 
         .. versionadded:: 8.0.0
@@ -1391,6 +1429,17 @@ with Conf('global.cylc', desc='''
                  desc=f'''
                 {LOG_RETR_SETTINGS['retrieve job logs command']}
 
+                .. note::
+                   The default command (``rsync -a``) means that the retrieved
+                   files (and the directories above including ``job/log``) get
+                   the same permissions as on the remote host. This can cause
+                   problems if the remote host uses different permissions to
+                   the scheduler host (e.g. no world read access). To avoid
+                   this problem you can set the command to
+                   ``rsync -a --no-p --no-g --chmod=ugo=rwX`` which means the
+                   retrieved files get the default permissions used on the
+                   scheduler host.
+
                 .. versionchanged:: 8.0.0
 
                    {REPLACES}``global.rc[hosts][<host>]retrieve job logs
@@ -1418,11 +1467,15 @@ with Conf('global.cylc', desc='''
                        "[remote]retrieve job logs retry delays")}
             ''')
             Conf('tail command template',
-                 VDR.V_STRING, 'tail -n +1 -F %(filename)s', desc=f'''
+                 VDR.V_STRING, 'tail -n +1 --follow=name %(filename)s',
+                 desc=f'''
                 A command template (with ``%(filename)s`` substitution) to
                 tail-follow job logs this platform, by ``cylc cat-log``.
 
-                You are are unlikely to need to override this.
+                .. warning::
+
+                   You are are unlikely to need to override this. Doing so may
+                   adversely affect the UI log view.
 
                 .. versionchanged:: 8.0.0
 
@@ -1534,8 +1587,8 @@ with Conf('global.cylc', desc='''
 
                 We recommend using a clean job submission environment for
                 consistent handling of local and remote jobs. However,
-                this is not the default behavior because it prevents
-                local task jobs from running, unless ``$PATH`` contains the
+                this is not the default behaviour because it prevents
+                local jobs from running, unless ``$PATH`` contains the
                 ``cylc`` wrapper script.
 
                 Specific environment variables can be singled out to pass
@@ -1614,11 +1667,14 @@ with Conf('global.cylc', desc='''
         with Conf('localhost', meta=Platform, desc='''
             A default platform for running jobs on the the scheduler host.
 
-            .. attention::
+            This platform configures the host on which
+            :term:`schedulers <scheduler>` run. By default this is the
+            host where ``cylc play`` is run, however, we often configure
+            Cylc to start schedulers on dedicated hosts by configuring
+            :cylc:conf:`global.cylc[scheduler][run hosts]available`.
 
-               It is common practice to start Cylc schedulers on dedicated
-               hosts, in which case **"localhost" is the scheduler host and
-               not necessarily where you ran "cylc play"**.
+            This platform affects connections made to the scheduler host and
+            any jobs run on it.
 
             .. versionadded:: 8.0.0
         '''):
@@ -1626,6 +1682,11 @@ with Conf('global.cylc', desc='''
                 List of hosts for the localhost platform. You are unlikely to
                 need to change this.
 
+                The scheduler hosts are configured by
+                :cylc:conf:`global.cylc[scheduler][run hosts]available`.
+                See :ref:`Submitting Workflows To a Pool Of Hosts` for
+                more information.
+
                 .. seealso::
 
                    :cylc:conf:`global.cylc[platforms][<platform name>]hosts`
@@ -1655,7 +1716,7 @@ with Conf('global.cylc', desc='''
         all be suitable for a given job.
 
 
-        When Cylc sets up a task job it will pick a platform from a group.
+        When Cylc submits a job it will pick a platform from a group.
         Cylc will then use the selected platform for all interactions with
         that job.
 
