@@ -222,10 +222,17 @@ with Conf(
 
                The following directories already get installed by default:
 
-               * ``app/``
-               * ``bin/``
-               * ``etc/``
-               * ``lib/``
+               ``ana/``
+                  Rose ana analysis modules
+               ``app/``
+                  Rose applications
+               ``bin/``
+                  Cylc bin directory (added to ``PATH``)
+               ``etc/``
+                  Miscellaneous resources
+               ``lib/``
+                  Cylc lib directory (``lib/python`` added to ``PYTHONPATH``
+                  for workflow config)
 
                These should be located in the top level of your Cylc workflow,
                i.e. the directory that contains your ``flow.cylc`` file.
@@ -598,7 +605,7 @@ with Conf(
             The stop cycle point can be overridden on the command line using
             ``cylc play --stop-cycle-point=POINT``
 
-            .. note:
+            .. note::
 
                Not to be confused with :cylc:conf:`[..]final cycle point`:
                There can be more graph beyond this point, but you are
@@ -681,7 +688,7 @@ with Conf(
 
                     If set to 0 this queue is not limited.
                 ''')
-                Conf('members', VDR.V_STRING_LIST, desc='''
+                Conf('members', VDR.V_SPACELESS_STRING_LIST, desc='''
                     A list of member tasks, or task family names to assign to
                     this queue.
 
@@ -691,7 +698,7 @@ with Conf(
             with Conf('default', meta=Queue, desc='''
                 The default queue for all tasks not assigned to other queues.
             '''):
-                Conf('limit', VDR.V_INTEGER, 0, desc='''
+                Conf('limit', VDR.V_INTEGER, 100, desc='''
                     Controls the total number of active tasks in the default
                     queue.
 
@@ -713,10 +720,11 @@ with Conf(
 
             .. deprecated:: 8.0.0
 
-               Please read :ref:`Section External Triggers` before
-               using the older clock triggers described in this section.
+               These are now auto-upgraded to the newer wall_clock xtriggers
+               (see :ref:`Section External Triggers`). The old way defining
+               clock-triggers will be removed in an upcoming Cylc version.
 
-            Clock-trigger tasks (see :ref:`ClockTriggerTasks`) wait on a wall
+            Clock-triggered tasks (see :ref:`ClockTriggerTasks`) wait on a wall
             clock time specified as an offset from their own cycle point.
 
             Example:
@@ -739,25 +747,29 @@ with Conf(
                 ``cylc ext-trigger`` command.
             ''')
             Conf('clock-expire', VDR.V_STRING_LIST, desc='''
-                Don't submit jobs if they are very late in wall clock time.
+                Don't submit jobs if they are too late in wall clock time.
 
                 Clock-expire tasks enter the ``expired`` state and skip job
                 submission if too far behind the wall clock when they become
                 ready to run.
 
-                The expiry time is specified as an offset from
-                wall-clock time; typically it should be negative - see
-                :ref:`ClockExpireTasks`.
-
-                .. note::
-                   The offset:
+                The expiry time is specified as an offset from the task's
+                cycle point. The offset:
 
                    * May be positive or negative
-                   * The offset may be omitted if it is zero.
+                   * May be omitted if it is zero
 
-                Example:
+                .. seealso::
 
-                ``PT1H`` - 1 hour
+                   :ref:`ClockExpireTasks`.
+
+                Examples:
+
+                ``foo(PT1H)`` - expire task ``foo`` if the current wall clock
+                time has reached 1 hour after the task's cycle point.
+
+                ``bar(-PT5M)`` - expire task ``bar`` if the current wall clock
+                time has reached 5 minutes *before* the task's cycle point.
             ''')
             Conf('sequential', VDR.V_STRING_LIST, desc='''
                 A list of tasks which automatically depend on their own
@@ -970,7 +982,7 @@ with Conf(
                 If no parents are listed default is ``root``.
             ''')
             Conf('script', VDR.V_STRING, desc=dedent('''
-                The main custom script run from the task job script.
+                The main custom script run from the job script.
 
                 It can be an external command or script, or inlined scripting.
 
@@ -980,7 +992,7 @@ with Conf(
                 this='script', example='my_script.sh'
             ))
             Conf('init-script', VDR.V_STRING, desc=dedent('''
-                Custom script run by the task job script before the task
+                Custom script run by the job script before the task
                 execution environment is configured.
 
                 By running before the task execution environment is configured,
@@ -992,7 +1004,7 @@ with Conf(
                 this should no longer be necessary.
             ''') + get_script_common_text(this='init-script'))
             Conf('env-script', VDR.V_STRING, desc=dedent('''
-                Custom script run by the task job script between the
+                Custom script run by the job script between the
                 cylc-defined environment (workflow and task identity, etc.) and
                 the user-defined task runtime environment.
 
@@ -1001,10 +1013,10 @@ with Conf(
                 It can be an external command or script, or inlined scripting.
             ''') + get_script_common_text(this='env-script'))
             Conf('err-script', VDR.V_STRING, desc=('''
-                Script run when a task job error is detected.
+                Script run when a job error is detected.
 
                 Custom script to be run at the end of the error trap,
-                which is triggered due to failure of a command in the task job
+                which is triggered due to failure of a command in the job
                 script or trappable job kill.
 
                 The output of this script will always
@@ -1029,7 +1041,7 @@ with Conf(
                 this='exit-script', example='rm -f "$TMP_FILES"'
             ))
             Conf('pre-script', VDR.V_STRING, desc=dedent('''
-                Custom script run by the task job script immediately
+                Custom script run by the job script immediately
                 before :cylc:conf:`[..]script`.
 
                 The pre-script can be an external command or script, or
@@ -1039,7 +1051,7 @@ with Conf(
                 example='echo "Hello from workflow ${CYLC_WORKFLOW_ID}!"'
             ))
             Conf('post-script', VDR.V_STRING, desc=dedent('''
-                Custom script run by the task job script immediately
+                Custom script run by the job script immediately
                 after :cylc:conf:`[..]script`.
 
                 The post-script can be an external
@@ -1049,7 +1061,7 @@ with Conf(
             Conf('work sub-directory', VDR.V_STRING, desc='''
                 The directory from which tasks are executed.
 
-                Task job scripts are executed from within *work directories*
+                Job scripts are executed from within *work directories*
                 created automatically under the workflow run directory. A task
                 can get its own work directory from ``$CYLC_TASK_WORK_DIR``
                 (or ``$PWD`` if it does not ``cd`` elsewhere at
@@ -1083,16 +1095,17 @@ with Conf(
                 )
             )
             Conf('execution retry delays', VDR.V_INTERVAL_LIST, None, desc=f'''
-                Cylc can automate resubmission of a failed task job.
+                Cylc can automate resubmission of failed jobs.
 
-                Execution retry delays are a list of ISO 8601
-                durations/intervals which tell Cylc how long to wait before
-                resubmitting a failed job.
+                Execution retry delays is a list of ISO 8601 durations which
+                tell Cylc how long to wait before the next try.
 
-                Each time Cylc resubmits a task job it will increment the
-                variable ``$CYLC_TASK_TRY_NUMBER`` in the task execution
-                environment. ``$CYLC_TASK_TRY_NUMBER`` allows you to vary task
-                behavior between submission attempts.
+                The job environment variable ``$CYLC_TASK_TRY_NUMBER``
+                increments with each automatic retry, allowing you to vary task
+                behaviour between retries.
+
+                Tasks only go to the ``failed`` state if job execution fails
+                with no retries left.
 
                 .. versionchanged:: 8.0.0
 
@@ -1101,13 +1114,13 @@ with Conf(
             ''')
             Conf('execution time limit', VDR.V_INTERVAL, desc=f'''
                 Set the execution (:term:`wallclock <wallclock time>`) time
-                limit of a task job.
+                limit of a job.
 
                 For ``background`` and ``at`` job runners Cylc runs the
                 job's script using the timeout command. For other job runners
                 Cylc will convert execution time limit to a :term:`directive`.
 
-                If a task job exceeds its execution time limit Cylc can
+                If a job exceeds its execution time limit Cylc can
                 poll the job multiple times. You can set polling
                 intervals using :cylc:conf:`global.cylc[platforms]
                 [<platform name>]execution time limit polling intervals`
@@ -1521,7 +1534,7 @@ with Conf(
 
                 The items in this section reflect
                 options and defaults of the ``cylc workflow-state`` command,
-                except that the target workflow name and the
+                except that the target workflow ID and the
                 ``--task``, ``--cycle``, and ``--status`` options are
                 taken from the graph notation.
 
@@ -1573,7 +1586,7 @@ with Conf(
                 The user defined task execution environment.
 
                 Variables defined here can refer to cylc workflow and task
-                identity variables, which are exported earlier in the task job
+                identity variables, which are exported earlier in the job
                 script. Variable assignment expressions can use cylc
                 utility commands because access to cylc is also configured
                 earlier in the script.
@@ -1585,13 +1598,13 @@ with Conf(
                 You can also specify job environment templates here for
                 :ref:`parameterized tasks <User Guide Param>`.
             '''):
-                Conf('<variable>', VDR.V_STRING, desc='''
+                Conf('<variable>', VDR.V_STRING, desc=r'''
                     A custom user defined variable for a task execution
                     environment.
 
                     The order of definition is preserved that each variable can
                     refer to previously defined
-                    variables. Values are passed through to the task job
+                    variables. Values are passed through to the job
                     script without evaluation or manipulation by Cylc
                     (with the exception of valid Python string templates
                     that match parameterized task names - see below), so any
@@ -1626,6 +1639,32 @@ with Conf(
                        MYITEM = %(item)s
                        MYFILE = /path/to/%(i)03d/%(item)s
 
+                    .. note::
+
+                       As with other Cylc configurations, leading or trailing
+                       whitespace will be stripped, so the following two
+                       examples are equivalent:
+
+                       .. list-table::
+                          :class: grid-table
+
+                          * - .. code-block:: cylc
+
+                                 [environment]
+                                     FOO = " a "
+                                     BAR = """
+                                       $(foo bar baz)
+                                   """
+                            - .. code-block:: cylc
+
+                                 [environment]
+                                     FOO = "a"
+                                     BAR = "$(foo bar baz)"
+
+                       If leading or trailing whitespace is required, consider
+                       using the ``\0`` escape character, or set the variable
+                       in :cylc:conf:`[..][..]env-script`.
+
                     .. versionchanged:: 7.8.7/7.9.2
 
                        Parameter environment templates (previously in
@@ -1646,10 +1685,25 @@ with Conf(
                 this section (:ref:`MessageTriggers`)
             '''):
                 Conf('<output>', VDR.V_STRING, desc='''
-                    Task output messages (:ref:`MessageTriggers`).
+                    Define custom task outputs (aka :ref:`MessageTriggers`).
 
-                    The item name is used to select the custom output
-                    message in graph trigger notation.
+                    :term:`Custom outputs <custom output>` allow you to extend
+                    the built-in task outputs e.g. ``succeeded`` and ``failed``
+                    in order to provide more detailed information about task
+                    state. Custom outputs can be used to express dependencies
+                    in the graph as with built-in outputs.
+
+                    Custom outputs are defined in the form:
+
+                    .. code-block:: cylc
+
+                       output = message
+
+                    Where ``output`` is the name of the output as it is used in
+                    the graph, and ``message`` is the task message sent by
+                    the ``cylc message`` command which tells Cylc that this
+                    output has been completed. See :ref:`MessageTriggers` for
+                    more details.
 
                     Examples:
 
@@ -1658,10 +1712,15 @@ with Conf(
                        out1 = "sea state products ready"
                        out2 = "NWP restart files completed"
 
-                    Task outputs are validated by
-                    :py:class:`cylc.flow.unicode_rules.TaskOutputValidator`.
+                    Custom outputs must satisfy these rules:
 
                     .. autoclass:: cylc.flow.unicode_rules.TaskOutputValidator
+                       :noindex:
+
+                    Task messages must satisfy these rules:
+
+                    .. autoclass:: cylc.flow.unicode_rules.TaskMessageValidator
+                       :noindex:
                 ''')
 
             with Conf('parameter environment templates', desc='''
@@ -1789,7 +1848,10 @@ def upg(cfg, descr):
         '8.0.0',
         ['scheduling', 'max active cycle points'],
         ['scheduling', 'runahead limit'],
-        cvtr=converter(lambda x: f'P{x}' if x != '' else '', '"n" -> "Pn"'),
+        cvtr=converter(
+            lambda x: f'P{int(x)-1}' if x != '' else '',
+            '"{old}" -> "{new}"'
+        ),
         silent=cylc.flow.flags.cylc7_back_compat,
     )
     u.deprecate(
