@@ -36,7 +36,7 @@ from textwrap import wrap
 import traceback
 from typing import (
     Any, Callable, Dict, List, Mapping, Optional, Set, TYPE_CHECKING, Tuple,
-    Union
+    Union, Iterable
 )
 
 from metomi.isodatetime.data import Calendar
@@ -59,11 +59,11 @@ from cylc.flow.cycling.integer import IntegerInterval
 from cylc.flow.cycling.iso8601 import ingest_time, ISO8601Interval
 from cylc.flow.exceptions import (
     CylcError,
-    WorkflowConfigError,
+    InputError,
     IntervalParsingError,
-    TaskDefError,
     ParamExpandError,
-    InputError
+    TaskDefError,
+    WorkflowConfigError,
 )
 import cylc.flow.flags
 from cylc.flow.graph_parser import GraphParser
@@ -75,11 +75,9 @@ from cylc.flow.parsec.exceptions import ItemNotFoundError
 from cylc.flow.parsec.OrderedDict import OrderedDictWithDefaults
 from cylc.flow.parsec.util import replicate
 from cylc.flow.pathutil import (
-    get_workflow_run_dir,
-    get_workflow_run_scheduler_log_dir,
-    get_workflow_run_share_dir,
-    get_workflow_run_work_dir,
-    get_workflow_name_from_id
+    get_workflow_name_from_id,
+    get_cylc_run_dir,
+    is_relative_to,
 )
 from cylc.flow.platforms import FORBIDDEN_WITH_PLATFORM
 from cylc.flow.print_tree import print_tree
@@ -98,6 +96,7 @@ from cylc.flow.taskdef import TaskDef
 from cylc.flow.unicode_rules import (
     TaskNameValidator,
     TaskOutputValidator,
+    TaskMessageValidator,
     XtriggerNameValidator,
 )
 from cylc.flow.wallclock import (
@@ -113,19 +112,45 @@ if TYPE_CHECKING:
     from optparse import Values
     from cylc.flow.cycling import IntervalBase, PointBase, SequenceBase
 
+RE_CLOCK_OFFSET = re.compile(
+    rf'''
+        ^
+        \s*
+        ({TaskID.NAME_RE})   # task name
+        (?:\(\s*(.+)\s*\))?  # optional (arguments, ...)
+        \s*
+        $
+    ''',
+    re.X,
+)
 
-RE_CLOCK_OFFSET = re.compile(r'(' + TaskID.NAME_RE + r')(?:\(\s*(.+)\s*\))?')
-RE_EXT_TRIGGER = re.compile(r'(.*)\s*\(\s*(.+)\s*\)\s*')
+RE_EXT_TRIGGER = re.compile(
+    r'''
+        ^
+        \s*
+        (.*)            # task name
+        \s*
+        \(\s*(.+)\s*\)  # required (arguments, ...)
+        \s*
+        $
+    ''',
+    re.X,
+)
 RE_SEC_MULTI_SEQ = re.compile(r'(?![^(]+\)),')
 RE_WORKFLOW_ID_VAR = re.compile(r'\${?CYLC_WORKFLOW_(REG_)?ID}?')
 RE_TASK_NAME_VAR = re.compile(r'\${?CYLC_TASK_NAME}?')
 RE_VARNAME = re.compile(r'^[a-zA-Z_][\w]*$')
 
 
-def check_varnames(env):
+def check_varnames(env: Iterable[str]) -> List[str]:
     """Check a list of env var names for legality.
 
     Return a list of bad names (empty implies success).
+
+    Examples:
+        >>> check_varnames(['foo', 'BAR', '+baz'])
+        ['+baz']
+
     """
     bad = []
     for varname in env:
@@ -142,6 +167,23 @@ def interpolate_template(tmpl, params_dict):
 
     If it fails, raises ParamExpandError, but if the string does not contain
     `%(`, it just returns the string.
+
+    Examples:
+        >>> interpolate_template('_%(a)s_', {'a': 'A'})
+        '_A_'
+
+        >>> interpolate_template('%(a)s', {'b': 'B'})
+        Traceback (most recent call last):
+        cylc.flow.exceptions.ParamExpandError: bad parameter
+
+        >>> interpolate_template('%(a)d', {'a': 'A'})
+        Traceback (most recent call last):
+        cylc.flow.exceptions.ParamExpandError: wrong data type for parameter
+
+        >>> interpolate_template('%(as', {})
+        Traceback (most recent call last):
+        cylc.flow.exceptions.ParamExpandError: bad template syntax
+
     """
     if '%(' not in tmpl:
         return tmpl  # User probably not trying to use param template
@@ -155,7 +197,27 @@ def interpolate_template(tmpl, params_dict):
         raise ParamExpandError('bad template syntax')
 
 
-# TODO: separate config for run and non-run purposes?
+def dequote(string):
+    """Strip quotes off a string.
+
+    Examples:
+        >>> dequote('"foo"')
+        'foo'
+        >>> dequote("'foo'")
+        'foo'
+        >>> dequote('foo')
+        'foo'
+        >>> dequote('"f')
+        '"f'
+        >>> dequote('f')
+        'f'
+
+    """
+    if len(string) < 2:
+        return string
+    if (string[0] == string[-1]) and string.startswith(("'", '"')):
+        return string[1:-1]
+    return string
 
 
 class WorkflowConfig:
@@ -163,6 +225,7 @@ class WorkflowConfig:
 
     CHECK_CIRCULAR_LIMIT = 100  # If no. tasks > this, don't check circular
     VIS_N_POINTS = 3
+    MAX_WARNING_LINES = 5
 
     def __init__(
         self,
@@ -178,27 +241,33 @@ class WorkflowConfig:
         work_dir: Optional[str] = None,
         share_dir: Optional[str] = None
     ) -> None:
+        """
+        Initialize the workflow config object.
+
+        Args:
+            workflow: workflow ID
+            fpath: workflow config file path
+            options: CLI options
+        """
         check_deprecation(Path(fpath))
         self.mem_log = mem_log_func
         if self.mem_log is None:
             self.mem_log = lambda x: None
         self.mem_log("config.py:config.py: start init config")
-        self.workflow = workflow  # workflow id
+        self.workflow = workflow
         self.workflow_name = get_workflow_name_from_id(self.workflow)
-        self.fpath = str(fpath)  # workflow definition
-        self.fdir = os.path.dirname(fpath)
-        self.run_dir = run_dir or get_workflow_run_dir(self.workflow)
-        self.log_dir = (log_dir or
-                        get_workflow_run_scheduler_log_dir(self.workflow))
-        self.share_dir = share_dir or get_workflow_run_share_dir(self.workflow)
-        self.work_dir = work_dir or get_workflow_run_work_dir(self.workflow)
+        self.fpath: Path = Path(fpath)
+        self.fdir = str(self.fpath.parent)
+        self.run_dir = run_dir
+        self.log_dir = log_dir
+        self.share_dir = share_dir
+        self.work_dir = work_dir
         self.options = options
         self.implicit_tasks: Set[str] = set()
         self.edges: Dict[
             'SequenceBase', Set[Tuple[str, str, bool, bool]]
         ] = {}
         self.taskdefs: Dict[str, TaskDef] = {}
-        self.clock_offsets = {}
         self.expiration_offsets = {}
         self.ext_triggers = {}  # Old external triggers (client/server)
         self.xtrigger_mgr = xtrigger_mgr
@@ -223,10 +292,10 @@ class WorkflowConfig:
             'linearized ancestors': {},
             # lists of first-parent ancestor namespaces
             'first-parent ancestors': {},
-            # lists of all descendant namespaces
+            # sets of all descendant namespaces
             # (not including the final tasks)
             'descendants': {},
-            # lists of all descendant namespaces from the first-parent
+            # sets of all descendant namespaces from the first-parent
             # hierarchy (first parents are collapsible in visualization)
             'first-parent descendants': {},
         }
@@ -294,13 +363,6 @@ class WorkflowConfig:
         except KeyError:
             parameter_templates = {}
 
-        # Check that parameter templates are a section
-        if not hasattr(parameter_templates, 'update'):
-            raise WorkflowConfigError(
-                '[task parameters][templates] is a section. Don\'t use it '
-                'as a parameter.'
-            )
-
         # parameter values and templates are normally needed together.
         self.parameters = (parameter_values, parameter_templates)
 
@@ -439,18 +501,14 @@ class WorkflowConfig:
                             # (sub-family)
                             continue
                         result.append(member + extn)
-                        if s_type == 'clock-trigger':
-                            self.clock_offsets[member] = offset_interval
                         if s_type == 'clock-expire':
                             self.expiration_offsets[member] = offset_interval
                         if s_type == 'external-trigger':
                             self.ext_triggers[member] = ext_trigger_msg
-                elif s_type == 'clock-trigger':
-                    self.clock_offsets[name] = offset_interval
                 elif s_type == 'clock-expire':
                     self.expiration_offsets[name] = offset_interval
                 elif s_type == 'external-trigger':
-                    self.ext_triggers[name] = self.dequote(ext_trigger_msg)
+                    self.ext_triggers[name] = dequote(ext_trigger_msg)
 
             self.cfg['scheduling']['special tasks'][s_type] = result
 
@@ -471,6 +529,9 @@ class WorkflowConfig:
         self._check_special_tasks()  # adds to self.implicit_tasks
         self._check_explicit_cycling()
 
+        self._warn_if_queues_have_implicit_tasks(
+            self.cfg, self.taskdefs, self.MAX_WARNING_LINES)
+
         self._check_implicit_tasks()
         self._check_sequence_bounds()
         self.validate_namespace_names()
@@ -490,6 +551,8 @@ class WorkflowConfig:
                     raise WorkflowConfigError(
                         "external triggers must be used only once.")
 
+        self.upgrade_clock_triggers()
+
         self.leaves = self.get_task_name_list()
         for ancestors in self.runtime['first-parent ancestors'].values():
             try:
@@ -510,6 +573,38 @@ class WorkflowConfig:
 
         self.mem_log("config.py: end init config")
 
+    @staticmethod
+    def _warn_if_queues_have_implicit_tasks(
+        config, taskdefs, max_warning_lines
+    ):
+        """Warn if queues contain implict tasks.
+        """
+        implicit_q_msg = ''
+
+        # Get the names of the first N implicit queue tasks:
+        for queue in config["scheduling"]["queues"]:
+            for name in config["scheduling"]["queues"][queue][
+                "members"
+            ]:
+                if (
+                    name not in taskdefs
+                    and name not in config['runtime']
+                    and len(implicit_q_msg.split('\n')) <= max_warning_lines
+                ):
+                    implicit_q_msg += f'\n * task {name!r} in queue {queue!r}'
+
+        # Warn users if tasks are implied by queues.
+        if implicit_q_msg:
+            truncation_msg = (
+                f"\n...showing first {max_warning_lines} tasks..."
+                if len(implicit_q_msg.split('\n')) > max_warning_lines
+                else ""
+            )
+            LOG.warning(
+                'Queues contain tasks not defined in'
+                f' runtime: {implicit_q_msg}{truncation_msg}'
+            )
+
     def prelim_process_graph(self) -> None:
         """Ensure graph is not empty; set integer cycling mode and icp/fcp = 1
         for simplest "R1 = foo" type graphs.
@@ -803,7 +898,7 @@ class WorkflowConfig:
             )
         # Allow implicit tasks in back-compat mode unless rose-suite.conf
         # present (to maintain compat with Rose 2019)
-        elif not Path(self.run_dir, 'rose-suite.conf').is_file():
+        elif not (self.fpath.parent / "rose-suite.conf").is_file():
             LOG.debug(msg)
             return
 
@@ -988,13 +1083,6 @@ class WorkflowConfig:
                     f'task/family name {message}\n[runtime][[{name}]]'
                 )
 
-    @staticmethod
-    def dequote(s):
-        """Strip quotes off a string."""
-        if (s[0] == s[-1]) and s.startswith(("'", '"')):
-            return s[1:-1]
-        return s
-
     def check_env_names(self):
         """Check for illegal environment variable names"""
         bad = {}
@@ -1005,14 +1093,15 @@ class WorkflowConfig:
                     if res:
                         bad[(label, key)] = res
         if bad:
-            err_msg = "bad env variable names:"
-            for (label, key), names in bad.items():
-                err_msg += '\nNamespace:\t%s [%s]' % (label, key)
-                for name in names:
-                    err_msg += "\n\t\t%s" % name
-            LOG.error(err_msg)
             raise WorkflowConfigError(
-                "Illegal environment variable name(s) detected")
+                "Illegal environment variable name(s) detected:\n* "
+                # f"\n{err_msg}"
+                + '\n* '.join(
+                    f'[runtime][{label}][{key}]{name}'
+                    for (label, key), names in bad.items()
+                    for name in names
+                )
+            )
 
     def check_param_env_tmpls(self):
         """Check for illegal parameter environment templates"""
@@ -1124,16 +1213,12 @@ class WorkflowConfig:
         for name in self.cfg['runtime']:
             ancestors = self.runtime['linearized ancestors'][name]
             for p in ancestors[1:]:
-                if p not in self.runtime['descendants']:
-                    self.runtime['descendants'][p] = []
-                if name not in self.runtime['descendants'][p]:
-                    self.runtime['descendants'][p].append(name)
+                self.runtime['descendants'].setdefault(p, set()).add(name)
             first_ancestors = self.runtime['first-parent ancestors'][name]
             for p in first_ancestors[1:]:
-                if p not in self.runtime['first-parent descendants']:
-                    self.runtime['first-parent descendants'][p] = []
-                if name not in self.runtime['first-parent descendants'][p]:
-                    self.runtime['first-parent descendants'][p].append(name)
+                self.runtime['first-parent descendants'].setdefault(
+                    p, set()
+                ).add(name)
 
     def compute_inheritance(self):
         LOG.debug("Parsing the runtime namespace hierarchy")
@@ -1414,19 +1499,41 @@ class WorkflowConfig:
         print_tree(tree, padding=padding, use_unicode=pretty)
 
     def process_workflow_env(self):
-        """Workflow context is exported to the local environment."""
+        """Export Workflow context to the local environment.
+
+        A source workflow has only a name.
+        Once installed it also has an ID and a run directory.
+        And at scheduler start-up it has work, share, and log sub-dirs too.
+        """
         for key, value in {
             **verbosity_to_env(cylc.flow.flags.verbosity),
-            'CYLC_WORKFLOW_ID': self.workflow,
             'CYLC_WORKFLOW_NAME': self.workflow_name,
             'CYLC_WORKFLOW_NAME_BASE': str(Path(self.workflow_name).name),
-            'CYLC_WORKFLOW_RUN_DIR': self.run_dir,
-            'CYLC_WORKFLOW_LOG_DIR': self.log_dir,
-            'CYLC_WORKFLOW_WORK_DIR': self.work_dir,
-            'CYLC_WORKFLOW_SHARE_DIR': self.share_dir,
         }.items():
             os.environ[key] = value
 
+        if is_relative_to(self.fdir, get_cylc_run_dir()):
+            # This is an installed workflow.
+            #  - self.run_dir is only defined by the scheduler
+            #  - but the run dir exists, created at installation
+            #  - run sub-dirs may exist, if this installation was run already
+            #    but if the scheduler is not running they shouldn't be used.
+            for key, value in {
+                'CYLC_WORKFLOW_ID': self.workflow,
+                'CYLC_WORKFLOW_RUN_DIR': str(self.fdir),
+            }.items():
+                os.environ[key] = value
+
+        if self.run_dir is not None:
+            # Run directory is only defined if the scheduler is running; in
+            # which case the following run sub-directories must exist.
+            for key, value in {
+                'CYLC_WORKFLOW_LOG_DIR': str(self.log_dir),
+                'CYLC_WORKFLOW_WORK_DIR': str(self.work_dir),
+                'CYLC_WORKFLOW_SHARE_DIR': str(self.share_dir),
+            }.items():
+                os.environ[key] = value
+
     def process_config_env(self):
         """Set local config derived environment."""
         os.environ['CYLC_UTC'] = str(get_utc_mode())
@@ -1659,7 +1766,6 @@ class WorkflowConfig:
                 )
 
         for label in xtrig_labels:
-
             try:
                 xtrig = self.cfg['scheduling']['xtriggers'][label]
             except KeyError:
@@ -1953,7 +2059,7 @@ class WorkflowConfig:
         # the graph (which would probably be quite reasonable).
         family_map = {
             family: [
-                task for task in tasks
+                task for task in sorted(tasks)
                 if (
                     task in self.runtime['parents'] and
                     task not in self.runtime['descendants']
@@ -2096,36 +2202,38 @@ class WorkflowConfig:
                     continue
                 taskdef.set_required_output(output, not optional)
 
-    def find_taskdefs(self, name: str) -> List[TaskDef]:
+    def find_taskdefs(self, name: str) -> Set[TaskDef]:
         """Find TaskDef objects in family "name" or matching "name".
 
         Return a list of TaskDef objects which:
         * have names that glob matches "name".
         * are in a family that glob matches "name".
         """
-        ret: List[TaskDef] = []
         if name in self.taskdefs:
             # Match a task name
-            ret.append(self.taskdefs[name])
-        else:
-            fams = self.runtime['first-parent descendants']
+            return {self.taskdefs[name]}
+
+        fams = self.runtime['first-parent descendants']
+        if name in fams:
             # Match a family name
-            if name in fams:
-                for member in fams[name]:
-                    if member in self.taskdefs:
-                        ret.append(self.taskdefs[member])
-            else:
-                # Glob match task names
-                for key, taskdef in self.taskdefs.items():
-                    if fnmatchcase(key, name):
-                        ret.append(taskdef)
-                # Glob match family names
-                for key, members in fams.items():
-                    if fnmatchcase(key, name):
-                        for member in members:
-                            if member in self.taskdefs:
-                                ret.append(self.taskdefs[member])
-        return ret
+            return {
+                self.taskdefs[member] for member in fams[name]
+                if member in self.taskdefs
+            }
+
+        # Glob match
+        from_task_names = {
+            taskdef for key, taskdef in self.taskdefs.items()
+            if fnmatchcase(key, name)
+        }
+        from_family_names = {
+            self.taskdefs[member]
+            for key, members in fams.items()
+            if fnmatchcase(key, name)
+            for member in members
+            if member in self.taskdefs
+        }
+        return from_task_names.union(from_family_names)
 
     def get_taskdef(
         self, name: str, orig_expr: Optional[str] = None
@@ -2144,15 +2252,15 @@ class WorkflowConfig:
                           self.cfg['runtime']['root'])
                 if 'root' not in self.runtime['descendants']:
                     # (happens when no runtimes are defined in flow.cylc)
-                    self.runtime['descendants']['root'] = []
+                    self.runtime['descendants']['root'] = set()
                 if 'root' not in self.runtime['first-parent descendants']:
                     # (happens when no runtimes are defined in flow.cylc)
-                    self.runtime['first-parent descendants']['root'] = []
+                    self.runtime['first-parent descendants']['root'] = set()
                 self.runtime['parents'][name] = ['root']
                 self.runtime['linearized ancestors'][name] = [name, 'root']
                 self.runtime['first-parent ancestors'][name] = [name, 'root']
-                self.runtime['descendants']['root'].append(name)
-                self.runtime['first-parent descendants']['root'].append(name)
+                self.runtime['descendants']['root'].add(name)
+                self.runtime['first-parent descendants']['root'].add(name)
                 self.ns_defn_order.append(name)
 
             try:
@@ -2166,10 +2274,17 @@ class WorkflowConfig:
                 for output, message in (
                     self.cfg['runtime'][name]['outputs'].items()
                 ):
-                    valid, msg = TaskOutputValidator.validate(message)
+                    valid, msg = TaskOutputValidator.validate(output)
                     if not valid:
                         raise WorkflowConfigError(
-                            f'Invalid message trigger "'
+                            f'Invalid task output "'
+                            f'[runtime][{name}][outputs]'
+                            f'{output} = {message}" - {msg}'
+                        )
+                    valid, msg = TaskMessageValidator.validate(message)
+                    if not valid:
+                        raise WorkflowConfigError(
+                            f'Invalid task message "'
                             f'[runtime][{name}][outputs]'
                             f'{output} = {message}" - {msg}'
                         )
@@ -2195,8 +2310,6 @@ class WorkflowConfig:
 
         # TODO - put all taskd.foo items in a single config dict
 
-        if name in self.clock_offsets:
-            taskd.clocktrigger_offset = self.clock_offsets[name]
         if name in self.expiration_offsets:
             taskd.expiration_offset = self.expiration_offsets[name]
         if name in self.ext_triggers:
@@ -2347,3 +2460,34 @@ class WorkflowConfig:
             for task, _ in list(owners.items())[:5]:
                 msg += f'\n  * {task}"'
             raise WorkflowConfigError(msg)
+
+    def upgrade_clock_triggers(self):
+        """Convert old-style clock triggers to clock xtriggers.
+
+        [[special tasks]]
+           clock-trigger = foo(PT1D)
+
+        becomes:
+
+        [[xtriggers]]
+           _cylc_wall_clock_foo = wallclock(PT1D)
+
+        Not done by parsec upgrade because the graph has to be parsed first.
+        """
+        for item in self.cfg['scheduling']['special tasks']['clock-trigger']:
+            match = RE_CLOCK_OFFSET.match(item)
+            # (Already validated during "special tasks" parsing above.)
+            task_name, offset = match.groups()
+            # Derive an xtrigger label.
+            label = '_'.join(('_cylc', 'wall_clock', task_name))
+            # Define the xtrigger function.
+            xtrig = SubFuncContext(label, 'wall_clock', [], {})
+            xtrig.func_kwargs["offset"] = offset
+            if self.xtrigger_mgr is None:
+                XtriggerManager.validate_xtrigger(label, xtrig, self.fdir)
+            else:
+                self.xtrigger_mgr.add_trig(label, xtrig, self.fdir)
+            # Add it to the task, for each sequence that the task appears in.
+            taskdef = self.get_taskdef(task_name)
+            for seq in taskdef.sequences:
+                taskdef.add_xtrig_label(label, seq)
