@@ -26,6 +26,7 @@ from metomi.isodatetime.dumpers import TimePointDumper
 from metomi.isodatetime.timezone import (
     get_local_time_zone, get_local_time_zone_format, TimeZoneFormatMode)
 from metomi.isodatetime.exceptions import IsodatetimeError
+from metomi.isodatetime.parsers import ISO8601SyntaxError
 from cylc.flow.time_parser import CylcTimeParser
 from cylc.flow.cycling import (
     PointBase, IntervalBase, SequenceBase, ExclusionBase, cmp
@@ -34,7 +35,8 @@ from cylc.flow.exceptions import (
     CylcConfigError,
     IntervalParsingError,
     PointParsingError,
-    SequenceDegenerateError
+    SequenceDegenerateError,
+    WorkflowConfigError
 )
 from cylc.flow.wallclock import get_current_time_string
 from cylc.flow.parsec.validate import IllegalValueError
@@ -267,8 +269,14 @@ class ISO8601Exclusions(ExclusionBase):
         for point in excl_points:
             try:
                 # Try making an ISO8601Sequence
-                exclusion = ISO8601Sequence(point, self.exclusion_start_point,
-                                            self.exclusion_end_point)
+                exclusion = ISO8601Sequence(
+                    point,
+                    self.exclusion_start_point,
+                    self.exclusion_end_point,
+                    # disable warnings which are logged when exclusion is a
+                    # time point
+                    zero_duration_warning=False,
+                )
                 self.exclusion_sequences.append(exclusion)
             except (AttributeError, TypeError, ValueError):
                 # Try making an ISO8601Point
@@ -282,7 +290,20 @@ class ISO8601Sequence(SequenceBase):
 
     """A sequence of ISO8601 date time points separated by an interval.
     Note that an ISO8601Sequence object (may) contain
-    ISO8601ExclusionSequences"""
+    ISO8601ExclusionSequences
+
+    Args:
+        dep_section:
+            The full sequence expression.
+        context_start_point:
+            Sequence start point from the global context.
+        context_end_point:
+            Sequence end point from the global context.
+        zero_duration_warning:
+            If `False`, then zero-duration recurrence warnings will be turned
+            off. This is set for exclusion parsing.
+
+    """
 
     TYPE = CYCLER_TYPE_ISO8601
     TYPE_SORT_KEY = CYCLER_TYPE_SORT_KEY_ISO8601
@@ -301,8 +322,13 @@ class ISO8601Sequence(SequenceBase):
             return "R1"
         return "R1/" + str(start_point)
 
-    def __init__(self, dep_section, context_start_point=None,
-                 context_end_point=None):
+    def __init__(
+        self,
+        dep_section,
+        context_start_point=None,
+        context_end_point=None,
+        zero_duration_warning=True,
+    ):
         SequenceBase.__init__(
             self, dep_section, context_start_point, context_end_point)
         self.dep_section = dep_section
@@ -342,7 +368,9 @@ class ISO8601Sequence(SequenceBase):
         # Parse_recurrence returns an isodatetime TimeRecurrence object
         # and a list of exclusion strings.
         self.recurrence, excl_points = self.abbrev_util.parse_recurrence(
-            dep_section)
+            dep_section,
+            zero_duration_warning=zero_duration_warning,
+        )
 
         # Determine the exclusion start point and end point
         try:
@@ -542,7 +570,10 @@ class ISO8601Sequence(SequenceBase):
             return self.get_next_point_on_sequence(result)
         return result
 
-    def get_first_point(self, point):
+    def get_first_point(
+        self,
+        point: ISO8601Point
+    ) -> Optional[ISO8601Point]:
         """Return the first point >= to point, or None if out of bounds."""
         with contextlib.suppress(KeyError):
             return ISO8601Point(self._cached_first_point_values[point.value])
@@ -737,7 +768,18 @@ def prev_next(
     }
 
     for my_time in str_points:
-        parsed_point = parser.parse(my_time.strip())
+        try:
+            parsed_point = parser.parse(my_time.strip())
+        except ISO8601SyntaxError as exc:
+            raise exc
+        except ValueError:
+            # If list is accidentally comma
+            suggest = my_time.replace(',', ';')
+            raise WorkflowConfigError(
+                f'Invalid offset: {my_time}:'
+                f' Offset lists are semicolon separated, try {suggest}'
+            )
+
         timepoints.append(parsed_point + now)
 
         if direction == 'previous':
@@ -842,6 +884,7 @@ def init(num_expanded_year_digits=0, custom_dump_format=None, time_zone=None,
     WorkflowSpecifics.abbrev_util = CylcTimeParser(
         None, None, WorkflowSpecifics.iso8601_parsers
     )
+    return WorkflowSpecifics
 
 
 def get_dump_format():
