@@ -16,8 +16,10 @@
 """Exceptions for "expected" errors."""
 
 import errno
+from textwrap import wrap
 from typing import (
     Callable,
+    Dict,
     Iterable,
     NoReturn,
     Optional,
@@ -115,6 +117,13 @@ class WorkflowEventError(CylcError):
 
 class CommandFailedError(CylcError):
     """Exception for when scheduler commands fail."""
+    def __init__(self, value: Union[str, Exception]):
+        self.value = value
+
+    def __str__(self) -> str:
+        if isinstance(self.value, Exception):
+            return f"{type(self.value).__name__}: {self.value}"
+        return self.value
 
 
 class ServiceFileError(CylcError):
@@ -126,6 +135,10 @@ class WorkflowFilesError(CylcError):
     bullet = "\n    -"
 
 
+class ContactFileExists(CylcError):
+    """Workflow contact file exists."""
+
+
 def handle_rmtree_err(
     function: Callable,
     path: str,
@@ -185,7 +198,7 @@ class PlatformError(CylcError):
         message: str,
         platform_name: str,
         *,
-        ctx: SubFuncContext = None,
+        ctx: Optional[SubFuncContext] = None,
         cmd: Optional[Union[str, Iterable]] = None,
         ret_code: Optional[int] = None,
         out: Optional[str] = None,
@@ -254,9 +267,17 @@ class XtriggerConfigError(WorkflowConfigError):
 
 class ClientError(CylcError):
 
-    def __init__(self, message: str, traceback: Optional[str] = None):
+    def __init__(
+        self,
+        message: str,
+        traceback: Optional[str] = None,
+        workflow: Optional[str] = None
+    ):
         self.message = message
         self.traceback = traceback
+        # Workflow not included in string representation but useful bit of
+        # info to attach to the exception object
+        self.workflow = workflow
 
     def __str__(self) -> str:
         ret = self.message
@@ -277,7 +298,13 @@ class WorkflowStopped(ClientError):
 
 
 class ClientTimeout(CylcError):
-    pass
+
+    def __init__(self, message: str, workflow: Optional[str] = None):
+        self.message = message
+        self.workflow = workflow
+
+    def __str__(self) -> str:
+        return self.message
 
 
 class CyclingError(CylcError):
@@ -309,6 +336,10 @@ class IntervalParsingError(CyclingError):
             self, 'Incompatible value for {0}: {1}'.format(*args))
 
 
+class SequenceParsingError(CyclingError):
+    """Error on parsing an invalid sequence."""
+
+
 class SequenceDegenerateError(CyclingError):
     """An error raised when adjacent points on a sequence are equal."""
 
@@ -340,37 +371,98 @@ class PlatformLookupError(CylcConfigError):
 class HostSelectException(CylcError):
     """No hosts could be selected from the provided configuration."""
 
-    def __init__(self, data):
+    def __init__(self, data: Dict[str, dict]):
         self.data = data
-        CylcError.__init__(self)
 
-    def __str__(self):
+    def __str__(self) -> str:
         ret = 'Could not select host from:'
         for host, data in sorted(self.data.items()):
-            ret += f'\n    {host}:'
-            for key, value in data.items():
-                ret += f'\n        {key}: {value}'
+            if host != 'ranking':
+                ret += f'\n    {host}:'
+                for key, value in data.items():
+                    ret += f'\n        {key}: {value}'
+        hint = self.get_hint()
+        if hint:
+            ret += f'\n\n{hint}'
         return ret
 
+    def get_hint(self):
+        """Return a hint to explain this error for certain cases."""
+        if all(
+            # all procs came back with special SSH error code 255
+            datum.get('returncode') == 255
+            for key, datum in self.data.items()
+            if key != 'ranking'
+        ):
+            # likely SSH issues
+            return (
+                'Cylc could not establish SSH connection to the run hosts.'
+                '\nEnsure you can SSH to these hosts without having to'
+                ' answer any prompts.'
+            )
+
+        if (
+            # a ranking expression was used
+            self.data.get('ranking')
+            # and all procs came back with special 'cylc psutil' error code 2
+            # (which is used for errors relating to the extraction of metrics)
+            and all(
+                datum.get('returncode') == 2
+                for key, datum in self.data.items()
+                if key != 'ranking'
+            )
+        ):
+            # likely an issue with the ranking expression
+            ranking = "\n".join(
+                wrap(
+                    self.data.get("ranking"),
+                    initial_indent='    ',
+                    subsequent_indent='    ',
+                )
+            )
+            return (
+                'This is likely an error in the ranking expression:'
+                f'\n{ranking}'
+                '\n\nConfigured by:'
+                '\n    global.cylc[scheduler][run hosts]ranking'
+            )
+
+        return None
+
 
 class NoHostsError(CylcError):
     """None of the hosts of a given platform were reachable."""
     def __init__(self, platform):
         self.platform_name = platform['name']
-        super().__init__()
 
     def __str__(self):
         return f'Unable to find valid host for {self.platform_name}'
 
 
-class NoPlatformsError(CylcError):
-    """None of the platforms of a given group were reachable."""
-    def __init__(self, platform_group):
-        self.platform_group = platform_group
-        super().__init__()
+class NoPlatformsError(PlatformLookupError):
+    """None of the platforms of a given set were reachable.
+
+    Args:
+        identity: The name of the platform group or install target
+        set_type: Whether the set of platforms is a platform group or an
+            install target
+        place: Where the attempt to get the platform failed.
+    """
+    def __init__(
+        self, identity: str, set_type: str = 'group', place: str = ''
+    ):
+        self.identity = identity
+        self.type = set_type
+        if place:
+            self.place = f' during {place}.'
+        else:
+            self.place = '.'
 
     def __str__(self):
-        return f'Unable to find a platform from group {self.platform_group}.'
+        return (
+            f'Unable to find a platform from {self.type} {self.identity}'
+            f'{self.place}'
+        )
 
 
 class CylcVersionError(CylcError):
