@@ -16,8 +16,7 @@
 """Host name utilities
 
 ATTRIBUTION:
-https://web.archive.org/web/20140606052543/http://www.linux-support.com/cms/
-get-local-ip-address-with-python/
+https://web.archive.org/web/20140606052543/http://www.linux-support.com/cms/get-local-ip-address-with-python/
 
 Fetching the outgoing IP address of a computer might be a difficult
 task. Computers can contain a large set of network devices, each
@@ -150,7 +149,7 @@ class HostUtil:
 
         As specified by the "[scheduler][host self-identification]" settings in
         the site/user global.cylc files. This is mainly used for workflow host
-        identification by task jobs.
+        identification by tasks.
 
         """
         if self._host is None:
@@ -198,8 +197,8 @@ class HostUtil:
 
         """
         if name not in self._remote_hosts:
-            if not name or name.split(".")[0].startswith("localhost"):
-                # e.g. localhost.localdomain
+            if not name or name.startswith("localhost"):
+                # e.g. localhost4.localdomain4
                 self._remote_hosts[name] = False
             else:
                 try:
