@@ -25,6 +25,7 @@ from typing import (
     Iterable,
     List,
     Optional,
+    Set,
     Tuple,
     Union,
 )
@@ -80,16 +81,21 @@ class Tokens(dict):
         <id: ~u/w//c/t/02>
 
     """
-    # valid dictionary keys
-    _KEYS = {
-        # regular tokens
-        *{token.value for token in IDTokens},
-        # selector tokens
-        *{
-            f'{token.value}_sel'
-            for token in IDTokens
-            if token != IDTokens.User
-        },
+    _REGULAR_KEYS: Set[str] = {token.value for token in IDTokens}
+    _SELECTOR_KEYS = {
+        f'{token.value}_sel'
+        for token in IDTokens
+        if token != IDTokens.User
+    }
+
+    # all valid dictionary keys
+    _KEYS = _REGULAR_KEYS | _SELECTOR_KEYS
+
+    _TASK_LIKE_KEYS = {
+        key for key in _KEYS if not (
+            key.startswith(IDTokens.User.value)
+            or key.startswith(IDTokens.Workflow.value)
+        )
     }
 
     def __init__(
@@ -101,7 +107,10 @@ class Tokens(dict):
         if args:
             if len(args) > 1:
                 raise ValueError()
-            kwargs = tokenise(str(args[0]), relative)
+            if isinstance(args[0], str):
+                kwargs = tokenise(args[0], relative)
+            else:
+                kwargs = dict(args[0])
         else:
             for key in kwargs:
                 if key not in self._KEYS:
@@ -143,12 +152,16 @@ class Tokens(dict):
         return f'<id: {id_}>'
 
     def __eq__(self, other):
+        if not isinstance(other, self.__class__):
+            return False
         return all(
             self[key] == other[key]
             for key in self._KEYS
         )
 
     def __ne__(self, other):
+        if not isinstance(other, self.__class__):
+            return True
         return any(
             self[key] != other[key]
             for key in self._KEYS
@@ -182,6 +195,19 @@ class Tokens(dict):
         """
         return detokenise(self.task, relative=True)
 
+    @property
+    def relative_id_with_selectors(self) -> str:
+        """The relative ID (without the workflow part), with selectors.
+
+        Examples:
+            >>> Tokens('~u/w//c/t:failed/01').relative_id
+            'c/t/01'
+            >>> Tokens('~u/w//c/t:failed/01').relative_id_with_selectors
+            'c/t:failed/01'
+
+        """
+        return detokenise(self.task, relative=True, selectors=True)
+
     @property
     def workflow_id(self) -> str:
         """The workflow id (without the relative part).
@@ -193,7 +219,6 @@ class Tokens(dict):
             Traceback (most recent call last):
             ValueError: No tokens provided
 
-
         """
         return detokenise(self.workflow)
 
@@ -266,9 +291,7 @@ class Tokens(dict):
 
         """
         return any(
-            bool(self[token.value])
-            for token in IDTokens
-            if token not in {IDTokens.User, IDTokens.Workflow}
+            self[key] for key in self._TASK_LIKE_KEYS
         )
 
     @property
@@ -284,12 +307,7 @@ class Tokens(dict):
             **{
                 key: value
                 for key, value in self.items()
-                if key in (
-                    enum.value
-                    for enum in (
-                        {*IDTokens} - {IDTokens.User, IDTokens.Workflow}
-                    )
-                )
+                if key in self._TASK_LIKE_KEYS
             }
         )
 
@@ -306,12 +324,7 @@ class Tokens(dict):
             **{
                 key: value
                 for key, value in self.items()
-                if key not in (
-                    enum.value
-                    for enum in (
-                        {*IDTokens} - {IDTokens.User, IDTokens.Workflow}
-                    )
-                )
+                if key not in self._TASK_LIKE_KEYS
             }
         )
 
@@ -332,8 +345,7 @@ class Tokens(dict):
 
         """
         return not any(
-            bool(self[token.value])
-            for token in IDTokens
+            self[key] for key in self._REGULAR_KEYS
         )
 
     def update_tokens(
@@ -399,6 +411,10 @@ class Tokens(dict):
             Make a copy and modify it:
             >>> tokens1.duplicate(cycle='1').id
             '~u/w//1'
+
+            Original not changed
+            >>> tokens1.id
+            '~u/w'
         """
         ret = Tokens(self)
         ret.update_tokens(tokens, **kwargs)
@@ -529,6 +545,21 @@ LEGACY_CYCLE_SLASH_TASK = re.compile(
 )
 
 
+def quick_relative_detokenise(cycle, task):
+    """Generate a relative ID for a task.
+
+    This is a more efficient solution to `Tokens` for cases where
+    you only want the ID string and don't have any use for a Tokens object.
+
+    Example:
+        >>> q = quick_relative_detokenise
+        >>> q('1', 'a') == Tokens(cycle='1', task='a').relative_id
+        True
+
+    """
+    return f'{cycle}/{task}'
+
+
 def _dict_strip(dictionary):
     """Run str.strip against dictionary values.
 
@@ -730,44 +761,41 @@ def detokenise(
         ValueError: No tokens provided
 
     """
-    toks = {
-        token.value
-        for token in IDTokens
-        if tokens.get(token.value)
+    keys = {
+        key for key in Tokens._REGULAR_KEYS
+        if tokens.get(key)
     }
-    is_relative = not toks & {'user', 'workflow'}
-    is_partial = not toks & {'cycle', 'task', 'job'}
+    is_relative = keys.isdisjoint(('user', 'workflow'))
+    is_partial = keys.isdisjoint(('cycle', 'task', 'job'))
     if is_relative and is_partial:
         raise ValueError('No tokens provided')
 
     # determine the lowest token
     for lowest_token in reversed(IDTokens):
-        if lowest_token.value in toks:
+        if lowest_token.value in keys:
             break
 
-    highest_token: 'Optional[IDTokens]'
+    highest_token: Optional[IDTokens]
+    identifier = []
     if is_relative:
         highest_token = IDTokens.Cycle
-        identifier = []
         if not relative:
             identifier = ['/']
     else:
         highest_token = IDTokens.User
-        identifier = []
 
     for token in IDTokens:
-        if highest_token and token != highest_token:
-            continue
-        elif highest_token:
+        if highest_token:
+            if token != highest_token:
+                continue
             highest_token = None
-        value: 'Optional[str]'
-        value = tokens.get(token.value)
+        value: Optional[str] = tokens.get(token.value)
         if not value and token == IDTokens.User:
             continue
         elif token == IDTokens.User:
             value = f'~{value}'
         elif token == IDTokens.Job and value != 'NN':
-            value = f'{int(value):02}'  # type: ignore
+            value = f'{int(value):02}'  # type: ignore[arg-type]
         value = value or '*'
         if selectors and tokens.get(token.value + '_sel'):
             # include selectors
