@@ -62,15 +62,20 @@ Any uncommitted changes will also be saved as a diff in
 import json
 from pathlib import Path
 from subprocess import Popen, DEVNULL, PIPE
-from typing import Any, Dict, Iterable, List, Optional, TYPE_CHECKING, Union
+from typing import (
+    Any, Dict, Iterable, List, Optional, TYPE_CHECKING, TextIO, Union, overload
+)
 
-from cylc.flow import LOG
+from cylc.flow import LOG as _LOG, LoggerAdaptor
 from cylc.flow.exceptions import CylcError
+import cylc.flow.flags
 from cylc.flow.workflow_files import WorkflowFiles
 
 if TYPE_CHECKING:
     from optparse import Values
 
+LOG = LoggerAdaptor(_LOG, {'prefix': __name__})
+
 
 SVN = 'svn'
 GIT = 'git'
@@ -80,8 +85,6 @@ INFO_COMMANDS: Dict[str, List[str]] = {
     GIT: ['describe', '--always', '--dirty']
 }
 
-# git ['show', '--quiet', '--format=short'],
-
 STATUS_COMMANDS: Dict[str, List[str]] = {
     SVN: ['status', '--non-interactive'],
     GIT: ['status', '--short']
@@ -112,8 +115,6 @@ SVN_INFO_KEYS: List[str] = [
     'revision', 'url', 'working copy root path', 'repository uuid'
 ]
 
-
-LOG_VERSION_DIR = Path(WorkflowFiles.LOG_DIR, 'version')
 DIFF_FILENAME = 'uncommitted.diff'
 INFO_FILENAME = 'vcs.json'
 JSON_INDENT = 4
@@ -164,14 +165,14 @@ def get_vc_info(path: Union[Path, str]) -> Optional[Dict[str, Any]]:
             missing_base = True
             LOG.debug(exc)
         except OSError as exc:
-            if not any(
+            if any(
                 exc.strerror.lower().startswith(err)
                 for err in NOT_REPO_ERRS[vcs]
             ):
-                raise exc
-            else:
                 LOG.debug(f"Source dir {path} is not a {vcs} repository")
-                continue
+            elif cylc.flow.flags.verbosity > -1:
+                LOG.warning(f"$ {vcs} {' '.join(args)}\n{exc}")
+            continue
 
         info['version control system'] = vcs
         if vcs == SVN:
@@ -189,13 +190,40 @@ def get_vc_info(path: Union[Path, str]) -> Optional[Dict[str, Any]]:
     return None
 
 
-def _run_cmd(vcs: str, args: Iterable[str], cwd: Union[Path, str]) -> str:
-    """Run a VCS command, return stdout.
+@overload
+def _run_cmd(
+    vcs: str, args: Iterable[str], cwd: Union[Path, str], stdout: int = PIPE
+) -> str:
+    ...
+
+
+@overload
+def _run_cmd(
+    vcs: str, args: Iterable[str], cwd: Union[Path, str], stdout: TextIO
+) -> None:
+    ...
+
+
+def _run_cmd(
+    vcs: str,
+    args: Iterable[str],
+    cwd: Union[Path, str],
+    stdout: Union[TextIO, int] = PIPE
+) -> Optional[str]:
+    """Run a VCS command.
 
     Args:
         vcs: The version control system.
         args: The args to pass to the version control command.
         cwd: Directory to run the command in.
+        stdout: Where to redirect output (either PIPE or a
+            text stream/file object). Note: only use PIPE for
+            commands that will not generate a large output, otherwise
+            the pipe might get blocked.
+
+    Returns:
+        Stdout output if stdout=PIPE, else None as the output has been
+        written directly to the specified file.
 
     Raises:
         VCSNotInstalledError: The VCS is not found.
@@ -208,7 +236,7 @@ def _run_cmd(vcs: str, args: Iterable[str], cwd: Union[Path, str]) -> str:
             cmd,
             cwd=cwd,
             stdin=DEVNULL,
-            stdout=PIPE,
+            stdout=stdout,
             stderr=PIPE,
             text=True,
         )
@@ -238,7 +266,12 @@ def write_vc_info(
     """
     if not info:
         raise ValueError("Nothing to write")
-    info_file = Path(run_dir, LOG_VERSION_DIR, INFO_FILENAME)
+    info_file = Path(
+        run_dir,
+        WorkflowFiles.LogDir.DIRNAME,
+        WorkflowFiles.LogDir.VERSION,
+        INFO_FILENAME
+    )
     info_file.parent.mkdir(exist_ok=True, parents=True)
     with open(info_file, 'w') as f:
         f.write(
@@ -275,41 +308,45 @@ def _parse_svn_info(info_text: str) -> Dict[str, Any]:
     return ret
 
 
-def get_diff(vcs: str, path: Union[Path, str]) -> Optional[str]:
-    """Return the diff of uncommitted changes for a repository.
+def write_diff(
+    vcs: str, repo_path: Union[Path, str], run_dir: Union[Path, str]
+) -> Path:
+    """Get and write the diff of uncommitted changes for a repository to the
+    workflow's vcs log dir.
 
     Args:
         vcs: The version control system.
-        path: The path to the repo.
-    """
-    args_ = DIFF_COMMANDS[vcs]
-    if Path(path).is_absolute():
-        args_.append(str(path))
-    else:
-        args_.append(str(Path().cwd() / path))
-
-    try:
-        diff = _run_cmd(vcs, args_, cwd=path)
-    except (VCSNotInstalledError, VCSMissingBaseError):
-        return None
-    header = (
-        "# Auto-generated diff of uncommitted changes in the Cylc "
-        "workflow repository:\n"
-        f"#   {path}")
-    return f"{header}\n{diff}"
-
-
-def write_diff(diff: str, run_dir: Union[Path, str]) -> None:
-    """Write a diff to the workflow's vcs log dir.
-
-    Args:
-        diff: The diff.
+        repo_path: The path to the repo.
         run_dir: The workflow run directory.
+
+    Returns the path to diff file.
     """
-    diff_file = Path(run_dir, LOG_VERSION_DIR, DIFF_FILENAME)
+    args = DIFF_COMMANDS[vcs]
+    args.append(
+        str(repo_path) if Path(repo_path).is_absolute() else
+        str(Path().cwd() / repo_path)
+    )
+
+    diff_file = Path(
+        run_dir,
+        WorkflowFiles.LogDir.DIRNAME,
+        WorkflowFiles.LogDir.VERSION,
+        DIFF_FILENAME
+    )
     diff_file.parent.mkdir(exist_ok=True)
-    with open(diff_file, 'w') as f:
-        f.write(diff)
+
+    with open(diff_file, 'a') as f:
+        f.write(
+            "# Auto-generated diff of uncommitted changes in the Cylc "
+            "workflow repository:\n"
+            f"#   {repo_path}\n"
+        )
+        f.flush()
+        try:
+            _run_cmd(vcs, args, repo_path, stdout=f)
+        except VCSMissingBaseError as exc:
+            f.write(f"# No diff - {exc}")
+    return diff_file
 
 
 # Entry point:
@@ -331,8 +368,6 @@ def main(
     if vc_info is None:
         return False
     vcs = vc_info['version control system']
-    diff = get_diff(vcs, srcdir)
     write_vc_info(vc_info, rundir)
-    if diff is not None:
-        write_diff(diff, rundir)
+    write_diff(vcs, srcdir, rundir)
     return True
