@@ -13,7 +13,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""Write task job files."""
+"""Write job files."""
 
 from contextlib import suppress
 import os
@@ -28,10 +28,17 @@ import cylc.flow.flags
 from cylc.flow.option_parsers import verbosity_to_env
 from cylc.flow.config import interpolate_template, ParamExpandError
 
+# the maximum number of task dependencies which Cylc will list before
+# omitting the CYLC_TASK_DEPENDENCIES environment variable
+# see: https://github.com/cylc/cylc-flow/issues/5551
+# NOTE: please update `src/reference/job-script-vars/var-list.txt`
+#       in cylc-doc if changing this value
+MAX_CYLC_TASK_DEPENDENCIES_LEN = 50
+
 
 class JobFileWriter:
 
-    """Write task job files."""
+    """Write job files."""
 
     def __init__(self):
         self.workflow_env = {}
@@ -83,13 +90,14 @@ class JobFileWriter:
                     ['/usr/bin/env', 'bash', '-n', tmp_name],
                     stderr=PIPE,
                     stdin=DEVNULL,
+                    text=True
                     # * the purpose of this is to evaluate user defined code
                     #   prior to it being executed
                 ) as proc:
                     if proc.wait():
                         # This will leave behind the temporary file,
                         # which is useful for debugging syntax errors, etc.
-                        raise RuntimeError(proc.communicate()[1].decode())
+                        raise RuntimeError(proc.communicate()[1])
             except OSError as exc:
                 # Popen has a bad habit of not telling you anything if it fails
                 # to run the executable.
@@ -120,7 +128,7 @@ class JobFileWriter:
     def _write_header(handle, job_conf):
         """Write job script header."""
         handle.write("#!/bin/bash -l\n")
-        handle.write("#\n# ++++ THIS IS A CYLC TASK JOB SCRIPT ++++")
+        handle.write("#\n# ++++ THIS IS A CYLC JOB SCRIPT ++++")
         for prefix, value in [
                 ("# Workflow: ", job_conf['workflow_name']),
                 ("# Task: ", job_conf['task_id']),
@@ -219,9 +227,18 @@ class JobFileWriter:
         handle.write(
             '\n    export CYLC_TASK_NAMESPACE_HIERARCHY="%s"' %
             ' '.join(job_conf['namespace_hierarchy']))
-        handle.write(
-            '\n    export CYLC_TASK_DEPENDENCIES="%s"' %
-            ' '.join(job_conf['dependencies']))
+        if len(job_conf['dependencies']) <= MAX_CYLC_TASK_DEPENDENCIES_LEN:
+            handle.write(
+                '\n    export CYLC_TASK_DEPENDENCIES="%s"' %
+                ' '.join(job_conf['dependencies']))
+        else:
+            # redact the CYLC_TASK_DEPENDENCIES variable but leave a note
+            # explaining why
+            # see: https://github.com/cylc/cylc-flow/issues/5551
+            handle.write(
+                '\n    # CYLC_TASK_DEPENDENCIES=disabled'
+                f' (more than {MAX_CYLC_TASK_DEPENDENCIES_LEN} dependencies)'
+            )
         handle.write(
             '\n    export CYLC_TASK_TRY_NUMBER=%s' % job_conf['try_num'])
         handle.write(
