@@ -16,100 +16,10 @@
 
 """Manage submission, poll and kill of a job to the job runners.
 
-Export the JobRunnerManager class.
+The job runner interface is documented in
+cylc.flow.job_runner_handlers.documentation.
 
-Job runner handler (a.k.a. job submission method) modules should be placed
-under the "cylc.flow.job_runner_handlers" package. Each module should export
-the symbol "JOB_RUNNER_HANDLER" for the singleton instance that implements the
-job system handler logic.
-
-Each job runner handler class should instantiate with no argument, and may
-have the following constants and methods:
-
-job_runner.filter_poll_many_output(out) => job_ids
-    * Called after the job runner's poll many command. The method should read
-      the output and return a list of job IDs that are still in the
-      job runner.
-
-job_runner.filter_submit_output(out, err) => new_out, new_err
-    * Filter the standard output and standard error of the job submission
-      command. This is useful if the job submission command returns information
-      that should just be ignored. See also "job_runner.SUBMIT_CMD_TMPL".
-
-job_runner.format_directives(job_conf) => lines
-    * If relevant, this method formats the job directives for a job file, if
-      job file directives are relevant for the job runner. The argument
-      "job_conf" is a dict containing the job configuration.
-
-job_runner.get_poll_many_cmd(job-id-list) => list
-    * Return a list containing the shell command to poll the jobs in the
-      argument list.
-
-job_runner.get_submit_stdin(job_file_path: str, submit_opts: dict) => tuple
-    * Return a 2-element tuple `(proc_stdin_arg, proc_stdin_value)`.
-      Element 1 is suitable for the `stdin=...` argument of `subprocess.Popen`
-      so it can be a file handle, `subprocess.PIPE` or `None`. Element 2 is the
-      string content to pipe to STDIN of the submit command (relevant only if
-      `proc_stdin_arg` is `subprocess.PIPE`.
-
-job_runner.get_vacation_signal(job_conf) => str
-    * If relevant, return a string containing the name of the signal that
-      indicates the job has been vacated by the job runner.
-
-job_runner.submit(job_file_path, submit_opts) => ret_code, out, err
-    * Submit a job and return an instance of the Popen object for the
-      submission. This method is useful if the job submission requires logic
-      beyond just running a system or shell command. See also
-      "job_runner.SUBMIT_CMD". You must pass "env=submit_opts.get('env')" to
-      Popen - see background.py for example.
-
-job_runner.manip_job_id(job_id) => job_id
-    * Modify the job ID that is returned by the job submit command.
-
-job_runner.FAIL_SIGNALS => tuple<str>
-    * A tuple containing the names of signals to trap for reporting errors.
-      Default is ("EXIT", "ERR", "TERM", "XCPU"). ERR and EXIT are always
-      recommended.  EXIT is used to report premature stopping of the job
-      script, and its trap is unset at the end of the script.
-
-job_runner.KILL_CMD_TMPL
-    *  A Python string template for getting the job runner command to remove
-       and terminate a job ID. The command is formed using the logic:
-           job_runner.KILL_CMD_TMPL % {"job_id": job_id}
-
-job_runner.POLL_CANT_CONNECT_ERR
-    * A string containing an error message. If this is defined, when a poll
-      command returns a non-zero return code and its STDERR contains this
-      string, then the poll result will not be trusted, because it is assumed
-      that the job runner is currently unavailable. Jobs submitted to the
-      job runner will be assumed OK until we are able to connect to the
-      job runner again.
-
-job_runner.SHOULD_KILL_PROC_GROUP
-    * A boolean to indicate whether it is necessary to kill a job by sending
-      a signal to its Unix process group. This boolean also indicates that
-      a job submitted via this job runner will physically run on the same
-      host it is submitted to.
-
-job_runner.SHOULD_POLL_PROC_GROUP
-    * A boolean to indicate whether it is necessary to poll a job by its PID
-      as well as the job ID.
-
-job_runner.REC_ID_FROM_SUBMIT_ERR
-job_runner.REC_ID_FROM_SUBMIT_OUT
-    * A regular expression (compiled) to extract the job "id" from the standard
-      output or standard error of the job submission command.
-
-job_runner.SUBMIT_CMD_ENV
-    * A Python dict (or an iterable that can be used to update a dict)
-      containing extra environment variables for getting the job runner
-      command to submit a job file.
-
-job_runner.SUBMIT_CMD_TMPL
-    * A Python string template for getting the job runner command to submit a
-      job file. The command is formed using the logic:
-          job_runner.SUBMIT_CMD_TMPL % {"job": job_file_path}
-      See also "job_runner._job_submit_impl".
+Please update this file as the interface changes.
 
 """
 
@@ -282,7 +192,7 @@ class JobRunnerManager():
         """Kill multiple jobs.
 
         job_log_root -- The log/job/ sub-directory of the workflow.
-        job_log_dirs -- A list containing point/name/submit_num for task jobs.
+        job_log_dirs -- A list containing point/name/submit_num for jobs.
 
         """
         # Note: The more efficient way to do this is to group the jobs by their
@@ -311,7 +221,7 @@ class JobRunnerManager():
         """Poll multiple jobs.
 
         job_log_root -- The log/job/ sub-directory of the workflow.
-        job_log_dirs -- A list containing point/name/submit_num for task jobs.
+        job_log_dirs -- A list containing point/name/submit_num for jobs.
 
         """
         if "$" in job_log_root:
@@ -368,7 +278,7 @@ class JobRunnerManager():
         """Submit multiple jobs.
 
         job_log_root -- The log/job/ sub-directory of the workflow.
-        job_log_dirs -- A list containing point/name/submit_num for task jobs.
+        job_log_dirs -- A list containing point/name/submit_num for jobs.
         remote_mode -- am I running on the remote job host?
         utc_mode -- is the workflow running in UTC mode?
 
@@ -493,7 +403,8 @@ class JobRunnerManager():
                     rmtree(
                         os.path.join(task_log_dir, name), ignore_errors=True)
 
-    def _filter_submit_output(self, st_file_path, job_runner, out, err):
+    @classmethod
+    def _filter_submit_output(cls, st_file_path, job_runner, out, err):
         """Filter submit command output, if relevant."""
         job_id = None
         if hasattr(job_runner, "REC_ID_FROM_SUBMIT_ERR"):
@@ -511,9 +422,9 @@ class JobRunnerManager():
                         job_id = job_runner.manip_job_id(job_id)
                     with open(st_file_path, "a") as job_status_file:
                         job_status_file.write("{0}={1}\n".format(
-                            self.CYLC_JOB_ID, job_id))
+                            cls.CYLC_JOB_ID, job_id))
                         job_status_file.write("{0}={1}\n".format(
-                            self.CYLC_JOB_RUNNER_SUBMIT_TIME,
+                            cls.CYLC_JOB_RUNNER_SUBMIT_TIME,
                             get_current_time_string()))
                     break
         if hasattr(job_runner, "filter_submit_output"):
