@@ -28,6 +28,7 @@ from typing import (
     Dict,
     Iterable,
     List,
+    NamedTuple,
     Optional,
     Tuple,
     TYPE_CHECKING,
@@ -36,8 +37,8 @@ from typing import (
 from uuid import uuid4
 
 from graphene.utils.str_converters import to_snake_case
-from cylc.flow import LOG
 
+from cylc.flow import LOG
 from cylc.flow.data_store_mgr import (
     EDGES, FAMILY_PROXIES, TASK_PROXIES, WORKFLOW,
     DELTA_ADDED, create_delta_store
@@ -45,6 +46,7 @@ from cylc.flow.data_store_mgr import (
 from cylc.flow.id import Tokens
 from cylc.flow.network.schema import (
     DEF_TYPES,
+    RUNTIME_FIELD_TO_CFG_MAP,
     NodesEdges,
     PROXY_NODES,
     SUB_RESOLVERS,
@@ -59,6 +61,14 @@ if TYPE_CHECKING:
     from cylc.flow.workflow_status import StopMode
 
 
+class TaskMsg(NamedTuple):
+    """Tuple for Scheduler.message_queue"""
+    job_id: Union[Tokens, str]
+    event_time: str
+    severity: Union[str, int]
+    message: str
+
+
 logger = logging.getLogger(__name__)
 
 DELTA_SLEEP_INTERVAL = 0.5
@@ -257,6 +267,10 @@ def node_filter(node, node_type, args, state):
             args.get('maxdepth', -1) < 0
             or node.depth <= args['maxdepth']
         )
+        and (
+            args.get('graph_depth', -1) < 0
+            or node.graph_depth <= args['graph_depth']
+        )
         # Now filter node against id arg lists
         and (
             not args.get('ids')
@@ -375,7 +389,7 @@ class BaseResolvers(metaclass=ABCMeta):  # noqa: SIM119
 
     async def get_nodes_by_ids(self, node_type, args):
         """Return protobuf node objects for given id."""
-        nat_ids = set(args.get('native_ids', []))
+        nat_ids = uniq(args.get('native_ids', []))
         # Both cases just as common so 'if' not 'try'
         if 'sub_id' in args and args['delta_store']:
             flow_data = [
@@ -437,7 +451,7 @@ class BaseResolvers(metaclass=ABCMeta):  # noqa: SIM119
 
     async def get_edges_by_ids(self, args):
         """Return protobuf edge objects for given id."""
-        nat_ids = set(args.get('native_ids', []))
+        nat_ids = uniq(args.get('native_ids', []))
         if 'sub_id' in args and args['delta_store']:
             flow_data = [
                 delta[args['delta_type']]
@@ -460,6 +474,8 @@ class BaseResolvers(metaclass=ABCMeta):  # noqa: SIM119
         nodes = root_nodes
         node_ids = {n.id for n in root_nodes}
         edges = []
+        # NOTE: we don't expect edges to be returned in definition order
+        # so it is ok to use `set` here
         edge_ids = set()
         # Setup for edgewise search.
         new_nodes = root_nodes
@@ -482,6 +498,8 @@ class BaseResolvers(metaclass=ABCMeta):  # noqa: SIM119
             edges += new_edges
             # Gather nodes.
             # One of source or target will be in current set of nodes.
+            # NOTE: we don't expect node-edges to be returned in definition
+            # order so it is ok to use `set` here
             new_node_ids = set(
                 [e.source for e in new_edges]
                 + [e.target for e in new_edges]).difference(node_ids)
@@ -508,6 +526,8 @@ class BaseResolvers(metaclass=ABCMeta):  # noqa: SIM119
         yielded GraphQL subscription objects.
 
         """
+        # NOTE: we don't expect workflows to be returned in definition order
+        # so it is ok to use `set` here
         workflow_ids = set(args.get('workflows', args.get('ids', ())))
         sub_id = uuid4()
         info.variable_values['backend_sub_id'] = sub_id
@@ -532,10 +552,12 @@ class BaseResolvers(metaclass=ABCMeta):  # noqa: SIM119
             w_ids = workflow_ids
             sub_resolver = SUB_RESOLVERS.get(to_snake_case(info.field_name))
             interval = args['ignore_interval']
-            old_time = time()
+            old_time = 0.0
             while True:
                 if not workflow_ids:
                     old_ids = w_ids
+                    # NOTE: we don't expect workflows to be returned in
+                    # definition order so it is ok to use `set` here
                     w_ids = set(delta_queues.keys())
                     for remove_id in old_ids.difference(w_ids):
                         if remove_id in self.delta_store[sub_id]:
@@ -550,6 +572,9 @@ class BaseResolvers(metaclass=ABCMeta):  # noqa: SIM119
                                     workflow_id=w_id)
                                 delta_store[DELTA_ADDED] = (
                                     self.data_store_mgr.data[w_id])
+                                delta_store[DELTA_ADDED][
+                                    WORKFLOW
+                                ].reloaded = True
                                 deltas_queue.put(
                                     (w_id, 'initial_burst', delta_store))
                     elif w_id in self.delta_store[sub_id]:
@@ -664,23 +689,35 @@ class Resolvers(BaseResolvers):
             result = (True, 'Command queued')
         return [{'id': w_id, 'response': result}]
 
+    def _log_command(self, command: str, user: str) -> None:
+        """Log receipt of command, with user name if not owner."""
+        is_owner = user == self.schd.owner
+        if command == 'put_messages' and is_owner:
+            # Logging put_messages is overkill.
+            return
+        log_msg = f"[command] {command}"
+        if not is_owner:
+            log_msg += (f" (issued by {user})")
+        LOG.info(log_msg)
+
     async def _mutation_mapper(
         self, command: str, kwargs: Dict[str, Any], meta: Dict[str, Any]
     ) -> Optional[Tuple[bool, str]]:
         """Map between GraphQL resolvers and internal command interface."""
+
+        self._log_command(
+            command,
+            meta.get('auth_user', self.schd.owner)
+        )
         method = getattr(self, command, None)
         if method is not None:
             return method(**kwargs)
+
         try:
             self.schd.get_command_method(command)
         except AttributeError:
             raise ValueError(f"Command '{command}' not found")
-        if command != "put_messages":
-            log_msg = f"[command] {command}"
-            user = meta.get('auth_user', self.schd.owner)
-            if user != self.schd.owner:
-                log_msg += (f" (issued by {user})")
-            LOG.info(log_msg)
+
         self.schd.queue_command(
             command,
             kwargs
@@ -688,14 +725,22 @@ class Resolvers(BaseResolvers):
         return None
 
     def broadcast(
-            self,
-            mode,
-            cycle_points=None,
-            namespaces=None,
-            settings=None,
-            cutoff=None
+        self,
+        mode: str,
+        cycle_points: Optional[List[str]] = None,
+        namespaces: Optional[List[str]] = None,
+        settings: Optional[List[Dict[str, str]]] = None,
+        cutoff: Any = None
     ):
         """Put or clear broadcasts."""
+        if settings is not None:
+            # Convert schema field names to workflow config setting names if
+            # applicable:
+            for i, dict_ in enumerate(settings):
+                settings[i] = {
+                    RUNTIME_FIELD_TO_CFG_MAP.get(key, key): value
+                    for key, value in dict_.items()
+                }
         if mode == 'put_broadcast':
             return self.schd.task_events_mgr.broadcast_mgr.put_broadcast(
                 cycle_points, namespaces, settings)
@@ -732,35 +777,32 @@ class Resolvers(BaseResolvers):
 
     def put_messages(
         self,
-        task_job=None,
-        event_time=None,
-        messages=None
-    ):
+        task_job: str,
+        event_time: str,
+        messages: List[list]
+    ) -> Tuple[bool, str]:
         """Put task messages in queue for processing later by the main loop.
 
         Arguments:
-            task_job (str, optional):
-                Task job in the format ``CYCLE/TASK_NAME/SUBMIT_NUM``.
-            event_time (str, optional):
+            task_job:
+                Job ID in the format ``CYCLE/TASK_NAME/SUBMIT_NUM``.
+            event_time:
                 Event time as an ISO8601 string.
-            messages (list, optional):
+            messages:
                 List in the format ``[[severity, message], ...]``.
 
         Returns:
-            tuple: (outcome, message)
-
-            outcome (bool)
+            outcome:
                 True if command successfully queued.
-            message (str)
+            message:
                 Information about outcome.
 
         """
-        #  TODO: standardise the task_job interface to one of the other
-        #        systems
         for severity, message in messages:
             self.schd.message_queue.put(
-                (task_job, event_time, severity, message))
-        return (True, 'Messages queued: %d' % len(messages))
+                TaskMsg(task_job, event_time, severity, message)
+            )
+        return (True, f'Messages queued: {len(messages)}')
 
     def set_graph_window_extent(self, n_edge_distance):
         """Set data-store graph window to new max edge distance.
