@@ -19,7 +19,6 @@
 from copy import deepcopy
 from functools import partial
 import json
-import logging
 from operator import attrgetter
 from textwrap import dedent
 from typing import (
@@ -38,6 +37,7 @@ from graphene import (
 from graphene.types.generic import GenericScalar
 from graphene.utils.str_converters import to_snake_case
 
+from cylc.flow import LOG_LEVELS
 from cylc.flow.broadcast_mgr import ALL_CYCLE_POINTS_STRS, addict
 from cylc.flow.flow_mgr import FLOW_ALL, FLOW_NEW, FLOW_NONE
 from cylc.flow.id import Tokens
@@ -203,6 +203,7 @@ PROXY_ARGS = {
     'is_runahead': Boolean(),
     'mindepth': Int(default_value=-1),
     'maxdepth': Int(default_value=-1),
+    'graph_depth': Int(default_value=-1),
     'sort': SortArgs(default_value=None),
 }
 
@@ -218,6 +219,7 @@ ALL_PROXY_ARGS = {
     'is_runahead': Boolean(),
     'mindepth': Int(default_value=-1),
     'maxdepth': Int(default_value=-1),
+    'graph_depth': Int(default_value=-1),
     'sort': SortArgs(default_value=None),
 }
 
@@ -226,8 +228,6 @@ EDGE_ARGS = {
     'exids': graphene.List(ID, default_value=[]),
     'states': graphene.List(String, default_value=[]),
     'exstates': graphene.List(String, default_value=[]),
-    'mindepth': Int(default_value=-1),
-    'maxdepth': Int(default_value=-1),
     'sort': SortArgs(default_value=None),
 }
 
@@ -606,91 +606,172 @@ class Workflow(ObjectType):
     class Meta:
         description = """Global workflow info."""
     id = ID()  # noqa: A003 (required for definition)
-    name = String()
-    status = String()
-    status_msg = String()
-    host = String()
-    port = Int()
-    pub_port = Int()
-    owner = String()
+    name = String(
+        description='The workflow ID with the ~user/ prefix removed.',
+    )
+    status = String(
+        description='The workflow status e.g. `running`.',
+    )
+    status_msg = String(
+        description='A description of the workflow status.',
+    )
+    host = String(
+        description='The host where the scheduler process is running.',
+    )
+    port = Int(
+        description='The port for sending ZMQ requests to the scheduler.',
+    )
+    pub_port = Int(
+        description=sstrip('''
+            The port for subscribing to ZMQ updates from the scheduler.
+        '''),
+    )
+    owner = String(
+        description='The user account that the workflow is running under.',
+    )
     tasks = graphene.List(
         lambda: Task,
-        description="""Task definitions.""",
+        description="Task definitions.",
         args=DEF_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
-        resolver=get_nodes_by_ids)
+        resolver=get_nodes_by_ids,
+    )
     families = graphene.List(
         lambda: Family,
-        description="""Family definitions.""",
+        description="Family definitions.",
         args=DEF_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
-        resolver=get_nodes_by_ids)
+        resolver=get_nodes_by_ids,
+    )
     task_proxies = graphene.List(
         lambda: TaskProxy,
-        description="""Task cycle instances.""",
+        description="Task instances.",
         args=PROXY_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
-        resolver=get_nodes_by_ids)
+        resolver=get_nodes_by_ids,
+    )
     family_proxies = graphene.List(
         lambda: FamilyProxy,
-        description="""Family cycle instances.""",
+        description="Family instances.",
         args=PROXY_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
-        resolver=get_nodes_by_ids)
+        resolver=get_nodes_by_ids,
+    )
     jobs = graphene.List(
         lambda: Job,
-        description="""Task jobs.""",
+        description="Jobs.",
         args=JOB_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
-        resolver=get_nodes_by_ids)
+        resolver=get_nodes_by_ids,
+    )
     edges = Field(
         lambda: Edges,
         args=EDGE_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
-        description="""Graph edges""")
+        description="Graph edges.",
+    )
     nodes_edges = Field(
         lambda: NodesEdges,
         args=NODES_EDGES_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
-        resolver=get_nodes_edges)
-    api_version = Int()
-    cylc_version = String()
-    last_updated = Float()
-    meta = Field(NodeMeta)
-    newest_active_cycle_point = String()
-    oldest_active_cycle_point = String()
-    reloaded = Boolean()
-    run_mode = String()
-    is_held_total = Int()
-    is_queued_total = Int()
-    is_runahead_total = Int()
-    state_totals = GenericScalar(resolver=resolve_state_totals)
+        resolver=get_nodes_edges,
+        description='Graph nodes and edges.'
+    )
+    api_version = Int(
+        description='The Cylc scheduler communication protocol version number.'
+    )
+    cylc_version = String(
+        description='The Cylc version this workflow is running under.',
+    )
+    last_updated = Float(
+        description='The time of the most recent state change in the workflow.'
+    )
+    meta = Field(
+        NodeMeta,
+        description="The workflow's `[meta]` section.",
+    )
+    newest_active_cycle_point = String(
+        description='The newest cycle point which has active tasks.'
+    )
+    oldest_active_cycle_point = String(
+        description='The oldest cycle point which has active tasks.'
+    )
+    reloaded = Boolean(
+        description=sstrip('''
+            When subscribing to workflow updates, this field is `True` if the
+            update relates to a workflow reload.
+        '''),
+    )
+    run_mode = String(
+        description="The scheduler's run-mode e.g. `live`.",
+    )
+    is_held_total = Int(
+        description='The number of "held" tasks.',
+    )
+    is_queued_total = Int(
+        description='The number of queued tasks.',
+    )
+    is_runahead_total = Int(
+        description=sstrip('''
+            The number of tasks which are held back by the runahead limit.
+        ''')
+    )
+    state_totals = GenericScalar(
+        resolver=resolve_state_totals,
+        description='The number of tasks in each state as a JSON object.',
+    )
     latest_state_tasks = GenericScalar(
         states=graphene.List(
             String,
             description="List of task states to show",
             default_value=TASK_STATUSES_ORDERED),
-        resolver=resolve_state_tasks)
-    workflow_log_dir = String()
-    time_zone_info = Field(TimeZone)
-    tree_depth = Int()
-    ns_def_order = graphene.List(String)
-    job_log_names = graphene.List(String)
-    states = graphene.List(String)
+        resolver=resolve_state_tasks,
+        description='The latest tasks to have entered each task state.',
+    )
+    workflow_log_dir = String(
+        description="The path to the workflow's run directory.",
+    )
+    time_zone_info = Field(
+        TimeZone,
+        description='The scheduler time zone.',
+    )
+    tree_depth = Int()  # TODO: what is this? write description
+    ns_def_order = graphene.List(
+        String,
+        description=sstrip('''
+            Namespace definition order.
+
+            The order in which tasks were defined in the workflow
+            configuration.
+        '''),
+    )
+    job_log_names = graphene.List(
+        # TODO: remove, see https://github.com/cylc/cylc-flow/issues/5610
+        String,
+        description='Deprecated, do not use this.',
+    )
+    states = graphene.List(
+        String,
+        description=sstrip('''
+            The task states present in the workflow.
+
+            Similar to stateTotals.
+        '''),
+    )
     broadcasts = GenericScalar(
         ids=graphene.List(
             ID,
@@ -698,61 +779,168 @@ class Workflow(ObjectType):
                 Node IDs, cycle point and/or-just family/task namespace:
                     ["1234/foo", "1234/FAM", "*/FAM"]
             '''),
-            default_value=[]),
-        resolver=resolve_broadcasts)
-    pruned = Boolean()
+            default_value=[]
+        ),
+        resolver=resolve_broadcasts,
+        description='Any active workflow broadcasts.'
+    )
+    pruned = Boolean()  # TODO: what is this? write description
+    n_edge_distance = Int(
+        description=sstrip('''
+            The maximum graph distance (n) from an active node
+            of the data-store graph window.
+        '''),
+    )
+
+
+class RuntimeSetting(ObjectType):
+    """Key = value setting for a `[runtime][<namespace>]` configuration."""
+    key = String(default_value=None)
+    value = String(default_value=None)
+
+
+class Runtime(ObjectType):
+    class Meta:
+        description = sstrip("""
+            Subset of runtime fields.
+
+            Existing on 3 different node types:
+            - Task/family definition (from the flow.cylc file)
+            - Task/family cycle instance (includes any broadcasts on top
+              of the definition)
+            - Job (a record of what was run for a particular submit)
+        """)
+    platform = String(default_value=None)
+    script = String(default_value=None)
+    init_script = String(default_value=None)
+    env_script = String(default_value=None)
+    err_script = String(default_value=None)
+    exit_script = String(default_value=None)
+    pre_script = String(default_value=None)
+    post_script = String(default_value=None)
+    work_sub_dir = String(default_value=None)
+    execution_polling_intervals = String(default_value=None)
+    execution_retry_delays = String(default_value=None)
+    execution_time_limit = String(default_value=None)
+    submission_polling_intervals = String(default_value=None)
+    submission_retry_delays = String(default_value=None)
+    directives = graphene.List(RuntimeSetting, resolver=resolve_json_dump)
+    environment = graphene.List(RuntimeSetting, resolver=resolve_json_dump)
+    outputs = graphene.List(RuntimeSetting, resolver=resolve_json_dump)
+
+
+RUNTIME_FIELD_TO_CFG_MAP = {
+    **{
+        k: k.replace('_', ' ') for k in Runtime.__dict__
+        if not k.startswith('_')
+    },
+    'init_script': 'init-script',
+    'env_script': 'env-script',
+    'err_script': 'err-script',
+    'exit_script': 'exit-script',
+    'pre_script': 'pre-script',
+    'post_script': 'post-script',
+    'work_sub_dir': 'work sub-directory',
+}
+"""Map GQL Runtime fields' names to workflow config setting names."""
 
 
 class Job(ObjectType):
     class Meta:
-        description = """Jobs."""
+        description = "Jobs."
+
     id = ID()  # noqa: A003 (required for definition)
-    submit_num = Int()
-    state = String()
-    # name and cycle_point for filtering/sorting
-    name = String()
-    cycle_point = String()
+    submit_num = Int(
+        description='The submission number for this job, starts at 1.',
+    )
+    state = String(
+        description='The job state e.g. `running` or `succeeded`.',
+    )
+    name = String(
+        description='The name of the task which submitted this job.',
+    )
+    cycle_point = String(
+        description='The cycle of the task which submitted this job.',
+    )
     task_proxy = Field(
         lambda: TaskProxy,
-        description="""Associated Task Proxy""",
+        description="The TaskProxy of the task which submitted this job",
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
         resolver=get_node_by_id)
-    submitted_time = String()
-    started_time = String()
-    finished_time = String()
-    job_id = ID()
-    job_runner_name = String()
-    env_script = String()
-    err_script = String()
-    exit_script = String()
-    execution_time_limit = Float()
-    platform = String()
-    init_script = String()
-    job_log_dir = String()
-    post_script = String()
-    pre_script = String()
-    script = String()
-    work_sub_dir = String()
-    environment = GenericScalar(resolver=resolve_json_dump)
-    directives = GenericScalar(resolver=resolve_json_dump)
-    param_var = GenericScalar(resolver=resolve_json_dump)
-    extra_logs = graphene.List(String)
-    messages = graphene.List(String)
+    submitted_time = String(
+        description='The time this job was submitted to the job runner.',
+    )
+    started_time = String(
+        description='The time this job started running (if it has yet).',
+    )
+    finished_time = String(
+        description='The time this job finished running (if it has yet).',
+    )
+    job_id = ID(
+        description='The ID of this job in the job runner it was submitted to.'
+    )
+    job_runner_name = String(
+        description='The job runner this job was submitted to.',
+    )
+    execution_time_limit = Float(
+        description='The time limit for this job if configured.',
+    )
+    platform = String(
+        description='The Cylc platform this job was submitted to.',
+    )
+    job_log_dir = String(
+        description="The path to the job's log directory.",
+    )
+    extra_logs = graphene.List(
+        # TODO: remove. see https://github.com/cylc/cylc-flow/issues/5610
+        String,
+        description='Obsolete, do not use.',
+    )
+    messages = graphene.List(
+        String,
+        description='The list of task messages generated by this job.',
+    )
+    runtime = Field(
+        Runtime,
+        description=sstrip('''
+            The `[runtime]` configuration of the task which submitted this job.
+        '''),
+    )
 
 
 class Task(ObjectType):
     class Meta:
-        description = """Task definition, static fields"""
+        description = sstrip("""
+            Task definitions.
+
+            These are the task "definitions" as they appear in the
+            configuration as opposed to task "instances" which you will find
+            in the `TaskProxies` field.
+        """)
+
     id = ID()  # noqa: A003 (required for definition)
-    name = String()
-    meta = Field(NodeMeta)
-    mean_elapsed_time = Float()
-    depth = Int()
+    name = String(
+        description='The task name.'
+    )
+    meta = Field(
+        NodeMeta,
+        description="The task's `[meta]` section.",
+    )
+    runtime = Field(
+        Runtime,
+        description="The task's `[runtime`] section.",
+    )
+    mean_elapsed_time = Float(
+        description="The task's average runtime."
+    )
+    depth = Int(
+        description='The family inheritance depth.'
+    )
     proxies = graphene.List(
         lambda: TaskProxy,
-        description="""Associated cycle point proxies""",
+        description="Associated cycle point proxies",
         args=PROXY_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
@@ -760,7 +948,7 @@ class Task(ObjectType):
         resolver=get_nodes_by_ids)
     parents = graphene.List(
         lambda: Family,
-        description="""Family definition parent.""",
+        description="Family definition parent.",
         args=DEF_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
@@ -769,7 +957,7 @@ class Task(ObjectType):
     namespace = graphene.List(String)
     first_parent = Field(
         lambda: Family,
-        description="""Task first parent.""",
+        description="Task first parent.",
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
@@ -826,14 +1014,6 @@ class Output(ObjectType):
     time = Float()
 
 
-class ClockTrigger(ObjectType):
-    class Meta:
-        description = """Task clock-trigger"""
-    time = Float()
-    time_string = String()
-    satisfied = Boolean()
-
-
 class XTrigger(ObjectType):
     class Meta:
         description = """Task trigger"""
@@ -850,23 +1030,65 @@ class TaskProxy(ObjectType):
     id = ID()  # noqa: A003 (required for schema definition)
     task = Field(
         Task,
-        description="""Task definition""",
+        description="The task definition relating to this task instance.",
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
         resolver=get_node_by_id)
-    state = String()
-    cycle_point = String()
-    is_held = Boolean()
-    is_queued = Boolean()
-    is_runahead = Boolean()
-    flow_nums = String()
-    flow_wait = Boolean()
-    depth = Int()
-    job_submits = Int()
+    runtime = Field(
+        Runtime,
+        description="This task's `[runtime]` section.",
+    )
+    state = String(
+        description='The task state e.g. `running`.',
+    )
+    name = String(
+        description="The task's name.",
+    )
+    cycle_point = String(
+        description="The task's cycle point.",
+    )
+    namespace = graphene.List(
+        String,
+        description='The inheritance order for this task.',
+    )
+    is_held = Boolean(
+        description='True if this task is "held".',
+    )
+    is_queued = Boolean(
+        description=sstrip('''
+            True if this task is "queued".
+
+            This relates to Cylc's internal task queues, not a job runner
+            queue.
+        '''),
+    )
+    is_runahead = Boolean(
+        description='True if this task is held back by the "runahead limit".',
+    )
+    flow_nums = String(
+        description='The flows this task instance belongs to.',
+    )
+    flow_wait = Boolean(
+        description=sstrip('''
+            True if this task will wait for an approaching flow before spawning
+            outputs downstream.
+        '''),
+    )
+    depth = Int(
+        description='The family inheritance depth',
+    )
+    graph_depth = Int(
+        description=sstrip('''
+            The n-window graph edge depth from closet active task(s).
+        '''),
+    )
+    job_submits = Int(
+        description='The number of job submissions for this task instance.',
+    )
     outputs = graphene.List(
         Output,
-        description="""Task outputs.""",
+        description="Outputs this task instance has generated.",
         sort=SortArgs(default_value=None),
         sort_order=graphene.List(
             String,
@@ -875,10 +1097,9 @@ class TaskProxy(ObjectType):
         limit=Int(default_value=0),
         satisfied=Boolean(),
         resolver=resolve_mapping_to_list)
-    clock_trigger = Field(ClockTrigger)
     external_triggers = graphene.List(
         XTrigger,
-        description="""Task external trigger prerequisites.""",
+        description="Task external trigger prerequisites.",
         sort=SortArgs(default_value=None),
         sort_order=graphene.List(String),
         limit=Int(default_value=0),
@@ -886,49 +1107,56 @@ class TaskProxy(ObjectType):
         resolver=resolve_mapping_to_list)
     xtriggers = graphene.List(
         XTrigger,
-        description="""Task xtrigger prerequisites.""",
+        description="Task xtrigger prerequisites.",
         sort=SortArgs(default_value=None),
         sort_order=graphene.List(String),
         limit=Int(default_value=0),
         satisfied=Boolean(),
         resolver=resolve_mapping_to_list)
-    extras = GenericScalar(resolver=resolve_json_dump)
-    # name & namespace for filtering/sorting
-    name = String()
-    namespace = graphene.List(String)
-    prerequisites = graphene.List(Prerequisite)
+    extras = GenericScalar(
+        # TODO: what is this? write description
+        resolver=resolve_json_dump,
+    )
+    prerequisites = graphene.List(
+        Prerequisite,
+        description='The prerequisites of this task.',
+    )
     jobs = graphene.List(
         Job,
-        description="""Task jobs.""",
+        description="Jobs.",
         args=JOB_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
-        resolver=get_nodes_by_ids)
+        resolver=get_nodes_by_ids,
+    )
     parents = graphene.List(
         lambda: FamilyProxy,
-        description="""Task parents.""",
+        description="Task parents (i.e. families).",
         args=PROXY_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
-        resolver=get_nodes_by_ids)
+        resolver=get_nodes_by_ids,
+    )
     first_parent = Field(
         lambda: FamilyProxy,
-        description="""Task first parent.""",
+        description="The task's first parent (i.e. its containing family).",
         args=PROXY_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
-        resolver=get_node_by_id)
+        resolver=get_node_by_id,
+    )
     ancestors = graphene.List(
         lambda: FamilyProxy,
-        description="""First parent ancestors.""",
+        description="First parent ancestors (i.e. inheritance hierarchy).",
         args=PROXY_ARGS,
         strip_null=STRIP_NULL_DEFAULT,
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
-        resolver=get_nodes_by_ids)
+        resolver=get_nodes_by_ids,
+    )
 
 
 class Family(ObjectType):
@@ -937,6 +1165,7 @@ class Family(ObjectType):
     id = ID()  # noqa: A003 (required for schema definition)
     name = String()
     meta = Field(NodeMeta)
+    runtime = Field(Runtime)
     depth = Int()
     proxies = graphene.List(
         lambda: FamilyProxy,
@@ -993,6 +1222,7 @@ class FamilyProxy(ObjectType):
         delta_store=DELTA_STORE_DEFAULT,
         delta_type=DELTA_TYPE_DEFAULT,
         resolver=get_node_by_id)
+    runtime = Field(Runtime)
     state = String()
     states = graphene.List(String)
     state_totals = GenericScalar(resolver=resolve_state_totals)
@@ -1003,6 +1233,12 @@ class FamilyProxy(ObjectType):
     is_runahead = Boolean()
     is_runahead_total = Int()
     depth = Int()
+    graph_depth = Int(
+        description=sstrip('''
+            The n-window graph edge smallest child task/family depth
+            from closet active task(s).
+        '''),
+    )
     child_tasks = graphene.List(
         TaskProxy,
         description="""Descendant task proxies.""",
@@ -1203,7 +1439,7 @@ async def mutator(
     root: Optional[Any],
     info: 'ResolveInfo',
     *,
-    command: str,
+    command: Optional[str] = None,
     workflows: Optional[List[str]] = None,
     exworkflows: Optional[List[str]] = None,
     **kwargs: Any
@@ -1218,11 +1454,13 @@ async def mutator(
         root: Parent field (if any) value object.
         info: GraphQL execution info.
         command: Mutation command name (name of method of
-            cylc.flow.network.resolvers.Resolvers or
-            Scheduler command_<name> method).
+            cylc.flow.network.resolvers.Resolvers or Scheduler command_<name>
+            method). If None, uses mutation class name converted to snake_case.
         workflows: List of workflow IDs.
         exworkflows: List of workflow IDs.
     """
+    if command is None:
+        command = to_snake_case(info.field_name)
     if workflows is None:
         workflows = []
     if exworkflows is None:
@@ -1378,9 +1616,11 @@ class TimePoint(String):
 
 LogLevels = graphene.Enum(
     'LogLevels',
-    list(logging._nameToLevel.items()),
-    description=lambda x: f'Python logging level: {x.name} = {x.value}.'
-    if x else ''
+    list(LOG_LEVELS.items()),
+    description=lambda x: (
+        f'Logging level: {x.name} = {x.value}.'
+        if x else ''
+    )
 )
 
 
@@ -1441,8 +1681,9 @@ class Broadcast(Mutation):
             Note: a "clear" broadcast for a specific cycle or namespace does
             *not* clear all-cycle or all-namespace broadcasts.
 
+            Valid for: paused, running workflows.
         ''')
-        resolver = partial(mutator, command='broadcast')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1500,8 +1741,10 @@ class SetHoldPoint(Mutation):
         description = sstrip('''
             Set workflow hold after cycle point. All tasks after this point
             will be held.
+
+            Valid for: paused, running workflows.
         ''')
-        resolver = partial(mutator, command='set_hold_point')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1517,10 +1760,11 @@ class Pause(Mutation):
     class Meta:
         description = sstrip('''
             Pause a workflow.
+            This suspends submission of tasks.
 
-            This prevents submission of any task jobs.
+            Valid for: running workflows.
         ''')
-        resolver = partial(mutator, command='pause')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1531,17 +1775,19 @@ class Pause(Mutation):
 class Message(Mutation):
     class Meta:
         description = sstrip('''
-            Record task job messages.
+            Record task messages.
 
-            Send task job messages to:
+            Send messages to:
             - The job stdout/stderr.
             - The job status file, if there is one.
             - The scheduler, if communication is possible.
 
-            Task jobs use this to record and report status such
-            as success and failure. Applications run by task jobs can use
+            Jobs use this to record and report status such
+            as success and failure. Applications run by jobs can use
             this command to report messages and to report registered task
             outputs.
+
+            Valid for: paused, running, stopping workflows.
         ''')
         resolver = partial(mutator, command='put_messages')
 
@@ -1564,8 +1810,10 @@ class ReleaseHoldPoint(Mutation):
             Release all tasks and unset the workflow hold point, if set.
 
             Held tasks do not submit their jobs even if ready to run.
+
+            Valid for: paused, running workflows.
         ''')
-        resolver = partial(mutator, command='release_hold_point')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1577,10 +1825,11 @@ class Resume(Mutation):
     class Meta:
         description = sstrip('''
             Resume a paused workflow.
-
             See also the opposite command `pause`.
+
+            Valid for: paused workflows.
         ''')
-        resolver = partial(mutator, command='resume')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1605,6 +1854,8 @@ class Reload(Mutation):
 
             If the modified workflow config does not parse, failure to reload
             will be reported but no harm will be done to the running workflow.
+
+            Valid for: paused, running workflows.
         ''')
         resolver = partial(mutator, command='reload_workflow')
 
@@ -1619,11 +1870,13 @@ class SetVerbosity(Mutation):
         description = sstrip('''
             Change the logging severity level of a running workflow.
 
-            Only messages at or above the chosen severity level will be logged;
-            for example, if you choose `WARNING`, only warnings and critical
-            messages will be logged.
+            Only messages at or above the chosen severity level will be logged.
+            For example, if you choose `WARNING`, only warning, error and
+            critical level messages will be logged.
+
+            Valid for: paused, running workflows.
         ''')
-        resolver = partial(mutator, command='set_verbosity')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1638,8 +1891,9 @@ class SetGraphWindowExtent(Mutation):
             Set the maximum graph distance (n) from an active node
             of the data-store graph window.
 
+            Valid for: paused, running workflows.
         ''')
-        resolver = partial(mutator, command='set_graph_window_extent')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1663,8 +1917,10 @@ class Stop(Mutation):
             Remaining task event handlers, job poll and kill commands, will
             be executed prior to shutdown, unless
             the stop mode is `{WorkflowStopMode.Now.name}`.
+
+            Valid for: paused, running, stopping workflows.
         ''')
-        resolver = partial(mutator, command='stop')
+        resolver = mutator
 
     class Arguments:
         workflows = graphene.List(WorkflowID, required=True)
@@ -1711,6 +1967,8 @@ class ExtTrigger(Mutation):
 
             Note: To manually trigger a task use "Trigger" not
             "ExtTrigger".
+
+            Valid for: paused, running workflows.
         ''')
         resolver = partial(mutator, command='put_ext_trigger')
 
@@ -1752,15 +2010,35 @@ class FlowMutationArguments:
             This should be a list of flow numbers OR a single-item list
             containing one of the following three strings:
 
-            Alternatively this may be a single-item list containing one of
-            the following values:
-
             * {FLOW_ALL} - Triggered tasks belong to all active flows
               (default).
             * {FLOW_NEW} - Triggered tasks are assigned to a new flow.
             * {FLOW_NONE} - Triggered tasks do not belong to any flow.
         ''')
     )
+    flow_wait = Boolean(
+        default_value=False,
+        description=sstrip('''
+            Should the workflow "wait" or "continue on" from this task?
+
+            If `false` the scheduler will spawn and run downstream tasks
+            as normal (default).
+
+            If `true` the scheduler will not spawn the downstream tasks
+            unless it has been caught by the same flow at a later time.
+
+            For example you might set this to True to trigger a task
+            ahead of a flow, where you don't want the scheduler to
+            "continue on" from this task until the flow has caught up
+            with it.
+        ''')
+    )
+    flow_descr = String(
+        description=sstrip('''
+            If starting a new flow, this field can be used to provide the
+            new flow with a description for later reference.
+        ''')
+    )
 
 
 class Hold(Mutation, TaskMutation):
@@ -1769,8 +2047,10 @@ class Hold(Mutation, TaskMutation):
             Hold tasks within a workflow.
 
             Held tasks do not submit their jobs even if ready to run.
+
+            Valid for: paused, running workflows.
         ''')
-        resolver = partial(mutator, command='hold')
+        resolver = mutator
 
 
 class Release(Mutation, TaskMutation):
@@ -1779,8 +2059,10 @@ class Release(Mutation, TaskMutation):
             Release held tasks within a workflow.
 
             See also the opposite command `hold`.
+
+            Valid for: paused, running workflows.
         ''')
-        resolver = partial(mutator, command='release')
+        resolver = mutator
 
 
 class Kill(Mutation, TaskMutation):
@@ -1788,6 +2070,8 @@ class Kill(Mutation, TaskMutation):
     class Meta:
         description = sstrip('''
             Kill running or submitted jobs.
+
+            Valid for: paused, running, stopping workflows.
         ''')
         resolver = partial(mutator, command='kill_tasks')
 
@@ -1795,7 +2079,7 @@ class Kill(Mutation, TaskMutation):
 class Poll(Mutation, TaskMutation):
     class Meta:
         description = sstrip('''
-            Poll (query) task jobs to verify and update their statuses.
+            Poll (query) jobs to verify and update their statuses.
 
             This checks the job status file and queries the
             job runner on the job platform.
@@ -1803,6 +2087,8 @@ class Poll(Mutation, TaskMutation):
             Pollable tasks are those in the n=0 window with
             an associated job ID, including incomplete finished
             tasks.
+
+            Valid for: paused, running, stopping workflows.
         ''')
         resolver = partial(mutator, command='poll_tasks')
 
@@ -1815,6 +2101,7 @@ class Remove(Mutation, TaskMutation):
         description = sstrip('''
             Remove one or more task instances from a running workflow.
 
+            Valid for: paused, running workflows.
         ''')
         resolver = partial(mutator, command='remove_tasks')
 
@@ -1828,6 +2115,8 @@ class SetOutputs(Mutation, TaskMutation):
             algorithm by artificially satisfying outputs of tasks.
 
             By default this makes tasks appear as if they succeeded.
+
+            Valid for: paused, running workflows.
         ''')
         resolver = partial(mutator, command='force_spawn_children')
 
@@ -1850,33 +2139,13 @@ class Trigger(Mutation, TaskMutation):
             tasks will submit immediately if triggered, even if that violates
             the queue limit (so you may need to trigger a queue-limited task
             twice to get it to submit immediately).
+
+            Valid for: paused, running workflows.
         ''')
         resolver = partial(mutator, command='force_trigger_tasks')
 
     class Arguments(TaskMutation.Arguments, FlowMutationArguments):
-        flow_wait = Boolean(
-            default_value=False,
-            description=sstrip('''
-                Should the workflow "wait" or "continue on" from this task?
-
-                If `false` the scheduler will spawn and run downstream tasks
-                as normal (default).
-
-                If `true` the scheduler will not spawn the downstream tasks
-                unless it has been caught by the same flow at a later time.
-
-                For example you might set this to True to trigger a task
-                ahead of a flow, where you don't want the scheduled to
-                "continue on" from this task until the flow has caught up
-                with it.
-            ''')
-        )
-        flow_descr = String(
-            description=sstrip('''
-                If starting a new flow, this field can be used to provide the
-                new flow with a description for later reference.
-            ''')
-        )
+        ...
 
 
 def _mut_field(cls):
@@ -1992,7 +2261,7 @@ class Delta(Interface):
     )
     jobs = graphene.List(
         Job,
-        description="""Task jobs.""",
+        description="""Jobs.""",
         args=JOB_ARGS,
         strip_null=Boolean(),
         delta_store=Boolean(default_value=True),
@@ -2067,7 +2336,7 @@ class Updated(ObjectType):
     )
     jobs = graphene.List(
         Job,
-        description="""Task jobs.""",
+        description="""Jobs.""",
         args=JOB_ARGS,
         strip_null=Boolean(),
         delta_store=Boolean(default_value=True),
