@@ -14,79 +14,32 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from async_timeout import timeout as ascyncto
+from async_timeout import timeout as asyncto
 import asyncio
 import json
 import os
-from typing import Union, Dict
+from typing import Any, List, Optional, Tuple, Union, Dict
 
 from cylc.flow.exceptions import ClientError, ClientTimeout
+from cylc.flow.network.client import WorkflowRuntimeClientBase
 from cylc.flow.network.client_factory import CommsMeth
-from cylc.flow.network import get_location
-from cylc.flow.remote import _remote_cylc_cmd
+from cylc.flow.remote import remote_cylc_cmd
 from cylc.flow.workflow_files import load_contact_file, ContactFileFields
 
 
-class WorkflowRuntimeClient():
-    """Client to scheduler communication using ssh.
+class WorkflowRuntimeClient(WorkflowRuntimeClientBase):
+    """Client to scheduler communication using ssh."""
 
-    Determines host from the contact file unless provided.
+    DEFAULT_TIMEOUT = 300  # seconds
+    SLEEP_INTERVAL = 0.1
 
-    Args:
-        workflow (str):
-            Name of the workflow to connect to.
-        timeout (float):
-            Set the default timeout in seconds.
-        host (str):
-            The host where the flow is running if known.
-    """
-    def __init__(
-            self,
-            workflow: str,
-            host: str = None,
-            timeout: Union[float, str] = None
+    async def async_request(
+        self, command: str,
+        args: Optional[Dict[str, Any]] = None,
+        timeout: Optional[float] = None,
+        req_meta: Optional[Dict[str, Any]] = None
     ):
-        self.workflow = workflow
-        self.SLEEP_INTERVAL = 0.1
-        if not host:
-            self.host, _, _ = get_location(workflow)
-        # 5 min default timeout
-        self.timeout = timeout if timeout is not None else 300
-
-    async def async_request(self, command, args=None, timeout=None):
         """Send asynchronous request via SSH.
-        """
-        timeout = timeout if timeout is not None else self.timeout
-        try:
-            async with ascyncto(timeout):
-                cmd, ssh_cmd, login_sh, cylc_path, msg = self.prepare_command(
-                    command, args, timeout)
-                proc = _remote_cylc_cmd(
-                    cmd,
-                    host=self.host,
-                    stdin_str=msg,
-                    ssh_cmd=ssh_cmd,
-                    remote_cylc_path=cylc_path,
-                    ssh_login_shell=login_sh,
-                    capture_process=True)
-                while True:
-                    if proc.poll() is not None:
-                        break
-                    await asyncio.sleep(self.SLEEP_INTERVAL)
-                out, err = (f.decode() for f in proc.communicate())
-                return_code = proc.wait()
-                if return_code:
-                    raise ClientError(err, f"return-code={return_code}")
-                return json.loads(out)
-        except asyncio.TimeoutError:
-            raise ClientTimeout(
-                f"Command exceeded the timeout {timeout}. "
-                f"This could be due to network problems. "
-                "Check the workflow log."
-            )
-
-    def serial_request(self, command, args=None, timeout=None):
-        """Send a request, using ssh.
 
         Determines ssh_cmd, cylc_path and login_shell settings from the contact
         file.
@@ -94,8 +47,6 @@ class WorkflowRuntimeClient():
         Converts message to JSON and sends this to stdin. Executes the Cylc
         command, then deserialises the output.
 
-        Use ``__call__`` to call this method.
-
         Args:
             command (str): The name of the endpoint to call.
             args (dict): Arguments to pass to the endpoint function.
@@ -105,24 +56,51 @@ class WorkflowRuntimeClient():
         Returns:
             object: Deserialized output from function called.
         """
-        loop = asyncio.new_event_loop()
-        task = loop.create_task(
-            self.async_request(command, args, timeout))
-        loop.run_until_complete(task)
-        loop.close()
-        return task.result()
+        if timeout is None:
+            timeout = self.timeout
+        try:
+            async with asyncto(timeout):
+                cmd, ssh_cmd, login_sh, cylc_path, msg = self.prepare_command(
+                    command, args, timeout
+                )
+                platform = {
+                    'ssh command': ssh_cmd,
+                    'cylc path': cylc_path,
+                    'use login shell': login_sh,
+                }
+                # NOTE: this can not raise NoHostsError
+                # because we have provided the host
+                proc = remote_cylc_cmd(
+                    cmd,
+                    platform,
+                    host=self.host,
+                    stdin_str=msg,
+                    capture_process=True
+                )
+                while proc.poll() is None:
+                    await asyncio.sleep(self.SLEEP_INTERVAL)
+                out, err = proc.communicate()
+                if proc.returncode:
+                    raise ClientError(err, f"return-code={proc.returncode}")
+                return json.loads(out)
+        except asyncio.TimeoutError:
+            self.timeout_handler()
+            raise ClientTimeout(
+                f"Command exceeded the timeout {timeout}s. "
+                "This could be due to network problems. "
+                "Check the workflow log."
+            )
 
     def prepare_command(
-        self, command: str, args: Dict, timeout: Union[float, str]
-    ):
-        """Prepare command for submission.
-        """
+        self, command: str, args: Optional[dict], timeout: Union[float, str]
+    ) -> Tuple[List[str], str, str, Optional[str], str]:
+        """Prepare command for submission."""
         # Set environment variable to determine the communication for use on
         # the scheduler
         os.environ["CLIENT_COMMS_METH"] = CommsMeth.SSH.value
         cmd = ["client"]
         if timeout:
-            cmd += [f'--comms-timeout={timeout}']
+            cmd.append(f'--comms-timeout={timeout}')
         cmd += [self.workflow, command]
         contact = load_contact_file(self.workflow)
         ssh_cmd = contact[ContactFileFields.SCHEDULER_SSH_COMMAND]
@@ -133,5 +111,3 @@ class WorkflowRuntimeClient():
             args = {}
         message = json.dumps(args)
         return cmd, ssh_cmd, login_shell, cylc_path, message
-
-    __call__ = serial_request
