@@ -30,7 +30,7 @@ def empyprocess(
     _fpath: str,
     flines: t.List[str],
     dir_: str,
-    template_vars: t.Dict[str, t.Any] = None,
+    template_vars: t.Optional[t.Dict[str, t.Any]] = None,
 ) -> t.List[str]:
     """Pass configure file through EmPy processor.
 
