@@ -31,6 +31,7 @@ parsec config file parsing:
 """
 
 import os
+from optparse import Values
 from pathlib import Path
 import re
 import sys
@@ -45,6 +46,9 @@ from cylc.flow.parsec.exceptions import (
 from cylc.flow.parsec.OrderedDict import OrderedDictWithDefaults
 from cylc.flow.parsec.include import inline
 from cylc.flow.parsec.util import itemstr
+from cylc.flow.templatevars import get_template_vars_from_db
+from cylc.flow.workflow_files import (
+    get_workflow_source_dir, check_flow_file)
 
 
 # heading/sections can contain commas (namespace name lists) and any
@@ -127,15 +131,11 @@ def _concatenate(lines):
 
 
 def addsect(cfig, sname, parents):
-    """Add a new section to a nested dict."""
+    """Add a new section to a nested dict, if it doesn't exist."""
     for p in parents:
         # drop down the parent list
         cfig = cfig[p]
-    if sname in cfig:
-        # this doesn't warrant a warning unless contained items are repeated
-        LOG.debug(
-            'Section already encountered: %s', itemstr(parents + [sname]))
-    else:
+    if sname not in cfig:
         cfig[sname] = OrderedDictWithDefaults()
 
 
@@ -347,6 +347,43 @@ def merge_template_vars(
         return native_tvars
 
 
+def _prepend_old_templatevars(fpath: str, template_vars: t.Dict) -> t.Dict:
+    """If the fpath is in a rundir, extract template variables from database.
+
+    Args:
+        fpath: filepath of workflow config file.
+        template_vars: Template vars to prepend old tvars to.
+    """
+    rundir = Path(fpath).parent
+    old_tvars = get_template_vars_from_db(rundir)
+    old_tvars.update(template_vars)
+    template_vars = old_tvars
+    return template_vars
+
+
+def _get_fpath_for_source(fpath: str, opts: "Values") -> str:
+    """If `--against-source` is set and a sourcedir can be found, return
+    sourcedir.
+    Else return fpath unchanged.
+
+    Raises:
+        ParsecError: If validate against source is set and this is not
+        an installed workflow then we don't want to continue.
+    """
+    if getattr(opts, 'against_source', False):
+        thispath = Path(fpath)
+        source_dir = get_workflow_source_dir(thispath.parent)[0]
+        if source_dir:
+            retpath = check_flow_file(source_dir)
+            return str(retpath)
+        else:
+            raise ParsecError(
+                f'Cannot validate {thispath.parent} against source: '
+                'it is not an installed workflow.')
+    else:
+        return fpath
+
+
 def read_and_proc(
     fpath: str,
     template_vars: t.Optional[t.Dict[str, t.Any]] = None,
@@ -359,9 +396,24 @@ def read_and_proc(
     Jinja2 processing must be done before concatenation - it could be
     used to generate continuation lines.
     """
-
+    template_vars = template_vars if template_vars is not None else {}
+    template_vars = _prepend_old_templatevars(fpath, template_vars)
+    fpath = _get_fpath_for_source(fpath, opts)
     fdir = os.path.dirname(fpath)
 
+    try:
+        original_cwd = os.getcwd()
+    except FileNotFoundError:
+        # User's current working directory does not actually exist, so we won't
+        # be able to change back to it later. (Note this might not be enough to
+        # prevent file parsing commands failing due to missing cwd elsewhere in
+        # the Python library).
+        original_cwd = None
+
+    # Move to the file location to give the template processor easy access to
+    # other files in the workflow directory (whether source or installed).
+    os.chdir(fdir)
+
     # Allow Python modules in lib/python/ (e.g. for use by Jinja2 filters).
     workflow_lib_python = os.path.join(fdir, "lib", "python")
     if (
@@ -455,6 +507,10 @@ def read_and_proc(
     if do_contin:
         flines = _concatenate(flines)
 
+    # If the user's original working directory exists, change back to it.
+    if original_cwd is not None:
+        os.chdir(original_cwd)
+
     # return rstripped lines
     return [fl.rstrip() for fl in flines]
 
