@@ -26,8 +26,9 @@ import re
 import shlex
 from collections import deque
 from textwrap import dedent
+from typing import List, Dict, Any, Tuple
 
-from metomi.isodatetime.data import Duration, TimePoint, Calendar
+from metomi.isodatetime.data import Duration, TimePoint
 from metomi.isodatetime.dumpers import TimePointDumper
 from metomi.isodatetime.parsers import TimePointParser, DurationParser
 from metomi.isodatetime.exceptions import IsodatetimeError, ISO8601SyntaxError
@@ -204,10 +205,28 @@ class ParsecValidator:
                 else:
                     speckey = key
                 specval = spec[speckey]
-                if isinstance(value, dict) and not specval.is_leaf():
+
+                cfg_is_section = isinstance(value, dict)
+                spec_is_section = not specval.is_leaf()
+                if cfg_is_section and not spec_is_section:
+                    # config is a [section] but it should be a setting=
+                    raise IllegalItemError(
+                        keys,
+                        key,
+                        msg=f'"{key}" should be a setting not a [section]',
+                    )
+                if (not cfg_is_section) and spec_is_section:
+                    # config is a setting= but it should be a [section]
+                    raise IllegalItemError(
+                        keys,
+                        key,
+                        msg=f'"{key}" should be a [section] not a setting',
+                    )
+
+                if cfg_is_section and spec_is_section:
                     # Item is dict, push to queue
                     queue.append([value, specval, keys + [key]])
-                elif value is not None and specval.is_leaf():
+                elif value is not None and not spec_is_section:
                     # Item is value, coerce according to value type
                     cfg[key] = self.coercers[specval.vdr](value, keys + [key])
                     if specval.options:
@@ -904,9 +923,7 @@ class CylcConfigValidator(ParsecValidator):
             interval = DurationParser().parse(value)
         except IsodatetimeError as exc:
             raise IllegalValueError("ISO 8601 interval", keys, value, exc=exc)
-        days, seconds = interval.get_days_and_seconds()
-        return DurationFloat(
-            days * Calendar.default().SECONDS_IN_DAY + seconds)
+        return DurationFloat(interval.get_seconds())
 
     @classmethod
     def coerce_interval_list(cls, value, keys):
@@ -978,6 +995,66 @@ class CylcConfigValidator(ParsecValidator):
         except ValueError:
             return items
 
+    @staticmethod
+    def parse_xtrig_arglist(value: str) -> Tuple[List[Any], Dict[str, Any]]:
+        """Parse Pythonic-like arg/kwarg signatures.
+
+        A stateful parser treats all args/kwargs as strings with
+        implicit quoting.
+
+        Examples:
+            >>> parse = CylcConfigValidator.parse_xtrig_arglist
+
+            # Parse pythonic syntax
+            >>> parse('a, b, c, d=1, e=2,')
+            (['a', 'b', 'c'], {'d': '1', 'e': '2'})
+            >>> parse('a, "1,2,3", b=",=",')
+            (['a', '"1,2,3"'], {'b': '",="'})
+            >>> parse('a, "b c", '"'d=e '")
+            (['a', '"b c"', "'d=e '"], {})
+
+            # Parse implicit (i.e. unquoted) strings
+            >>> parse('%(cycle)s, %(task)s, output=succeeded')
+            (['%(cycle)s', '%(task)s'], {'output': 'succeeded'})
+
+        """
+        # results
+        args = []
+        kwargs = {}
+        # state
+        in_str = False  # are we inside a quoted string
+        in_kwarg = False  # are we after the = sign of a kwarg
+        buffer = ''  # the current argument being parsed
+        kwarg_buffer = ''  # the key of a kwarg if in_kwarg == True
+        # parser
+        for char in value:
+            if char in {'"', "'"}:
+                in_str = not in_str
+                buffer += char
+            elif not in_str and char == ',':
+                if in_kwarg:
+                    kwargs[kwarg_buffer.strip()] = buffer.strip()
+                    in_kwarg = False
+                    kwarg_buffer = ''
+                else:
+                    args.append(buffer.strip())
+                buffer = ''
+            elif char == '=' and not in_str and not in_kwarg:
+                in_kwarg = True
+                kwarg_buffer = buffer
+                buffer = ''
+            else:
+                buffer += char
+
+        # reached the end of the string
+        if buffer:
+            if in_kwarg:
+                kwargs[kwarg_buffer.strip()] = buffer.strip()
+            else:
+                args.append(buffer.strip())
+
+        return args, kwargs
+
     @classmethod
     def coerce_xtrigger(cls, value, keys):
         """Coerce a string into an xtrigger function context object.
@@ -986,17 +1063,26 @@ class CylcConfigValidator(ParsecValidator):
         Checks for legal string templates in arg values too.
 
         Examples:
-            >>> CylcConfigValidator.coerce_xtrigger('a(b, c):PT1M', [None])
-            a(b, c):60.0
+            >>> xtrig = CylcConfigValidator.coerce_xtrigger
+
+            >>> ctx = xtrig('a(b, c):PT1M', [None])
+            >>> ctx.get_signature()
+            'a(b, c)'
+            >>> ctx.intvl
+            60.0
+
+            # cast types
+            >>> x = xtrig('a(1, 1.1, True, abc, x=True, y=1.1)', [None])
+            >>> x.func_args
+            [1, 1.1, True, 'abc']
+            >>> x.func_kwargs
+            {'x': True, 'y': 1.1}
 
         """
-
         label = keys[-1]
         value = cls.strip_and_unquote(keys, value)
         if not value:
             raise IllegalValueError("xtrigger", keys, value)
-        args = []
-        kwargs = {}
         match = cls._REC_TRIG_FUNC.match(value)
         if match is None:
             raise IllegalValueError("xtrigger", keys, value)
@@ -1004,15 +1090,18 @@ class CylcConfigValidator(ParsecValidator):
         if intvl:
             intvl = cls.coerce_interval(intvl, keys)
 
-        if fargs:
-            # Extract function args and kwargs.
-            for farg in fargs.split(r','):
-                try:
-                    key, val = farg.strip().split(r'=', 1)
-                except ValueError:
-                    args.append(cls._coerce_type(farg.strip()))
-                else:
-                    kwargs[key.strip()] = cls._coerce_type(val.strip())
+        # parse args
+        args, kwargs = CylcConfigValidator.parse_xtrig_arglist(fargs or '')
+
+        # cast types
+        args = [
+            CylcConfigValidator._coerce_type(arg)
+            for arg in args
+        ]
+        kwargs = {
+            key: CylcConfigValidator._coerce_type(value)
+            for key, value in kwargs.items()
+        }
 
         return SubFuncContext(label, fname, args, kwargs, intvl)
 
@@ -1047,6 +1136,76 @@ class CylcConfigValidator(ParsecValidator):
         return val
 
 
+# BACK COMPAT: BroadcastConfigValidator
+# The DB at 8.0.x stores Interval values as neither ISO8601 duration
+# string or DurationFloat. This has been fixed at 8.1.0, and
+# the following class acts as a bridge between fixed and broken.
+# url:
+#     https://github.com/cylc/cylc-flow/pull/5138
+# from:
+#    8.0.x
+# to:
+#    8.1.x
+# remove at:
+#    8.x
+class BroadcastConfigValidator(CylcConfigValidator):
+    """Validate and Coerce DB loaded broadcast config to internal objects."""
+    def __init__(self):
+        CylcConfigValidator.__init__(self)
+
+    @classmethod
+    def strip_and_unquote_list(cls, keys, value):
+        """Remove leading and trailing spaces and unquote list value.
+
+        Args:
+            keys (list):
+                Keys in nested dict that represents the raw configuration.
+            value (str):
+                String value in raw configuration that is supposed to be a
+                comma separated list.
+
+        Return (list):
+            Processed value as a list.
+
+        Examples:
+            >>> BroadcastConfigValidator.strip_and_unquote_list(
+            ...    None, '["1, 2", 3]'
+            ... )
+            ['1, 2', '3']
+        """
+        if value.startswith('[') and value.endswith(']'):
+            value = value.lstrip('[').rstrip(']')
+        return ParsecValidator.strip_and_unquote_list(keys, value)
+
+    @classmethod
+    def coerce_interval(cls, value, keys):
+        """Coerce an ISO 8601 interval into seconds.
+
+        Examples:
+            >>> BroadcastConfigValidator.coerce_interval('PT1H', None)
+            3600.0
+            >>> x = BroadcastConfigValidator.coerce_interval('62', None)
+            >>> x
+            62.0
+            >>> str(x)
+            'PT1M2S'
+
+        """
+        value = cls.strip_and_unquote(keys, value)
+        if not value:
+            # Allow explicit empty values.
+            return None
+        try:
+            interval = DurationParser().parse(value)
+        except IsodatetimeError:
+            try:
+                interval = DurationParser().parse(str(DurationFloat(value)))
+            except IsodatetimeError as exc:
+                raise IllegalValueError(
+                    "ISO 8601 interval", keys, value, exc=exc)
+        return DurationFloat(interval.get_seconds())
+
+
 def cylc_config_validate(cfg_root, spec_root):
     """Short for "CylcConfigValidator().validate(...)"."""
     return CylcConfigValidator().validate(cfg_root, spec_root)
