@@ -15,16 +15,18 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """Functions to return paths to common workflow files and directories."""
 
+import errno
 import os
 from pathlib import Path
 import re
 from shutil import rmtree
+from time import sleep
 from typing import Dict, Iterable, Set, Union, Optional, Any
 
 from cylc.flow import LOG
 from cylc.flow.cfgspec.glbl_cfg import glbl_cfg
 from cylc.flow.exceptions import (
-    InputError, WorkflowFilesError, handle_rmtree_err
+    FileRemovalError, InputError, WorkflowFilesError
 )
 from cylc.flow.platforms import get_localhost_install_target
 
@@ -42,6 +44,8 @@ EXPLICIT_RELATIVE_PATH_REGEX = re.compile(
 )
 """Matches relative paths that are explicit (starts with ./)"""
 
+SHELL_ENV_VARS = re.compile(r'\$[^$/]*')
+
 
 def expand_path(*args: Union[Path, str]) -> str:
     """Expand both vars and user in path and normalise it, joining any
@@ -52,19 +56,19 @@ def expand_path(*args: Union[Path, str]) -> str:
 
 
 def get_remote_workflow_run_dir(
-    workflow_name: Union[Path, str], *args: Union[Path, str]
+    workflow_id: Union[Path, str], *args: Union[Path, str]
 ) -> str:
     """Return remote workflow run directory, joining any extra args,
     NOT expanding vars or user."""
-    return os.path.join(_CYLC_RUN_DIR, workflow_name, *args)
+    return os.path.join(_CYLC_RUN_DIR, workflow_id, *args)
 
 
 def get_remote_workflow_run_job_dir(
-    workflow_name: Union[Path, str], *args: Union[Path, str]
+    workflow_id: Union[Path, str], *args: Union[Path, str]
 ) -> str:
     """Return remote workflow job log directory, joining any extra args,
     NOT expanding vars or user."""
-    return get_remote_workflow_run_dir(workflow_name, 'log', 'job', *args)
+    return get_remote_workflow_run_dir(workflow_id, 'log', 'job', *args)
 
 
 def get_cylc_run_dir() -> str:
@@ -73,14 +77,14 @@ def get_cylc_run_dir() -> str:
 
 
 def get_workflow_run_dir(
-    workflow_name: Union[Path, str], *args: Union[Path, str]
+    workflow_id: Union[Path, str], *args: Union[Path, str]
 ) -> str:
     """Return local workflow run directory, joining any extra args, and
     expanding vars and user.
 
     Does not check that the directory exists.
     """
-    return expand_path(_CYLC_RUN_DIR, workflow_name, *args)
+    return expand_path(_CYLC_RUN_DIR, workflow_id, *args)
 
 
 def get_workflow_run_job_dir(workflow, *args):
@@ -174,11 +178,13 @@ def make_localhost_symlinks(
         else:
             symlink_path = os.path.join(rund, key)
         target = expand_path(value)
-        if '$' in target:
+        env_vars = SHELL_ENV_VARS.findall(target)
+        if env_vars:
             raise WorkflowFilesError(
-                f'Unable to create symlink to {target}.'
-                f" '{value}' contains an invalid environment variable."
-                ' Please check configuration.')
+                f"Can't symlink to {target}\n"
+                "Undefined variables, check "
+                f"global config: {', '.join(env_vars)}")
+
         symlink_success = make_symlink_dir(symlink_path, target)
         # Symlink info returned for logging purposes. Symlinks should be
         # created before logs as the log dir may be a symlink.
@@ -189,7 +195,7 @@ def make_localhost_symlinks(
 
 def get_dirs_to_symlink(
     install_target: str,
-    workflow_name: str,
+    workflow_id: str,
     symlink_conf: Optional[Dict[str, Dict[str, Any]]] = None
 ) -> Dict[str, str]:
     """Returns dictionary of directories to symlink.
@@ -198,7 +204,7 @@ def get_dirs_to_symlink(
 
     Args:
         install_target: Symlinks to be created on this install target
-        flow_name: full name of the run, e.g. myflow/run1
+        workflow_id: full id of the run, e.g. myflow/run1
         symlink_conf: Symlink dirs, if sent on the cli.
             Defaults to None, in which case global config symlink dirs will
             be applied.
@@ -214,13 +220,13 @@ def get_dirs_to_symlink(
     base_dir = symlink_conf[install_target]['run']
     if base_dir:
         dirs_to_symlink['run'] = os.path.join(
-            base_dir, 'cylc-run', workflow_name)
+            base_dir, 'cylc-run', workflow_id)
     for dir_ in ['log', 'share', 'share/cycle', 'work']:
         link = symlink_conf[install_target].get(dir_, None)
         if (not link) or link == base_dir:
             continue
         dirs_to_symlink[dir_] = os.path.join(
-            link, 'cylc-run', workflow_name, dir_)
+            link, 'cylc-run', workflow_id, dir_)
     return dirs_to_symlink
 
 
@@ -293,7 +299,7 @@ def remove_dir_and_target(path: Union[Path, str]) -> None:
                 "Removing symlink and its target directory: "
                 f"{path} -> {target}"
             )
-            rmtree(target, onerror=handle_rmtree_err)
+            _rmtree(target)
         else:
             LOG.info(f'Removing broken symlink: {path}')
         os.remove(path)
@@ -301,7 +307,46 @@ def remove_dir_and_target(path: Union[Path, str]) -> None:
         raise FileNotFoundError(path)
     else:
         LOG.info(f'Removing directory: {path}')
-        rmtree(path, onerror=handle_rmtree_err)
+        _rmtree(path)
+
+
+def _rmtree(
+    target: Union[Path, str],
+    retries: int = 10,
+    sleep_time: float = 1,
+):
+    """Make rmtree more robust to nfs issues.
+
+    If a file is deleted which is being held open for reading by
+    another process. NFS will create a ".nfs" file in the
+    containing directory to handle this.
+
+    If you try to delete the directory which contains these
+    files you will get either a ENOTEMPTY or EBUSY error.
+
+    A likely cause of open file handles in cylc-run directories
+    is `cylc cat-log -m t`. If the file being cat-log'ged is removed,
+    the command will fail on its next poll. The default poll
+    interval is one second, so if we wait a couple of seconds and
+    retry the removal it will likely work.
+
+    This command retries removal a specified number
+    of times at a specified interval before failing to
+    give cat-log process a chance to die gracefully and
+    release their filesystem locks. For more info see:
+    https://github.com/cylc/cylc-flow/pull/5359#issuecomment-1479989975
+    """
+    for _try_num in range(retries):
+        try:
+            rmtree(target)
+            return
+        except OSError as exc:
+            if exc.errno in {errno.ENOTEMPTY, errno.EBUSY}:
+                err = exc
+                sleep(sleep_time)
+            else:
+                raise
+    raise FileRemovalError(err)
 
 
 def remove_dir_or_file(path: Union[Path, str]) -> None:
@@ -321,7 +366,7 @@ def remove_dir_or_file(path: Union[Path, str]) -> None:
         os.remove(path)
     else:
         LOG.info(f"Removing directory: {path}")
-        rmtree(path, onerror=handle_rmtree_err)
+        _rmtree(path)
 
 
 def remove_empty_parents(
@@ -423,7 +468,11 @@ def parse_rm_dirs(rm_dirs: Iterable[str]) -> Set[str]:
 
 
 def is_relative_to(path1: Union[Path, str], path2: Union[Path, str]) -> bool:
-    """Return whether or not path1 is relative to path2."""
+    """Return whether or not path1 is relative to path2.
+
+    Normalizes both paths to avoid trickery with paths containing `..`
+    somewhere in them.
+    """
     # In future, we can just use pathlib.Path.is_relative_to()
     # when Python 3.9 becomes the minimum supported version
     try:
