@@ -21,12 +21,16 @@ from os.path import expandvars
 from pprint import pformat
 import sqlite3
 import traceback
-from typing import List, Tuple
+from typing import TYPE_CHECKING, Iterable, List, Optional, Tuple, Union
 
 from cylc.flow import LOG
+from cylc.flow.exceptions import PlatformLookupError
 from cylc.flow.util import deserialise
 import cylc.flow.flags
 
+if TYPE_CHECKING:
+    from pathlib import Path
+
 
 @dataclass
 class CylcWorkflowDAOTableColumn:
@@ -170,11 +174,18 @@ class CylcWorkflowDAO:
     CONN_TIMEOUT = 0.2
     DB_FILE_BASE_NAME = "db"
     MAX_TRIES = 100
-    RESTART_INCOMPAT_VERSION = "8.0b3"  # Can't restart if <= this version
+    RESTART_INCOMPAT_VERSION = "8.0rc2"  # Can't restart if <= this version
     TABLE_BROADCAST_EVENTS = "broadcast_events"
     TABLE_BROADCAST_STATES = "broadcast_states"
     TABLE_INHERITANCE = "inheritance"
     TABLE_WORKFLOW_PARAMS = "workflow_params"
+    # BACK COMPAT: suite_params
+    # This Cylc 7 DB table is needed to allow workflow-state
+    # xtriggers (and the `cylc workflow-state` command) to
+    # work with Cylc 7 workflows.
+    # url: https://github.com/cylc/cylc-flow/issues/5236
+    # remove at: 8.x
+    TABLE_SUITE_PARAMS = "suite_params"
     TABLE_WORKFLOW_FLOWS = "workflow_flows"
     TABLE_WORKFLOW_TEMPLATE_VARS = "workflow_template_vars"
     TABLE_TASK_JOBS = "task_jobs"
@@ -232,10 +243,12 @@ class CylcWorkflowDAO:
             ["delay"],
             ["timeout"],
         ],
+        # NOTE: this table is used by `cylc clean`, don't rename me!
         TABLE_TASK_JOBS: [
             ["cycle", {"is_primary_key": True}],
             ["name", {"is_primary_key": True}],
             ["submit_num", {"datatype": "INTEGER", "is_primary_key": True}],
+            ["flow_nums"],
             ["is_manual_submit", {"datatype": "INTEGER"}],
             ["try_num", {"datatype": "INTEGER"}],
             ["time_submit"],
@@ -245,6 +258,7 @@ class CylcWorkflowDAO:
             ["time_run_exit"],
             ["run_signal"],
             ["run_status", {"datatype": "INTEGER"}],
+            # NOTE: this field is used by `cylc clean` don't rename me!
             ["platform_name"],
             ["job_runner_name"],
             ["job_id"],
@@ -297,6 +311,7 @@ class CylcWorkflowDAO:
             ["submit_num", {"datatype": "INTEGER"}],
             ["status"],
             ["flow_wait", {"datatype": "INTEGER"}],
+            ["is_manual_submit", {"datatype": "INTEGER"}],
         ],
         TABLE_TASK_TIMEOUT_TIMERS: [
             ["cycle", {"is_primary_key": True}],
@@ -314,26 +329,44 @@ class CylcWorkflowDAO:
         ],
     }
 
-    def __init__(self, db_file_name, is_public=False):
+    def __init__(
+        self,
+        db_file_name: Union['Path', str],
+        is_public: bool = False,
+        create_tables: bool = False
+    ):
         """Initialise database access object.
 
+        An instance of this class can also be opened as a context manager
+        which will automatically close the DB connection.
+
         Args:
-            db_file_name (str): Path to the database file.
-            is_public (bool): If True, allow retries, etc.
+            db_file_name: Path to the database file.
+            is_public: If True, allow retries.
+            create_tables: If True, create the tables if they
+                don't already exist.
 
         """
         self.db_file_name = expandvars(db_file_name)
         self.is_public = is_public
-        self.conn = None
+        self.conn: Optional[sqlite3.Connection] = None
         self.n_tries = 0
 
-        self.tables = {}
-        for name, attrs in sorted(self.TABLES_ATTRS.items()):
-            self.tables[name] = CylcWorkflowDAOTable(name, attrs)
+        self.tables = {
+            name: CylcWorkflowDAOTable(name, attrs)
+            for name, attrs in sorted(self.TABLES_ATTRS.items())
+        }
 
-        if not self.is_public:
+        if create_tables:
             self.create_tables()
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        """Close DB connection when leaving context manager."""
+        self.close()
+
     def add_delete_item(self, table_name, where_args=None):
         """Queue a DELETE item for a given table.
 
@@ -519,13 +552,10 @@ class CylcWorkflowDAO:
         for row_idx, row in enumerate(self.connect().execute(stmt)):
             callback(row_idx, list(row))
 
-    def select_workflow_params(self, callback):
-        """Select from workflow_params.
+    def select_workflow_params(self) -> Iterable[Tuple[str, Optional[str]]]:
+        """Select all from workflow_params.
 
-        Invoke callback(row_idx, row) on each row, where each row contains:
-            [key, value]
-
-        E.g. a row might be ['UTC mode', '1']
+        E.g. a row might be ('UTC mode', '1')
         """
         stmt = rf'''
             SELECT
@@ -533,8 +563,7 @@ class CylcWorkflowDAO:
             FROM
                 {self.TABLE_WORKFLOW_PARAMS}
         '''  # nosec (table name is code constant)
-        for row_idx, row in enumerate(self.connect().execute(stmt)):
-            callback(row_idx, list(row))
+        return self.connect().execute(stmt)
 
     def select_workflow_flows(self, flow_nums):
         """Return flow data for selected flows."""
@@ -714,7 +743,16 @@ class CylcWorkflowDAO:
             callback(row_idx, list(row))
 
     def select_task_job_platforms(self):
-        """Return the set of platform names from task_jobs table."""
+        """Return the set of platform names from task_jobs table.
+
+        Warning:
+            This interface is used by `cylc clean` which does not upgrade the
+            DB first (it could, this would only extend backwards
+            compatibility but would not help with forwards compatibility).
+            Keep this query to the minimum of tables/fields to avoid
+            breaking compatibility with older/newer versions of Cylc.
+
+        """
         stmt = rf'''
             SELECT DISTINCT
                 platform_name
@@ -748,6 +786,14 @@ class CylcWorkflowDAO:
             ret[submit_num] = (flow_wait == 1, deserialise(flow_nums_str))
         return ret
 
+    def select_latest_flow_nums(self):
+        """Return a list of the most recent previous flow numbers."""
+        stmt = rf'''
+            SELECT flow_nums, MAX(time_created) FROM {self.TABLE_TASK_STATES}
+        '''  # nosec (table name is code constant)
+        flow_nums_str = list(self.connect().execute(stmt))[0][0]
+        return deserialise(flow_nums_str)
+
     def select_task_outputs(self, name, point):
         """Select task outputs for each flow.
 
@@ -802,14 +848,20 @@ class CylcWorkflowDAO:
         """Select from task_pool+task_states+task_jobs for restart.
 
         Invoke callback(row_idx, row) on each row, where each row contains:
-            [cycle, name, is_late, status, is_held, submit_num,
-             try_num, platform_name, time_submit, time_run, timeout, outputs]
+        the fields in the SELECT statement below.
+
+        Raises:
+            PlatformLookupError: Do not start up if platforms for running
+            tasks cannot be found in global.cylc. This exception should
+            not be caught.
         """
         form_stmt = r"""
             SELECT
                 %(task_pool)s.cycle,
                 %(task_pool)s.name,
                 %(task_pool)s.flow_nums,
+                %(task_states)s.flow_wait,
+                %(task_states)s.is_manual_submit,
                 %(task_late_flags)s.value,
                 %(task_pool)s.status,
                 %(task_pool)s.is_held,
@@ -855,8 +907,24 @@ class CylcWorkflowDAO:
             "task_outputs": self.TABLE_TASK_OUTPUTS,
         }
         stmt = form_stmt % form_data
+
+        # Run the callback, collecting any platform errors to be handled later:
+        platform_errors = []
         for row_idx, row in enumerate(self.connect().execute(stmt)):
-            callback(row_idx, list(row))
+            platform_error = callback(row_idx, list(row))
+            if platform_error:
+                platform_errors.append(platform_error)
+
+        # If any of the platforms could not be found, raise an exception
+        # and stop trying to play this workflow:
+        if platform_errors:
+            msg = (
+                "The following platforms are not defined in"
+                " the global.cylc file:"
+            )
+            for platform in platform_errors:
+                msg += f"\n * {platform}"
+            raise PlatformLookupError(msg)
 
     def select_task_prerequisites(
         self, cycle: str, name: str, flow_nums: str
@@ -1020,46 +1088,3 @@ class CylcWorkflowDAO:
     def vacuum(self):
         """Vacuum to the database."""
         return self.connect().execute("VACUUM")
-
-    def remove_columns(self, table, to_drop):
-        conn = self.connect()
-
-        # get list of columns to keep
-        schema = conn.execute(
-            rf'''
-                PRAGMA table_info({table})
-            '''
-        )
-        new_cols = [
-            name
-            for _, name, *_ in schema
-            if name not in to_drop
-        ]
-
-        # copy table
-        conn.execute(
-            rf'''
-                CREATE TABLE {table}_new AS
-                SELECT {', '.join(new_cols)}
-                FROM {table}
-            '''
-        )
-
-        # remove original
-        conn.execute(
-            rf'''
-                DROP TABLE {table}
-            '''
-        )
-
-        # copy table
-        conn.execute(
-            rf'''
-                CREATE TABLE {table} AS
-                SELECT {', '.join(new_cols)}
-                FROM {table}_new
-            '''
-        )
-
-        # done
-        conn.commit()
