@@ -201,7 +201,9 @@ ALIASES = {
     'ls': 'list',
     'shutdown': 'stop',
     'task-message': 'message',
-    'unhold': 'release'
+    'unhold': 'release',
+    'validate-install-play': 'vip',
+    'validate-reinstall': 'vr',
 }
 
 
@@ -311,12 +313,6 @@ def match_command(command):
             for cmd in COMMANDS
             if cmd.startswith(command)
         },
-        *{
-            # search aliases
-            cmd
-            for alias, cmd in ALIASES.items()
-            if alias.startswith(command)
-        }
     }
     if len(possible_cmds) == 0:
         print(
@@ -395,12 +391,19 @@ def print_id_help():
     print(ID_HELP)
 
 
-def print_license():
-    print(
-        pkg_resources.get_distribution('cylc-flow').get_resource_string(
-            __name__, 'COPYING'
-        ).decode()
-    )
+def print_license() -> None:
+    try:
+        from importlib.metadata import files
+    except ImportError:
+        # BACK COMPAT: importlib_metadata
+        #   importlib.metadata was added in Python 3.8
+        # FROM: Python 3.7
+        # TO: Python: 3.8
+        from importlib_metadata import files  # type: ignore[no-redef]
+    license_file = next(filter(
+        lambda f: f.name == 'COPYING', files('cylc-flow')
+    ))
+    print(license_file.read_text())
 
 
 def print_command_list(commands=None, indent=0):
