@@ -21,14 +21,14 @@
 Kill running or submitted jobs.
 
 Examples:
-  # kill a specific task in my_flow
-  $ cylc kill my_flow//1/a
+  # kill a specific task in my_workflow
+  $ cylc kill my_workflow//1/a
 
-  # kill multiple tasks in my_flow
+  # kill multiple tasks in my_workflow
   $ cylc kill myflow// //1/a //1/b //1/c
 
-  # kill all active tasks in the my_flow
-  $ cylc kill 'my_flow//*'
+  # kill all active tasks in the my_workflow
+  $ cylc kill 'my_workflow//*'
 """
 
 from functools import partial
@@ -81,7 +81,10 @@ async def run(options: 'Values', workflow_id: str, *tokens_list: Tokens):
         'request_string': MUTATION,
         'variables': {
             'wFlows': [workflow_id],
-            'tasks': [tokens.relative_id for tokens in tokens_list],
+            'tasks': [
+                tokens.relative_id_with_selectors
+                for tokens in tokens_list
+            ]
         }
     }
 
