@@ -18,7 +18,7 @@
 
 """cylc poll [OPTIONS] ARGS
 
-Poll (query) task jobs to verify and update their statuses.
+Poll (query) jobs to verify and update their statuses.
 
 This checks the job status file and queries the job runner on the job platform.
 
@@ -26,11 +26,11 @@ Pollable tasks are those in the n=0 window with an associated job ID,
 including incomplete finished tasks.
 
 Examples:
-  # poll all pollable tasks in my_flow
-  $ cylc poll 'my_flow//*'
+  # poll all pollable tasks in my_workflow
+  $ cylc poll 'my_workflow//*'
 
-  # poll specific tasks in my_flow
-  $ cylc poll my_flow// //1/a //1/b
+  # poll specific tasks in my_workflow
+  $ cylc poll my_workflow// //1/a //1/b
 """
 
 from functools import partial
@@ -82,7 +82,7 @@ async def run(options: 'Values', workflow_id: str, *tokens_list):
         'variables': {
             'wFlows': [workflow_id],
             'tasks': [
-                tokens.relative_id
+                tokens.relative_id_with_selectors
                 for tokens in tokens_list
             ],
         }
