@@ -132,8 +132,9 @@ def main(parser: COP, options: 'Values', workflow_id: str) -> None:
         # No output specified - choose summary by default
         options.show_summary = True
 
-    run_db = _get_dao(workflow_id)
-    row_buf = format_rows(*run_db.select_task_times())
+    db_file = get_workflow_run_pub_db_path(workflow_id)
+    with CylcWorkflowDAO(db_file, is_public=True) as dao:
+        row_buf = format_rows(*dao.select_task_times())
     with smart_open(options.output_filename) as output:
         if options.show_raw:
             output.write(row_buf.getvalue())
@@ -169,12 +170,6 @@ def format_rows(header, rows):
     return sio
 
 
-def _get_dao(workflow):
-    """Return the DAO (public) for workflow."""
-    return CylcWorkflowDAO(
-        get_workflow_run_pub_db_path(workflow), is_public=True)
-
-
 class TimingSummary:
     """Base class for summarizing timing output from cylc.flow run database."""
 
