@@ -36,13 +36,17 @@ Prerequisite satisfaction is not shown for past tasks reloaded from the
 workflow database.
 """
 
+import asyncio
+import re
 import json
-from optparse import Values
 import sys
-from typing import Dict
+from typing import Any, Dict, TYPE_CHECKING
 
 from ansimarkup import ansiprint
 
+from metomi.isodatetime.data import (
+    get_timepoint_from_seconds_since_unix_epoch as seconds2point)
+
 from cylc.flow.exceptions import InputError
 from cylc.flow.id import Tokens
 from cylc.flow.id_cli import parse_ids
@@ -52,12 +56,16 @@ from cylc.flow.task_state import (
     TASK_STATUS_RUNNING
 )
 from cylc.flow.option_parsers import (
+    CylcOptionParser as COP,
     ID_MULTI_ARG_DOC,
-    CylcOptionParser as COP
 )
 from cylc.flow.terminal import cli_function
 
 
+if TYPE_CHECKING:
+    from optparse import Values
+
+
 WORKFLOW_META_QUERY = '''
 query ($wFlows: [ID]!) {
   workflows (ids: $wFlows, stripNull: false) {
@@ -116,10 +124,6 @@ query ($wFlows: [ID]!, $taskIds: [ID]) {
       message
       satisfied
     }
-    clockTrigger {
-      timeString
-      satisfied
-    }
     externalTriggers {
       id
       label
@@ -144,6 +148,31 @@ def print_msg_state(msg, state):
 
 
 def flatten_data(data, flat_data=None):
+    """Reduce a nested data structure to a flat one.
+
+    Examples:
+        It flattens out nested dictionaries:
+        >>> flatten_data({})
+        {}
+        >>> flatten_data({'a': 1})
+        {'a': 1}
+        >>> flatten_data({'a': {'b': 2, 'c': {'d': 4}}})
+        {'b': 2, 'd': 4}
+
+        It iterates through any lists that it finds:
+        >>> flatten_data({'a': [{'b': 2}, {'c': 3}]})
+        {'b': 2, 'c': 3}
+
+        Overriding is determined by iteration order (don't rely on it):
+        >>> flatten_data({'a': 1, 'b': {'a': 2}})
+        {'a': 2}
+
+        It can't flatten things which aren't dicts:
+        >>> flatten_data({'a': ['x', 'y']})
+        Traceback (most recent call last):
+        AttributeError: 'str' object has no attribute 'items'
+
+    """
     if flat_data is None:
         flat_data = {}
     for key, value in data.items():
@@ -191,14 +220,14 @@ def get_option_parser():
     return parser
 
 
-def workflow_meta_query(workflow_id, pclient, options, json_filter):
+async def workflow_meta_query(workflow_id, pclient, options, json_filter):
     query = WORKFLOW_META_QUERY
     query_kwargs = {
         'request_string': query,
         'variables': {'wFlows': [workflow_id]}
     }
     # Print workflow info.
-    results = pclient('graphql', query_kwargs)
+    results = await pclient.async_request('graphql', query_kwargs)
     for workflow_id in results['workflows']:
         flat_data = flatten_data(workflow_id)
         if options.json:
@@ -211,7 +240,7 @@ def workflow_meta_query(workflow_id, pclient, options, json_filter):
     return 0
 
 
-def prereqs_and_outputs_query(
+async def prereqs_and_outputs_query(
     workflow_id,
     tokens_list,
     pclient,
@@ -220,7 +249,7 @@ def prereqs_and_outputs_query(
 ):
     ids_list = [
         # convert the tokens into standardised IDs
-        tokens.relative_id
+        tokens.relative_id_with_selectors
         for tokens in tokens_list
     ]
 
@@ -232,7 +261,7 @@ def prereqs_and_outputs_query(
             'taskIds': ids_list,
         }
     }
-    results = pclient('graphql', tp_kwargs)
+    results = await pclient.async_request('graphql', tp_kwargs)
     multi = len(results['taskProxies']) > 1
     for t_proxy in results['taskProxies']:
         task_id = Tokens(t_proxy['id']).relative_id
@@ -301,38 +330,63 @@ def prereqs_and_outputs_query(
                     info = f'{task_id} {output["label"]}'
                     print_msg_state(info, output['satisfied'])
                 if (
-                        t_proxy['clockTrigger']['timeString']
-                        or t_proxy['externalTriggers']
+                        t_proxy['externalTriggers']
                         or t_proxy['xtriggers']
                 ):
                     ansiprint(
                         "<bold>other:</bold> ('<red>-</red>': not satisfied)")
-                    if t_proxy['clockTrigger']['timeString']:
-                        state = t_proxy['clockTrigger']['satisfied']
-                        time_str = t_proxy['clockTrigger']['timeString']
-                        print_msg_state(
-                            'Clock trigger time reached',
-                            state)
-                        print(f'  o Triggers at ... {time_str}')
                     for ext_trig in t_proxy['externalTriggers']:
                         state = ext_trig['satisfied']
                         print_msg_state(
                             f'{ext_trig["label"]} ... {state}',
                             state)
                     for xtrig in t_proxy['xtriggers']:
+                        label = get_wallclock_label(xtrig) or xtrig['id']
                         state = xtrig['satisfied']
                         print_msg_state(
-                            f'xtrigger "{xtrig["label"]} = {xtrig["id"]}"',
+                            f'xtrigger "{xtrig["label"]} = {label}"',
                             state)
     if not results['taskProxies']:
         ansiprint(
-            f"<red>No matching n=0 tasks found: {', '.join(ids_list)}",
+            f"<red>No matching active tasks found: {', '.join(ids_list)}",
             file=sys.stderr)
         return 1
     return 0
 
 
-def task_meta_query(workflow_id, task_names, pclient, options, json_filter):
+def get_wallclock_label(xtrig: Dict[str, Any]) -> str:
+    """Return a label for an xtrigger if it is a wall_clock trigger.
+
+    Returns:
+        A label or False.
+
+    Examples:
+        >>> this = get_wallclock_label
+
+        >>> this({'id': 'wall_clock(trigger_time=0)'})
+        'wall_clock(trigger_time=1970-01-01T00:00:00Z)'
+
+        >>> this({'id': 'wall_clock(trigger_time=440143843)'})
+        'wall_clock(trigger_time=1983-12-13T06:10:43Z)'
+
+    """
+    wallclock_trigger = re.findall(
+        r'wall_clock\(trigger_time=(.*)\)', xtrig['id'])
+    if wallclock_trigger:
+        return (
+            'wall_clock(trigger_time='
+            f'{str(seconds2point(wallclock_trigger[0], True))})'
+        )
+    return ''
+
+
+async def task_meta_query(
+    workflow_id,
+    task_names,
+    pclient,
+    options,
+    json_filter,
+):
     tasks_query = TASK_META_QUERY
     tasks_kwargs = {
         'request_string': tasks_query,
@@ -342,7 +396,7 @@ def task_meta_query(workflow_id, task_names, pclient, options, json_filter):
         },
     }
     # Print workflow info.
-    results = pclient('graphql', tasks_kwargs)
+    results = await pclient.async_request('graphql', tasks_kwargs)
     multi = len(results['tasks']) > 1
     for task in results['tasks']:
         flat_data = flatten_data(task['meta'])
@@ -357,6 +411,44 @@ def task_meta_query(workflow_id, task_names, pclient, options, json_filter):
     return 0
 
 
+async def show(workflow_id, tokens_list, opts):
+    pclient = get_client(
+        workflow_id,
+        timeout=opts.comms_timeout,
+    )
+    json_filter: 'Dict' = {}
+
+    ret = 0
+    if opts.task_defs:
+        ret = await task_meta_query(
+            workflow_id,
+            opts.task_defs,
+            pclient,
+            opts,
+            json_filter,
+        )
+    elif not tokens_list:
+        ret = await workflow_meta_query(
+            workflow_id,
+            pclient,
+            opts,
+            json_filter,
+        )
+    else:
+        ret = await prereqs_and_outputs_query(
+            workflow_id,
+            tokens_list,
+            pclient,
+            opts,
+            json_filter,
+        )
+
+    if opts.json:
+        print(json.dumps(json_filter, indent=4))
+
+    return ret
+
+
 @cli_function(get_option_parser)
 def main(_, options: 'Values', *ids) -> None:
     """Implement "cylc show" CLI."""
@@ -373,30 +465,5 @@ def main(_, options: 'Values', *ids) -> None:
             'Cannot query both live tasks and task definitions.'
         )
 
-    pclient = get_client(workflow_id, timeout=options.comms_timeout)
-    json_filter: 'Dict' = {}
-
-    ret = 0
-    if options.task_defs:
-        ret = task_meta_query(
-            workflow_id,
-            options.task_defs,
-            pclient,
-            options,
-            json_filter,
-        )
-    elif not tokens_list:
-        ret = workflow_meta_query(workflow_id, pclient, options, json_filter)
-    else:
-        ret = prereqs_and_outputs_query(
-            workflow_id,
-            tokens_list,
-            pclient,
-            options,
-            json_filter,
-        )
-
-    if options.json:
-        print(json.dumps(json_filter, indent=4))
-
+    ret = asyncio.run(show(workflow_id, tokens_list, options))
     sys.exit(ret)
