@@ -31,12 +31,14 @@ from typing import Any, Callable, List, Optional
 from cylc.flow import LOG
 from cylc.flow.cfgspec.glbl_cfg import glbl_cfg
 from cylc.flow.cylc_subproc import procopen
+from cylc.flow.exceptions import PlatformLookupError
 from cylc.flow.hostuserutil import is_remote_host
 from cylc.flow.platforms import (
     log_platform_event,
     get_platform,
 )
 from cylc.flow.task_events_mgr import TaskJobLogsRetrieveContext
+from cylc.flow.task_proxy import TaskProxy
 from cylc.flow.wallclock import get_current_time_string
 
 _XTRIG_FUNCS: dict = {}
@@ -467,7 +469,7 @@ class SubProcPool:
         Args:
             ctx: SubProcContext object for this task.
             callback: Function to run on command exit.
-            callback_args: Arguments to proivide to callback
+            callback_args: Arguments to provide to callback
             callback_255: Function to run if command exits with a 255
                 error - usually associated with ssh being unable to
                 contact a remote host.
@@ -485,9 +487,17 @@ class SubProcPool:
 
         # If cmd is fileinstall, which uses rsync, get a platform so
         # that you can use that platform's ssh command.
+        platform_name = None
         platform = None
         if isinstance(ctx.cmd_key, TaskJobLogsRetrieveContext):
-            platform = get_platform(ctx.cmd_key.platform_name)
+            try:
+                platform = get_platform(ctx.cmd_key.platform_name)
+            except PlatformLookupError:
+                log_platform_event(
+                    'Unable to retrieve job logs.',
+                    {'name': ctx.cmd_key.platform_name},
+                    level='warning',
+                )
         elif callback_args:
             platform = callback_args[0]
             if not (
@@ -497,6 +507,15 @@ class SubProcPool:
             ):
                 # the first argument is not a platform
                 platform = None
+                # Backup, get a platform name from the config:
+                for arg in callback_args:
+                    if isinstance(arg, TaskProxy):
+                        platform_name = arg.tdef.rtconfig['platform']
+                    elif (
+                        isinstance(arg, list)
+                        and isinstance(arg[0], TaskProxy)
+                    ):
+                        platform_name = arg[0].tdef.rtconfig['platform']
 
         if cls.ssh_255_fail(ctx) or cls.rsync_255_fail(ctx, platform) is True:
             # Job log retrieval passes a special object as a command key
@@ -513,7 +532,7 @@ class SubProcPool:
                     ' unreachable hosts'
                     f'\n* {cmd_key} will retry if another host is available.'
                 ),
-                platform or {'name': None},
+                platform or {'name': platform_name},
                 level='warning',
             )
 
@@ -575,9 +594,10 @@ class SubProcPool:
                 f'testing connectivity for {ctx.host} using {ssh_test_cmd}'
             )
             ssh_test = run(
-                shlex.split(f'{ssh_cmd} {ctx.host} true'),
+                shlex.split(f'{ssh_cmd} {ctx.host} true'),  # nosec B603 *
                 capture_output=True
             )
+            # * (command is trusted input)
             if ssh_test.returncode == 255:
                 rsync_255_fail = True
         elif (
