@@ -17,7 +17,7 @@
 
 This module provides logic to:
 * Manage task messages (internal, polled or received).
-* Set up retries on task job failures (submission or execution).
+* Set up retries on job failures (submission or execution).
 * Generate task event handlers.
   * Retrieval of log files for completed remote jobs.
   * Email notification.
@@ -28,18 +28,18 @@ This module provides logic to:
 from contextlib import suppress
 from collections import namedtuple
 from enum import Enum
-from logging import getLevelName, INFO, DEBUG
+from logging import DEBUG, INFO, getLevelName
 import os
 from shlex import quote
 import shlex
 from time import time
-from typing import TYPE_CHECKING
-
-from cylc.flow.parsec.config import ItemNotFoundError
+from typing import TYPE_CHECKING, List, Optional, Union, cast
 
 from cylc.flow import LOG, LOG_LEVELS
 from cylc.flow.cfgspec.glbl_cfg import glbl_cfg
+from cylc.flow.exceptions import NoHostsError, PlatformLookupError
 from cylc.flow.hostuserutil import get_host, get_user, is_remote_platform
+from cylc.flow.parsec.config import ItemNotFoundError
 from cylc.flow.pathutil import (
     get_remote_workflow_run_job_dir,
     get_workflow_run_job_dir)
@@ -48,7 +48,10 @@ from cylc.flow.task_action_timer import (
     TaskActionTimer,
     TimerFlags
 )
-from cylc.flow.platforms import get_platform, get_host_from_platform
+from cylc.flow.platforms import (
+    get_platform, get_host_from_platform,
+    log_platform_event
+)
 from cylc.flow.task_job_logs import (
     get_task_job_log,
     get_task_job_activity_log,
@@ -183,7 +186,7 @@ class EventData(Enum):
 
     .. deprecated:: 8.0.0
 
-       Use 'uuid_str'.
+       Use 'uuid'.
     """
 
     CyclePoint = 'point'
@@ -430,7 +433,7 @@ class TaskEventsManager():
 
     def _get_remote_conf(self, itask, key):
         """Get deprecated "[remote]" items that default to platforms."""
-        overrides = self.broadcast_mgr.get_broadcast(itask.identity)
+        overrides = self.broadcast_mgr.get_broadcast(itask.tokens)
         SKEY = 'remote'
         if SKEY not in overrides:
             overrides[SKEY] = {}
@@ -442,7 +445,7 @@ class TaskEventsManager():
 
     def _get_workflow_platforms_conf(self, itask, key):
         """Return top level [runtime] items that default to platforms."""
-        overrides = self.broadcast_mgr.get_broadcast(itask.identity)
+        overrides = self.broadcast_mgr.get_broadcast(itask.tokens)
         return (
             overrides.get(key) or
             itask.tdef.rtconfig[key] or
@@ -523,13 +526,13 @@ class TaskEventsManager():
 
     def process_message(
         self,
-        itask,
-        severity,
-        message,
-        event_time=None,
-        flag=FLAG_INTERNAL,
-        submit_num=None,
-    ):
+        itask: 'TaskProxy',
+        severity: Union[str, int],
+        message: str,
+        event_time: Optional[str] = None,
+        flag: str = FLAG_INTERNAL,
+        submit_num: Optional[int] = None,
+    ) -> Optional[bool]:
         """Parse a task message and update task state.
 
         Incoming, e.g. "succeeded at <TIME>", may be from task job or polling.
@@ -545,16 +548,16 @@ class TaskEventsManager():
         somehow uniquely associating each poll with its result message.
 
         Arguments:
-            itask (cylc.flow.task_proxy.TaskProxy):
+            itask:
                 The task proxy object relevant for the message.
-            severity (str or int):
+            severity:
                 Message severity, should be a recognised logging level.
-            message (str):
+            message:
                 Message content.
-            event_time (str):
+            event_time:
                 Event time stamp. Expect ISO8601 date time string.
                 If not specified, use current time.
-            flag (str):
+            flag:
                 If specified, can be:
                     FLAG_INTERNAL (default):
                         To indicate an internal message.
@@ -563,7 +566,7 @@ class TaskEventsManager():
                         external source.
                     FLAG_POLLED:
                         To indicate a message resulted from a poll.
-            submit_num (int):
+            submit_num:
                 The submit number of the task relevant for the message.
                 If not specified, use latest submit number.
 
@@ -577,6 +580,9 @@ class TaskEventsManager():
             event_time = get_current_time_string()
         if submit_num is None:
             submit_num = itask.submit_num
+        if isinstance(severity, int):
+            severity = cast(str, getLevelName(severity))
+        lseverity = str(severity).lower()
 
         # Any message represents activity.
         self.reset_inactivity_timer_func()
@@ -591,7 +597,7 @@ class TaskEventsManager():
         else:
             new_msg = message
         self.data_store_mgr.delta_job_msg(
-            itask.tokens.duplicate(job=str(submit_num)).relative_id,
+            itask.tokens.duplicate(job=str(submit_num)),
             new_msg
         )
 
@@ -623,8 +629,7 @@ class TaskEventsManager():
                 # one, so assume that a successful submission occurred and act
                 # accordingly. Note the submitted message is internal, whereas
                 # the started message comes in on the network.
-                self._process_message_submitted(
-                    itask, event_time, itask.submit_num)
+                self._process_message_submitted(itask, event_time)
                 self.spawn_func(itask, TASK_OUTPUT_SUBMITTED)
 
             self._process_message_started(itask, event_time)
@@ -667,17 +672,17 @@ class TaskEventsManager():
                 # If not in the preparing state we already assumed and handled
                 # job submission under the started event above...
                 # (sim mode does not have the job prep state)
-                self._process_message_submitted(itask, event_time, submit_num)
+                self._process_message_submitted(itask, event_time)
                 self.spawn_func(itask, TASK_OUTPUT_SUBMITTED)
 
             # ... but either way update the job ID in the job proxy (it only
             # comes in via the submission message).
             if itask.tdef.run_mode != 'simulation':
-                job_d = itask.tokens.duplicate(
+                job_tokens = itask.tokens.duplicate(
                     job=str(itask.submit_num)
-                ).relative_id
+                )
                 self.data_store_mgr.delta_job_attr(
-                    job_d, 'job_id', itask.summary['submit_method_id'])
+                    job_tokens, 'job_id', itask.summary['submit_method_id'])
 
         elif message.startswith(FAIL_MESSAGE_PREFIX):
             # Task received signal.
@@ -736,24 +741,22 @@ class TaskEventsManager():
             # Note that all messages are logged already at the top.
             # No state change.
             LOG.debug(f"[{itask}] unhandled: {message}")
-            if severity in LOG_LEVELS.values():
-                severity = getLevelName(severity)
             self._db_events_insert(
-                itask, ("message %s" % str(severity).lower()), message)
-        lseverity = str(severity).lower()
+                itask, (f"message {lseverity}"), message)
         if lseverity in self.NON_UNIQUE_EVENTS:
             itask.non_unique_events.update({lseverity: 1})
             self.setup_event_handlers(itask, lseverity, message)
+        return None
 
     def _process_message_check(
         self,
-        itask,
-        severity,
-        message,
-        event_time,
-        flag,
-        submit_num,
-    ):
+        itask: 'TaskProxy',
+        severity: str,
+        message: str,
+        event_time: str,
+        flag: str,
+        submit_num: int,
+    ) -> bool:
         """Helper for `.process_message`.
 
         See `.process_message` for argument list
@@ -808,10 +811,15 @@ class TaskEventsManager():
                 )
             return False
 
-        LOG.log(
-            LOG_LEVELS.get(severity, INFO),
-            f"[{itask}] {flag}{message}{timestamp}"
-        )
+        severity = cast(int, LOG_LEVELS.get(severity, INFO))
+        # Demote log level to DEBUG if this is a message that duplicates what
+        # gets logged by itask state change anyway (and not manual poll)
+        if severity > DEBUG and flag != self.FLAG_POLLED and message in {
+            self.EVENT_SUBMITTED, self.EVENT_STARTED, self.EVENT_SUCCEEDED,
+            self.EVENT_SUBMIT_FAILED, f'{FAIL_MESSAGE_PREFIX}ERR'
+        }:
+            severity = DEBUG
+        LOG.log(severity, f"[{itask}] {flag}{message}{timestamp}")
         return True
 
     def setup_event_handlers(self, itask, event, message):
@@ -918,7 +926,7 @@ class TaskEventsManager():
     def _get_events_conf(self, itask, key, default=None):
         """Return an events setting from workflow then global configuration."""
         for getter in [
-                self.broadcast_mgr.get_broadcast(itask.identity).get("events"),
+                self.broadcast_mgr.get_broadcast(itask.tokens).get("events"),
                 itask.tdef.rtconfig["mail"],
                 itask.tdef.rtconfig["events"],
                 glbl_cfg().get(["scheduler", "mail"]),
@@ -935,8 +943,36 @@ class TaskEventsManager():
 
     def _process_job_logs_retrieval(self, schd, ctx, id_keys):
         """Process retrieval of task job logs from remote user@host."""
-        platform = get_platform(ctx.platform_name)
-        host = get_host_from_platform(platform, bad_hosts=self.bad_hosts)
+        # get a host to run retrieval on
+        try:
+            platform = get_platform(ctx.platform_name)
+            host = get_host_from_platform(platform, bad_hosts=self.bad_hosts)
+        except NoHostsError:
+            # All of the platforms hosts have been found to be uncontactable.
+            # Reset the bad hosts to allow retrieval retry to take place.
+            self.bad_hosts -= set(platform['hosts'])
+            try:
+                # Get a new host and try again.
+                host = get_host_from_platform(
+                    platform,
+                    bad_hosts=self.bad_hosts
+                )
+            except NoHostsError:
+                # We really can't get a host to try on e.g. no hosts
+                # configured (shouldn't happen). Nothing more we can do here,
+                # move onto the next submission retry.
+                for id_key in id_keys:
+                    self.unset_waiting_event_timer(id_key)
+                return
+        except PlatformLookupError:
+            log_platform_event(
+                'Unable to retrieve job logs.',
+                {'name': ctx.platform_name},
+                level='warning',
+            )
+            return
+
+        # construct the retrieval command
         ssh_str = str(platform["ssh command"])
         rsync_str = str(platform["retrieve job logs command"])
         cmd = shlex.split(rsync_str) + ["--rsh=" + ssh_str]
@@ -962,6 +998,8 @@ class TaskEventsManager():
         )
         # Local target
         cmd.append(get_workflow_run_job_dir(schd.workflow) + "/")
+
+        # schedule command
         self.proc_pool.put_command(
             SubProcContext(
                 ctx, cmd, env=dict(os.environ), id_keys=id_keys, host=host
@@ -1073,9 +1111,9 @@ class TaskEventsManager():
         if event_time is None:
             event_time = get_current_time_string()
         itask.set_summary_time('finished', event_time)
-        job_d = itask.tokens.duplicate(job=str(itask.submit_num)).relative_id
-        self.data_store_mgr.delta_job_time(job_d, 'finished', event_time)
-        self.data_store_mgr.delta_job_state(job_d, TASK_STATUS_FAILED)
+        job_tokens = itask.tokens.duplicate(job=str(itask.submit_num))
+        self.data_store_mgr.delta_job_time(job_tokens, 'finished', event_time)
+        self.data_store_mgr.delta_job_state(job_tokens, TASK_STATUS_FAILED)
         self.workflow_db_mgr.put_update_task_jobs(itask, {
             "run_status": 1,
             "time_run_exit": event_time,
@@ -1088,7 +1126,6 @@ class TaskEventsManager():
             if itask.state_reset(TASK_STATUS_FAILED):
                 self.setup_event_handlers(itask, self.EVENT_FAILED, message)
                 self.data_store_mgr.delta_task_state(itask)
-            LOG.critical(f"[{itask}] failed")
             no_retries = True
         else:
             # There is an execution retry lined up.
@@ -1106,9 +1143,9 @@ class TaskEventsManager():
         if itask.job_vacated:
             itask.job_vacated = False
             LOG.warning(f"[{itask}] Vacated job restarted")
-        job_d = itask.tokens.duplicate(job=str(itask.submit_num)).relative_id
-        self.data_store_mgr.delta_job_time(job_d, 'started', event_time)
-        self.data_store_mgr.delta_job_state(job_d, TASK_STATUS_RUNNING)
+        job_tokens = itask.tokens.duplicate(job=str(itask.submit_num))
+        self.data_store_mgr.delta_job_time(job_tokens, 'started', event_time)
+        self.data_store_mgr.delta_job_state(job_tokens, TASK_STATUS_RUNNING)
         itask.set_summary_time('started', event_time)
         self.workflow_db_mgr.put_update_task_jobs(itask, {
             "time_run": itask.summary['started_time_string']})
@@ -1124,9 +1161,10 @@ class TaskEventsManager():
 
     def _process_message_succeeded(self, itask, event_time):
         """Helper for process_message, handle a succeeded message."""
-        job_d = itask.tokens.duplicate(job=str(itask.submit_num)).relative_id
-        self.data_store_mgr.delta_job_time(job_d, 'finished', event_time)
-        self.data_store_mgr.delta_job_state(job_d, TASK_STATUS_SUCCEEDED)
+
+        job_tokens = itask.tokens.duplicate(job=str(itask.submit_num))
+        self.data_store_mgr.delta_job_time(job_tokens, 'finished', event_time)
+        self.data_store_mgr.delta_job_state(job_tokens, TASK_STATUS_SUCCEEDED)
         itask.set_summary_time('finished', event_time)
         self.workflow_db_mgr.put_update_task_jobs(itask, {
             "run_status": 0,
@@ -1179,15 +1217,20 @@ class TaskEventsManager():
             self.setup_event_handlers(itask, self.EVENT_SUBMIT_RETRY, msg)
 
         # Register newly submit-failed job with the database and datastore.
-        job_d = itask.tokens.duplicate(job=str(itask.submit_num)).relative_id
+        job_tokens = itask.tokens.duplicate(job=str(itask.submit_num))
         self._insert_task_job(itask, event_time, self.JOB_SUBMIT_FAIL_FLAG)
-        self.data_store_mgr.delta_job_state(job_d, TASK_STATUS_SUBMIT_FAILED)
+        self.data_store_mgr.delta_job_state(
+            job_tokens,
+            TASK_STATUS_SUBMIT_FAILED
+        )
 
         self._reset_job_timers(itask)
 
         return no_retries
 
-    def _process_message_submitted(self, itask, event_time, submit_num):
+    def _process_message_submitted(
+        self, itask: 'TaskProxy', event_time: str
+    ) -> None:
         """Helper for process_message, handle a submit-succeeded message."""
         with suppress(KeyError):
             summary = itask.summary
@@ -1228,13 +1271,24 @@ class TaskEventsManager():
         # Register the newly submitted job with the database and datastore.
         # Do after itask has changed state
         self._insert_task_job(itask, event_time, self.JOB_SUBMIT_SUCCESS_FLAG)
-        job_d = itask.tokens.duplicate(job=str(itask.submit_num)).relative_id
-        self.data_store_mgr.delta_job_time(job_d, 'submitted', event_time)
+        job_tokens = itask.tokens.duplicate(job=str(itask.submit_num))
+        self.data_store_mgr.delta_job_time(
+            job_tokens,
+            'submitted',
+            event_time,
+        )
         if itask.tdef.run_mode == 'simulation':
             # Simulate job started as well.
-            self.data_store_mgr.delta_job_time(job_d, 'started', event_time)
+            self.data_store_mgr.delta_job_time(
+                job_tokens,
+                'started',
+                event_time,
+            )
         else:
-            self.data_store_mgr.delta_job_state(job_d, TASK_STATUS_SUBMITTED)
+            self.data_store_mgr.delta_job_state(
+                job_tokens,
+                TASK_STATUS_SUBMITTED,
+            )
 
     def _insert_task_job(
         self,
@@ -1503,24 +1557,23 @@ class TaskEventsManager():
         if itask.state(TASK_STATUS_RUNNING):
             timeref = itask.summary['started_time']
             timeout_key = 'execution timeout'
+            # Actual timeout after all polling.
             timeout = self._get_events_conf(itask, timeout_key)
-            delays = list(self._get_workflow_platforms_conf(
-                itask, 'execution polling intervals'))
-            if itask.summary[self.KEY_EXECUTE_TIME_LIMIT]:
-                time_limit = itask.summary[self.KEY_EXECUTE_TIME_LIMIT]
-                time_limit_delays = itask.platform.get(
+            execution_polling_intervals = list(
+                self._get_workflow_platforms_conf(
+                    itask, 'execution polling intervals'))
+            if itask.summary[TaskEventsManager.KEY_EXECUTE_TIME_LIMIT]:
+                time_limit = itask.summary[
+                    TaskEventsManager.KEY_EXECUTE_TIME_LIMIT]
+                time_limit_polling_intervals = itask.platform.get(
                     'execution time limit polling intervals')
-                if sum(delays) > time_limit:
-                    # Remove excessive polling before time limit
-                    while sum(delays) > time_limit:
-                        del delays[-1]
-                else:
-                    # Fill up the gap before time limit
-                    if delays:
-                        size = int((time_limit - sum(delays)) / delays[-1])
-                        delays.extend([delays[-1]] * size)
-                time_limit_delays[0] += time_limit - sum(delays)
-                delays += time_limit_delays
+                delays = self.process_execution_polling_intervals(
+                    execution_polling_intervals,
+                    time_limit,
+                    time_limit_polling_intervals
+                )
+            else:
+                delays = execution_polling_intervals
         else:  # if itask.state.status == TASK_STATUS_SUBMITTED:
             timeref = itask.summary['submitted_time']
             timeout_key = 'submission timeout'
@@ -1535,7 +1588,7 @@ class TaskEventsManager():
             timeout_str = None
         itask.poll_timer = TaskActionTimer(ctx=ctx, delays=delays)
         # Log timeout and polling schedule
-        message = 'health: %s=%s' % (timeout_key, timeout_str)
+        message = f"health: {timeout_key}={timeout_str}"
         # Attempt to group identical consecutive delays as N*DELAY,...
         if itask.poll_timer.delays:
             items = []  # [(number of item - 1, item), ...]
@@ -1554,6 +1607,71 @@ class TaskEventsManager():
         # Set next poll time
         self.check_poll_time(itask)
 
+    @staticmethod
+    def process_execution_polling_intervals(
+        polling_intervals: List[float],
+        time_limit: float,
+        time_limit_polling_intervals: List[float]
+    ) -> List[float]:
+        """Create a list of polling times.
+
+        Args:
+            (execution) polling_intervals
+            (execution) time_limit
+            (execution) time_limit_polling_intervals
+
+        Examples:
+
+        >>> this = TaskEventsManager.process_execution_polling_intervals
+
+        # Basic example:
+        >>> this([40, 35], 100, [10])
+        [40, 35, 35, 10]
+
+        # Second 40 second delay gets lopped off the list because it's after
+        # the execution time limit:
+        >>> this([40, 40], 60, [10])
+        [40, 30, 10]
+
+        # Expand last item in exection polling intervals to fill the
+        # execution time limit:
+        >>> this([5, 20], 100, [10])
+        [5, 20, 20, 20, 20, 25, 10]
+
+        # There are no execution polling intervals set - polling starts
+        # at execution time limit:
+        >>> this([], 10, [5])
+        [15, 5]
+
+        # We have a list of execution time limit polling intervals,
+        >>> this([10], 25, [5, 6, 7, 8])
+        [10, 10, 10, 6, 7, 8]
+        """
+        delays = polling_intervals
+        if sum(delays) > time_limit:
+            # Remove execution polling which would overshoot the
+            # execution time limit:
+            while sum(delays) > time_limit:
+                del delays[-1]
+        elif delays:
+            # Repeat the last execution polling interval up to the execution
+            # time limit:
+            size = int((time_limit - sum(delays)) / delays[-1])
+            delays.extend([delays[-1]] * size)
+
+        # After the last delay before the execution time limit add the
+        # delay to get to the execution_time_limit
+        if len(time_limit_polling_intervals) == 1:
+            time_limit_polling_intervals.append(
+                time_limit_polling_intervals[0]
+            )
+        time_limit_polling_intervals[0] += time_limit - sum(delays)
+
+        # After the execution time limit poll at execution time limit polling
+        # intervals.
+        delays += time_limit_polling_intervals
+        return delays
+
     def add_event_timer(self, id_key, event_timer):
         """Add a new event timer.
 
