@@ -13,15 +13,15 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""Manage task jobs.
+"""Manage jobs.
 
 This module provides logic to:
 * Set up the directory structure on remote job hosts.
   * Copy workflow service files to remote job hosts for communication clients.
   * Clean up of service files on workflow shutdown.
-* Prepare task job files.
-* Prepare task jobs submission, and manage the callbacks.
-* Prepare task jobs poll/kill, and manage the callbacks.
+* Prepare job files.
+* Prepare jobs submission, and manage the callbacks.
+* Prepare jobs poll/kill, and manage the callbacks.
 """
 
 from contextlib import suppress
@@ -36,7 +36,7 @@ from logging import (
 )
 from shutil import rmtree
 from time import time
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Optional
 
 from cylc.flow import LOG
 from cylc.flow.job_runner_mgr import JobPollContext
@@ -58,8 +58,6 @@ from cylc.flow.parsec.util import (
 )
 from cylc.flow.pathutil import get_remote_workflow_run_job_dir
 from cylc.flow.platforms import (
-    HOST_REC_COMMAND,
-    PLATFORM_REC_COMMAND,
     get_host_from_platform,
     get_install_target_from_platform,
     get_localhost_install_target,
@@ -112,6 +110,7 @@ from cylc.flow.wallclock import (
     get_utc_mode
 )
 from cylc.flow.cfgspec.globalcfg import SYSPATH
+from cylc.flow.util import serialise
 
 if TYPE_CHECKING:
     from cylc.flow.task_proxy import TaskProxy
@@ -121,9 +120,9 @@ class TaskJobManager:
     """Manage task job submit, poll and kill.
 
     This class provides logic to:
-    * Submit task jobs.
-    * Poll task jobs.
-    * Kill task jobs.
+    * Submit jobs.
+    * Poll jobs.
+    * Kill jobs.
     * Set up the directory structure on job hosts.
     * Install workflow communicate client files on job hosts.
     * Remove workflow contact files on job hosts.
@@ -156,7 +155,7 @@ class TaskJobManager:
         self.bad_hosts = bad_hosts
         self.bad_hosts_to_clear = set()
         self.task_remote_mgr = TaskRemoteMgr(
-            workflow, proc_pool, self.bad_hosts)
+            workflow, proc_pool, self.bad_hosts, self.workflow_db_mgr)
 
     def check_task_jobs(self, workflow, task_pool):
         """Check submission and execution timeout and polling timers.
@@ -238,7 +237,6 @@ class TaskJobManager:
                 itask.submit_num += 1
                 itask.state_reset(TASK_STATUS_PREPARING)
                 self.data_store_mgr.delta_task_state(itask)
-                self.workflow_db_mgr.put_update_task_state(itask)
             prep_task = self._prep_submit_task_job(
                 workflow, itask, check_syntax=check_syntax)
             if prep_task:
@@ -310,7 +308,7 @@ class TaskJobManager:
                                 itask.tdef.rtconfig['platform'],
                                 bad_hosts=self.bad_hosts
                             )
-                        except NoPlatformsError:
+                        except PlatformLookupError:
                             pass
                         else:
                             # If were able to select a new platform;
@@ -370,7 +368,7 @@ class TaskJobManager:
                         self.data_store_mgr.delta_job_msg(
                             itask.tokens.duplicate(
                                 job=str(itask.submit_num)
-                            ).relative_id,
+                            ),
                             self.REMOTE_INIT_MSG,
                         )
                     continue
@@ -399,7 +397,7 @@ class TaskJobManager:
                         self.data_store_mgr.delta_job_msg(
                             itask.tokens.duplicate(
                                 job=str(itask.submit_num)
-                            ).relative_id,
+                            ),
                             self.REMOTE_INIT_MSG
                         )
                     continue
@@ -423,7 +421,7 @@ class TaskJobManager:
                     self.data_store_mgr.delta_job_msg(
                         itask.tokens.duplicate(
                             job=str(itask.submit_num)
-                        ).relative_id,
+                        ),
                         self.REMOTE_INIT_MSG,
                     )
                 continue
@@ -439,8 +437,9 @@ class TaskJobManager:
             done_tasks.extend(itasks)
             for itask in itasks:
                 # Log and persist
-                LOG.info(f"[{itask}] host={host}")
+                LOG.debug(f"[{itask}] host={host}")
                 self.workflow_db_mgr.put_insert_task_jobs(itask, {
+                    'flow_nums': serialise(itask.flow_nums),
                     'is_manual_submit': itask.is_manual_submit,
                     'try_num': itask.get_try_num(),
                     'time_submit': get_current_time_string(),
@@ -456,7 +455,7 @@ class TaskJobManager:
                     self.data_store_mgr.delta_job_msg(
                         itask.tokens.duplicate(
                             job=str(itask.submit_num)
-                        ).relative_id,
+                        ),
                         REMOTE_FILE_INSTALL_IN_PROGRESS
                     )
                 continue
@@ -523,9 +522,6 @@ class TaskJobManager:
                 cmd, [len(b) for b in itasks_batches])
 
             if remote_mode:
-                host = get_host_from_platform(
-                    platform, bad_hosts=self.task_remote_mgr.bad_hosts
-                )
                 cmd = construct_ssh_cmd(
                     cmd, platform, host
                 )
@@ -617,40 +613,6 @@ class TaskJobManager:
                 exc.filename = target
             raise exc
 
-    @staticmethod
-    def _get_job_scripts(itask, rtconfig):
-        """Return pre-script, script, post-script for a job."""
-        script = rtconfig['script']
-        pre_script = rtconfig['pre-script']
-        post_script = rtconfig['post-script']
-        if itask.tdef.workflow_polling_cfg:
-            # Automatic workflow state polling script
-            comstr = (
-                "cylc workflow-state "
-                + " --task=" + itask.tdef.workflow_polling_cfg['task']
-                + " --point=" + str(itask.point)
-            )
-            if LOG.isEnabledFor(DEBUG):
-                comstr += ' --debug'
-            for key, fmt in [
-                    ('user', ' --%s=%s'),
-                    ('host', ' --%s=%s'),
-                    ('interval', ' --%s=%d'),
-                    ('max-polls', ' --%s=%s'),
-                    ('run-dir', ' --%s=%s')]:
-                if rtconfig['workflow state polling'][key]:
-                    comstr += fmt % (key,
-                                     rtconfig['workflow state polling'][key])
-            if rtconfig['workflow state polling']['message']:
-                comstr += " --message='%s'" % (
-                    rtconfig['workflow state polling']['message'])
-            else:
-                comstr += (" --status=" +
-                           itask.tdef.workflow_polling_cfg['status'])
-            comstr += " " + itask.tdef.workflow_polling_cfg['workflow']
-            script = "echo " + comstr + "\n" + comstr
-        return pre_script, script, post_script
-
     @staticmethod
     def _job_cmd_out_callback(workflow, itask, cmd_ctx, line):
         """Callback on job command STDOUT/STDERR."""
@@ -666,11 +628,12 @@ class TaskJobManager:
             line = "%s %s" % (timestamp, content)
         job_activity_log = get_task_job_activity_log(
             workflow, itask.point, itask.tdef.name)
+        if not line.endswith("\n"):
+            line += "\n"
+        line = host + line
         try:
-            with open(os.path.expandvars(job_activity_log), "ab") as handle:
-                if not line.endswith("\n"):
-                    line += "\n"
-                handle.write((host + line).encode())
+            with open(os.path.expandvars(job_activity_log), "a") as handle:
+                handle.write(line)
         except IOError as exc:
             LOG.warning("%s: write failed\n%s" % (job_activity_log, exc))
             LOG.warning(f"[{itask}] {host}{line}")
@@ -699,7 +662,13 @@ class TaskJobManager:
 
     def _kill_task_job_callback_255(self, workflow, itask, cmd_ctx, line):
         """Helper for _kill_task_jobs_callback, on one task job."""
-        self.kill_task_jobs(workflow, [itask])
+        with suppress(NoHostsError):
+            # if there is another host to kill on, try again, otherwise fail
+            get_host_from_platform(
+                itask.platform,
+                bad_hosts=self.task_remote_mgr.bad_hosts
+            )
+            self.kill_task_jobs(workflow, [itask])
 
     def _kill_task_job_callback(self, workflow, itask, cmd_ctx, line):
         """Helper for _kill_task_jobs_callback, on one task job."""
@@ -736,7 +705,7 @@ class TaskJobManager:
         self.data_store_mgr.delta_job_msg(
             itask.tokens.duplicate(
                 job=str(itask.submit_num)
-            ).relative_id,
+            ),
             log_msg
         )
         LOG.log(log_lvl, f"[{itask}] {log_msg}")
@@ -767,7 +736,10 @@ class TaskJobManager:
         tasks = {}
         for itask in itasks:
             while itask.reload_successor is not None:
+                # Note submit number could be incremented since reload.
+                subnum = itask.submit_num
                 itask = itask.reload_successor
+                itask.submit_num = subnum
             if itask.point is not None and itask.submit_num:
                 submit_num = "%02d" % (itask.submit_num)
                 tasks[(str(itask.point), itask.tdef.name, submit_num)] = itask
@@ -824,7 +796,13 @@ class TaskJobManager:
              self._poll_task_job_message_callback})
 
     def _poll_task_job_callback_255(self, workflow, itask, cmd_ctx, line):
-        self.poll_task_jobs(workflow, [itask])
+        with suppress(NoHostsError):
+            # if there is another host to poll on, try again, otherwise fail
+            get_host_from_platform(
+                itask.platform,
+                bad_hosts=self.task_remote_mgr.bad_hosts
+            )
+            self.poll_task_jobs(workflow, [itask])
 
     def _poll_task_job_callback(self, workflow, itask, cmd_ctx, line):
         """Helper for _poll_task_jobs_callback, on one task job."""
@@ -832,33 +810,37 @@ class TaskJobManager:
         ctx.out = line
         ctx.ret_code = 0
         # See cylc.flow.job_runner_mgr.JobPollContext
-        job_d = itask.tokens.duplicate(job=str(itask.submit_num)).relative_id
+        job_tokens = itask.tokens.duplicate(job=str(itask.submit_num))
         try:
             job_log_dir, context = line.split('|')[1:3]
             items = json.loads(context)
             jp_ctx = JobPollContext(job_log_dir, **items)
         except TypeError:
-            self.data_store_mgr.delta_job_msg(job_d, self.POLL_FAIL)
+            self.data_store_mgr.delta_job_msg(job_tokens, self.POLL_FAIL)
             ctx.cmd = cmd_ctx.cmd  # print original command on failure
             return
         except ValueError:
-            self.data_store_mgr.delta_job_msg(job_d, self.POLL_FAIL)
+            self.data_store_mgr.delta_job_msg(job_tokens, self.POLL_FAIL)
             ctx.cmd = cmd_ctx.cmd  # print original command on failure
             return
         finally:
             log_task_job_activity(ctx, workflow, itask.point, itask.tdef.name)
 
         flag = self.task_events_mgr.FLAG_POLLED
+        # Only log at INFO level if manually polling
+        log_lvl = DEBUG if (
+            itask.platform.get('communication method') == 'poll'
+        ) else INFO
         if jp_ctx.run_status == 1 and jp_ctx.run_signal in ["ERR", "EXIT"]:
             # Failed normally
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_FAILED, jp_ctx.time_run_exit, flag)
+                itask, log_lvl, TASK_OUTPUT_FAILED, jp_ctx.time_run_exit, flag)
         elif jp_ctx.run_status == 1 and jp_ctx.job_runner_exit_polled == 1:
             # Failed by a signal, and no longer in job runner
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_FAILED, jp_ctx.time_run_exit, flag)
+                itask, log_lvl, TASK_OUTPUT_FAILED, jp_ctx.time_run_exit, flag)
             self.task_events_mgr.process_message(
-                itask, INFO, FAIL_MESSAGE_PREFIX + jp_ctx.run_signal,
+                itask, log_lvl, FAIL_MESSAGE_PREFIX + jp_ctx.run_signal,
                 jp_ctx.time_run_exit,
                 flag)
         elif jp_ctx.run_status == 1:  # noqa: SIM114
@@ -866,30 +848,30 @@ class TaskJobManager:
             # Some job runners may restart a job in this state, so don't
             # mark as failed yet.
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_STARTED, jp_ctx.time_run, flag)
+                itask, log_lvl, TASK_OUTPUT_STARTED, jp_ctx.time_run, flag)
         elif jp_ctx.run_status == 0:
             # The job succeeded
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_SUCCEEDED, jp_ctx.time_run_exit,
+                itask, log_lvl, TASK_OUTPUT_SUCCEEDED, jp_ctx.time_run_exit,
                 flag)
         elif jp_ctx.time_run and jp_ctx.job_runner_exit_polled == 1:
             # The job has terminated without executing the error trap
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_FAILED, get_current_time_string(),
+                itask, log_lvl, TASK_OUTPUT_FAILED, get_current_time_string(),
                 flag)
         elif jp_ctx.time_run:
             # The job has started, and is still managed by job runner
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_STARTED, jp_ctx.time_run, flag)
+                itask, log_lvl, TASK_OUTPUT_STARTED, jp_ctx.time_run, flag)
         elif jp_ctx.job_runner_exit_polled == 1:
             # The job never ran, and no longer in job runner
             self.task_events_mgr.process_message(
-                itask, INFO, self.task_events_mgr.EVENT_SUBMIT_FAILED,
+                itask, log_lvl, self.task_events_mgr.EVENT_SUBMIT_FAILED,
                 jp_ctx.time_submit_exit, flag)
         else:
             # The job never ran, and is in job runner
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_STATUS_SUBMITTED, jp_ctx.time_submit_exit,
+                itask, log_lvl, TASK_STATUS_SUBMITTED, jp_ctx.time_submit_exit,
                 flag)
 
     def _poll_task_job_message_callback(self, workflow, itask, cmd_ctx, line):
@@ -929,7 +911,19 @@ class TaskJobManager:
 
         # Go through each list of itasks and carry out commands as required.
         for platform_name, itasks in sorted(auth_itasks.items()):
-            platform = get_platform(platform_name)
+            try:
+                platform = get_platform(platform_name)
+            except NoPlatformsError:
+                LOG.error(
+                    f'Unable to run command {cmd_key}: Unable to find'
+                    f' platform {platform_name} with accessible hosts.'
+                )
+            except PlatformLookupError:
+                LOG.error(
+                    f'Unable to run command {cmd_key}: Unable to find'
+                    f' platform {platform_name}.'
+                )
+                continue
             if is_remote_platform(platform):
                 remote_mode = True
                 cmd = [cmd_key]
@@ -942,13 +936,23 @@ class TaskJobManager:
             cmd.append(get_remote_workflow_run_job_dir(workflow))
             job_log_dirs = []
             host = 'localhost'
+
+            ctx = SubProcContext(cmd_key, cmd, host=host)
             if remote_mode:
-                host = get_host_from_platform(
-                    platform, bad_hosts=self.task_remote_mgr.bad_hosts
-                )
-                cmd = construct_ssh_cmd(
-                    cmd, platform, host
-                )
+                try:
+                    host = get_host_from_platform(
+                        platform, bad_hosts=self.task_remote_mgr.bad_hosts
+                    )
+                    cmd = construct_ssh_cmd(
+                        cmd, platform, host
+                    )
+                except NoHostsError:
+                    ctx.err = f'No available hosts for {platform["name"]}'
+                    callback_255(ctx, workflow, itasks)
+                    continue
+                else:
+                    ctx = SubProcContext(cmd_key, cmd, host=host)
+
             for itask in sorted(itasks, key=lambda task: task.identity):
                 job_log_dirs.append(
                     itask.tokens.duplicate(
@@ -958,9 +962,7 @@ class TaskJobManager:
             cmd += job_log_dirs
             LOG.debug(f'{cmd_key} for {platform["name"]} on {host}')
             self.proc_pool.put_command(
-                SubProcContext(
-                    cmd_key, cmd, host=host
-                ),
+                ctx,
                 bad_hosts=self.task_remote_mgr.bad_hosts,
                 callback=callback,
                 callback_args=[workflow, itasks],
@@ -968,38 +970,29 @@ class TaskJobManager:
             )
 
     @staticmethod
-    def _set_retry_timers(itask, rtconfig=None, retry=True):
+    def _set_retry_timers(
+        itask: 'TaskProxy',
+        rtconfig: Optional[dict] = None
+    ) -> None:
         """Set try number and retry delays."""
         if rtconfig is None:
             rtconfig = itask.tdef.rtconfig
-        if (
-            itask.tdef.run_mode + ' mode' in rtconfig and
-            'disable retries' in rtconfig[itask.tdef.run_mode + ' mode']
-        ):
-            retry = False
 
-        if retry:
-            if rtconfig['submission retry delays']:
-                submit_delays = rtconfig['submission retry delays']
-            else:
-                submit_delays = itask.platform['submission retry delays']
+        submit_delays = (
+            rtconfig['submission retry delays']
+            or itask.platform['submission retry delays']
+        )
 
-            for key, delays in [
-                    (
-                        TimerFlags.SUBMISSION_RETRY,
-                        submit_delays
-                    ),
-                    (
-                        TimerFlags.EXECUTION_RETRY,
-                        rtconfig['execution retry delays']
-                    )
-            ]:
-                if delays is None:
-                    delays = []
-                try:
-                    itask.try_timers[key].set_delays(delays)
-                except KeyError:
-                    itask.try_timers[key] = TaskActionTimer(delays=delays)
+        for key, delays in [
+            (TimerFlags.SUBMISSION_RETRY, submit_delays),
+            (TimerFlags.EXECUTION_RETRY, rtconfig['execution retry delays'])
+        ]:
+            if delays is None:
+                delays = []
+            try:
+                itask.try_timers[key].set_delays(delays)
+            except KeyError:
+                itask.try_timers[key] = TaskActionTimer(delays=delays)
 
     def _simulation_submit_task_jobs(self, itasks, workflow):
         """Simulation mode task jobs submission."""
@@ -1077,7 +1070,7 @@ class TaskJobManager:
             itask.summary['submit_method_id'] = None
         if itask.summary['submit_method_id'] and ctx.ret_code == 0:
             self.task_events_mgr.process_message(
-                itask, INFO, TASK_OUTPUT_SUBMITTED, ctx.timestamp)
+                itask, DEBUG, TASK_OUTPUT_SUBMITTED, ctx.timestamp)
         else:
             self.task_events_mgr.process_message(
                 itask, CRITICAL, self.task_events_mgr.EVENT_SUBMIT_FAILED,
@@ -1102,7 +1095,7 @@ class TaskJobManager:
 
         # Handle broadcasts
         overrides = self.task_events_mgr.broadcast_mgr.get_broadcast(
-            itask.identity
+            itask.tokens
         )
         if overrides:
             rtconfig = pdeepcopy(itask.tdef.rtconfig)
@@ -1138,12 +1131,12 @@ class TaskJobManager:
         host_n, platform_name = None, None
         try:
             if rtconfig['remote']['host'] is not None:
-                host_n = self.task_remote_mgr.subshell_eval(
-                    rtconfig['remote']['host'], HOST_REC_COMMAND
+                host_n = self.task_remote_mgr.eval_host(
+                    rtconfig['remote']['host']
                 )
             else:
-                platform_name = self.task_remote_mgr.subshell_eval(
-                    rtconfig['platform'], PLATFORM_REC_COMMAND
+                platform_name = self.task_remote_mgr.eval_platform(
+                    rtconfig['platform']
                 )
         except PlatformError as exc:
             itask.waiting_on_job_prep = False
@@ -1183,13 +1176,11 @@ class TaskJobManager:
                 platform = get_platform(
                     rtconfig, itask.tdef.name, bad_hosts=self.bad_hosts
                 )
-
             except PlatformLookupError as exc:
                 itask.waiting_on_job_prep = False
                 itask.summary['platforms_used'][itask.submit_num] = ''
                 # Retry delays, needed for the try_num
                 self._create_job_log_path(workflow, itask)
-                self._set_retry_timers(itask, rtconfig, False)
                 self._prep_submit_task_job_error(
                     workflow, itask, '(platform not defined)', exc)
                 return False
@@ -1252,6 +1243,7 @@ class TaskJobManager:
         self.workflow_db_mgr.put_insert_task_jobs(
             itask,
             {
+                'flow_nums': serialise(itask.flow_nums),
                 'job_id': itask.summary.get('submit_method_id'),
                 'is_manual_submit': itask.is_manual_submit,
                 'try_num': itask.get_try_num(),
@@ -1301,10 +1293,10 @@ class TaskJobManager:
 
         Note that rtconfig should have any broadcasts applied.
         """
-        scripts = self._get_job_scripts(itask, rtconfig)
         return {
             # NOTE: these fields should match get_simulation_job_conf
-            # TODO: turn this into a namedtuple or similar
+            # TODO: formalise this
+            # https://github.com/cylc/cylc-flow/issues/5387
             'job_runner_name': itask.platform['job runner'],
             'job_runner_command_template': (
                 itask.platform['job runner command template']
@@ -1324,15 +1316,15 @@ class TaskJobManager:
             'job_d': job_d,
             'namespace_hierarchy': itask.tdef.namespace_hierarchy,
             'param_var': itask.tdef.param_var,
-            'post-script': scripts[2],
-            'pre-script': scripts[0],
-            'script': scripts[1],
+            'post-script': rtconfig['post-script'],
+            'pre-script': rtconfig['pre-script'],
+            'script': rtconfig['script'],
             'submit_num': itask.submit_num,
             'flow_nums': itask.flow_nums,
             'workflow_name': workflow,
             'task_id': itask.identity,
             'try_num': itask.get_try_num(),
-            'uuid_str': self.task_remote_mgr.uuid_str,
+            'uuid_str': self.task_events_mgr.uuid_str,
             'work_d': rtconfig['work sub-directory'],
             # this field is populated retrospectively for regular job subs
             'logfiles': [],
@@ -1365,7 +1357,7 @@ class TaskJobManager:
             'workflow_name': workflow,
             'task_id': itask.identity,
             'try_num': itask.get_try_num(),
-            'uuid_str': self.task_remote_mgr.uuid_str,
+            'uuid_str': self.task_events_mgr.uuid_str,
             'work_d': 'SIMULATION',
             # this field is populated retrospectively for regular job subs
             'logfiles': [],
