@@ -19,7 +19,6 @@
 from collections import Counter
 from copy import copy
 from fnmatch import fnmatchcase
-from time import time
 from typing import (
     Any, Callable, Dict, List, Set, Tuple, Optional, TYPE_CHECKING
 )
@@ -27,7 +26,6 @@ from typing import (
 from metomi.isodatetime.timezone import get_local_time_zone
 
 from cylc.flow import LOG
-from cylc.flow.id import Tokens
 from cylc.flow.platforms import get_platform
 from cylc.flow.task_action_timer import TimerFlags
 from cylc.flow.task_state import TaskState, TASK_STATUS_WAITING
@@ -40,6 +38,7 @@ from cylc.flow.cycling.iso8601 import (
 )
 
 if TYPE_CHECKING:
+    from cylc.flow.id import Tokens
     from cylc.flow.cycling import PointBase
     from cylc.flow.task_action_timer import TaskActionTimer
     from cylc.flow.taskdef import TaskDef
@@ -49,9 +48,9 @@ class TaskProxy:
     """Represent an instance of a cycling task in a running workflow.
 
     Attributes:
-        .clock_trigger_time:
-            Clock trigger time in seconds since epoch.
-            (Used for both old-style clock triggers and wall_clock xtrigger).
+        .clock_trigger_times:
+            Memoization of clock trigger times (Used for wall_clock xtrigger):
+            {offset string: seconds from epoch}
         .expire_time:
             Time in seconds since epoch when this task is considered expired.
         .identity:
@@ -150,11 +149,12 @@ class TaskProxy:
         is_held: True if the task is held, else False.
         submit_num: Number of times the task has attempted job submission.
         is_late: Is the task late?
+        data_mode: Reduced store reference data.
     """
 
     # Memory optimization - constrain possible attributes to this list.
     __slots__ = [
-        'clock_trigger_time',
+        'clock_trigger_times',
         'expire_time',
         'identity',
         'is_late',
@@ -184,6 +184,7 @@ class TaskProxy:
 
     def __init__(
         self,
+        scheduler_tokens: 'Tokens',
         tdef: 'TaskDef',
         start_point: 'PointBase',
         flow_nums: Optional[Set[int]] = None,
@@ -193,6 +194,7 @@ class TaskProxy:
         is_late: bool = False,
         is_manual_submit: bool = False,
         flow_wait: bool = False,
+        data_mode: bool = False,
     ) -> None:
 
         self.tdef = tdef
@@ -207,8 +209,7 @@ class TaskProxy:
             self.flow_nums = copy(flow_nums)
         self.flow_wait = flow_wait
         self.point = start_point
-        self.tokens = Tokens(
-            # TODO: make these absolute?
+        self.tokens = scheduler_tokens.duplicate(
             cycle=str(self.point),
             task=self.tdef.name,
         )
@@ -235,7 +236,10 @@ class TaskProxy:
 
         self.local_job_file_path: Optional[str] = None
 
-        self.platform = get_platform()
+        if data_mode:
+            self.platform = {}
+        else:
+            self.platform = get_platform()
 
         self.job_vacated = False
         self.poll_timer: Optional['TaskActionTimer'] = None
@@ -243,7 +247,7 @@ class TaskProxy:
         self.try_timers: Dict[str, 'TaskActionTimer'] = {}
         self.non_unique_events = Counter()  # type: ignore # TODO: figure out
 
-        self.clock_trigger_time: Optional[float] = None
+        self.clock_trigger_times: Dict[str, int] = {}
         self.expire_time: Optional[float] = None
         self.late_time: Optional[float] = None
         self.is_late = is_late
@@ -252,7 +256,10 @@ class TaskProxy:
         self.state = TaskState(tdef, self.point, status, is_held)
 
         # Determine graph children of this task (for spawning).
-        self.graph_children = generate_graph_children(tdef, self.point)
+        if data_mode:
+            self.graph_children = {}
+        else:
+            self.graph_children = generate_graph_children(tdef, self.point)
 
     def __repr__(self) -> str:
         return f"<{self.__class__.__name__} '{self.tokens}'>"
@@ -266,7 +273,7 @@ class TaskProxy:
             f" flows:{','.join(str(i) for i in self.flow_nums) or 'none'}"
         )
 
-    def copy_to_reload_successor(self, reload_successor):
+    def copy_to_reload_successor(self, reload_successor, check_output):
         """Copy attributes to successor on reload of this task proxy."""
         self.reload_successor = reload_successor
         reload_successor.submit_num = self.submit_num
@@ -283,7 +290,34 @@ class TaskProxy:
         reload_successor.state.is_held = self.state.is_held
         reload_successor.state.is_runahead = self.state.is_runahead
         reload_successor.state.is_updated = self.state.is_updated
-        reload_successor.state.prerequisites = self.state.prerequisites
+
+        # Prerequisites: the graph might have changed before reload, so
+        # we need to use the new prerequisites but update them with the
+        # pre-reload state of prerequisites that still exist post-reload.
+
+        # Get all prereq states, e.g. {('1', 'c', 'succeeded'): False, ...}
+        pre_reload = {
+            k: v
+            for pre in self.state.prerequisites
+            for (k, v) in pre.satisfied.items()
+        }
+        # Use them to update the new prerequisites.
+        # - unchanged prerequisites will keep their pre-reload state.
+        # - removed prerequisites will not be carried over
+        # - added prerequisites will be recorded as unsatisfied
+        #   NOTE: even if the corresponding output was completed pre-reload!
+        for pre in reload_successor.state.prerequisites:
+            for k in pre.satisfied.keys():
+                try:
+                    pre.satisfied[k] = pre_reload[k]
+                except KeyError:
+                    # Look through task outputs to see if is has been
+                    # satisfied
+                    pre.satisfied[k] = check_output(
+                        *k,
+                        self.flow_nums,
+                    )
+
         reload_successor.state.xtriggers.update({
             # copy across any special "_cylc" xtriggers which were added
             # dynamically at runtime (i.e. execution retry xtriggers)
@@ -324,25 +358,31 @@ class TaskProxy:
                 self.point_as_seconds += utc_offset_in_seconds
         return self.point_as_seconds
 
-    def get_clock_trigger_time(self, offset_str):
-        """Compute, cache, and return trigger time relative to cycle point.
+    def get_clock_trigger_time(
+        self,
+        point: 'PointBase', offset_str: Optional[str] = None
+    ) -> int:
+        """Compute, cache and return trigger time relative to cycle point.
 
         Args:
-            offset_str: ISO8601Interval string, e.g. "PT2M".
-                        Can be None for zero offset.
+            point: Task's cycle point.
+            offset_str: ISO8601 interval string, e.g. "PT2M".
+                Can be None for zero offset.
         Returns:
             Absolute trigger time in seconds since Unix epoch.
 
         """
-        if self.clock_trigger_time is None:
-            if offset_str is None:
-                trigger_time = self.point
+        offset_str = offset_str if offset_str else 'P0Y'
+        if offset_str not in self.clock_trigger_times:
+            if offset_str == 'P0Y':
+                trigger_time = point
             else:
-                trigger_time = self.point + ISO8601Interval(offset_str)
-            self.clock_trigger_time = int(
-                point_parse(str(trigger_time)).seconds_since_unix_epoch
-            )
-        return self.clock_trigger_time
+                trigger_time = point + ISO8601Interval(offset_str)
+
+            offset = int(
+                point_parse(str(trigger_time)).seconds_since_unix_epoch)
+            self.clock_trigger_times[offset_str] = offset
+        return self.clock_trigger_times[offset_str]
 
     def get_try_num(self):
         """Return the number of automatic tries (try number)."""
@@ -359,7 +399,7 @@ class TaskProxy:
         """Is this task ready to run?
 
         Takes account of all dependence: on other tasks, xtriggers, and
-        old-style ext- and clock-triggers. Or, manual triggering.
+        old-style ext-triggers. Or, manual triggering.
 
         """
         if self.is_manual_submit:
@@ -373,7 +413,6 @@ class TaskProxy:
             return (self.try_timers[self.state.status].is_delay_done(),)
         return (
             self.state(TASK_STATUS_WAITING),
-            self.is_waiting_clock_done(),
             self.is_waiting_prereqs_done()
         )
 
@@ -388,18 +427,6 @@ class TaskProxy:
             self.summary[event_key + '_time'] = float(str2time(time_str))
         self.summary[event_key + '_time_string'] = time_str
 
-    def is_waiting_clock_done(self):
-        """Is this task done waiting for its old-style clock trigger time?
-
-        Return True if there is no clock trigger or when clock trigger is done.
-        """
-        if self.tdef.clocktrigger_offset is None:
-            return True
-        return (
-            time() >
-            self.get_clock_trigger_time(str(self.tdef.clocktrigger_offset))
-        )
-
     def is_task_prereqs_not_done(self):
         """Are some task prerequisites not satisfied?"""
         return (not all(pre.is_satisfied()
@@ -409,7 +436,7 @@ class TaskProxy:
         """Are ALL prerequisites satisfied?"""
         return (
             all(pre.is_satisfied() for pre in self.state.prerequisites)
-            and all(tri for tri in self.state.external_triggers.values())
+            and self.state.external_triggers_all_satisfied()
             and self.state.xtriggers_all_satisfied()
         )
 
