@@ -93,7 +93,7 @@ class TemplateVariables(Enum):
 
     .. deprecated:: 8.0.0
 
-       Deprecated, use ``workflow``.
+       Use ``workflow`` instead.
     """
 
     # BACK COMPAT: suite_name deprecated
@@ -108,7 +108,7 @@ class TemplateVariables(Enum):
 
     .. deprecated:: 8.0.0
 
-       Deprecated, use ``workflow``.
+       Use ``workflow`` instead.
     """
 
     # BACK COMPAT: suite_run_dir deprecated
@@ -123,7 +123,7 @@ class TemplateVariables(Enum):
 
     .. deprecated:: 8.0.0
 
-       Deprecated, use ``run_dir``.
+       Use ``workflow_run_dir`` instead.
     """
 
     # BACK COMPAT: suite_share_dir deprecated
@@ -138,7 +138,7 @@ class TemplateVariables(Enum):
 
     .. deprecated:: 8.0.0
 
-       Deprecated, use ``share_dir``.
+       Use ``workflow_share_dir`` instead.
     """
 
 
@@ -388,6 +388,7 @@ class XtriggerManager:
                 # External (clock xtrigger): convert offset to trigger_time.
                 # Datetime cycling only.
                 kwargs["trigger_time"] = itask.get_clock_trigger_time(
+                    itask.point,
                     ctx.func_kwargs["offset"]
                 )
             else:
@@ -436,7 +437,7 @@ class XtriggerManager:
                     for key, val in self.sat_xtrig[sig].items():
                         res["%s_%s" % (label, key)] = val
                     if res:
-                        xtrigger_env = [{'environment': {key: val}} for
+                        xtrigger_env = [{'environment': {key: str(val)}} for
                                         key, val in res.items()]
                         self.broadcast_mgr.put_broadcast(
                             [str(itask.point)],
