@@ -20,18 +20,25 @@ from functools import partial
 from pathlib import Path
 import pytest
 from shutil import rmtree
-from typing import List, TYPE_CHECKING, Tuple, Set
+from typing import List, TYPE_CHECKING, Set, Tuple, Union
 
 from cylc.flow.config import WorkflowConfig
+from cylc.flow.option_parsers import Options
 from cylc.flow.network.client import WorkflowRuntimeClient
 from cylc.flow.pathutil import get_cylc_run_dir
 from cylc.flow.rundb import CylcWorkflowDAO
 from cylc.flow.scripts.validate import ValidateOptions
+from cylc.flow.scripts.install import (
+    install as cylc_install,
+    get_option_parser as install_gop
+)
 from cylc.flow.wallclock import get_current_time_string
+from cylc.flow.workflow_files import infer_latest_run_from_id
 
 from .utils import _rm_if_empty
 from .utils.flow_tools import (
     _make_flow,
+    _make_src_flow,
     _make_scheduler,
     _run_flow,
     _start_flow,
@@ -42,6 +49,9 @@ if TYPE_CHECKING:
     from cylc.flow.task_proxy import TaskProxy
 
 
+InstallOpts = Options(install_gop())
+
+
 @pytest.hookimpl(tryfirst=True, hookwrapper=True)
 def pytest_runtest_makereport(item, call):
     """Expose the result of tests to their fixtures.
@@ -90,7 +100,7 @@ def run_dir():
 
 @pytest.fixture(scope='session')
 def ses_test_dir(request, run_dir):
-    """The root reg dir for test flows in this test session."""
+    """The root run dir for test flows in this test session."""
     timestamp = get_current_time_string(use_basic_format=True)
     uuid = f'cit-{timestamp}'
     path = Path(run_dir, uuid)
@@ -101,7 +111,7 @@ def ses_test_dir(request, run_dir):
 
 @pytest.fixture(scope='module')
 def mod_test_dir(request, ses_test_dir):
-    """The root reg dir for test flows in this test module."""
+    """The root run dir for test flows in this test module."""
     path = Path(ses_test_dir, request.module.__name__)
     path.mkdir(exist_ok=True)
     yield path
@@ -115,7 +125,7 @@ def mod_test_dir(request, ses_test_dir):
 
 @pytest.fixture
 def test_dir(request, mod_test_dir):
-    """The root reg dir for test flows in this test function."""
+    """The root run dir for test flows in this test function."""
     path = Path(mod_test_dir, request.function.__name__)
     path.mkdir(parents=True, exist_ok=True)
     yield path
@@ -154,7 +164,7 @@ def scheduler():
     """Return a Scheduler object for a flow.
 
     Args:
-        reg (str): Workflow name.
+        id_ (str): Workflow name.
         **opts (Any): Options to be passed to the Scheduler.
     """
     with _make_scheduler() as _scheduler:
@@ -215,15 +225,16 @@ def mod_one_conf():
 
 @pytest.fixture
 def one(one_conf, flow, scheduler):
-    reg = flow(one_conf)
-    schd = scheduler(reg)
+    """Return a Scheduler for the simple "R1 = one" graph."""
+    id_ = flow(one_conf)
+    schd = scheduler(id_)
     return schd
 
 
 @pytest.fixture(scope='module')
 def mod_one(mod_one_conf, mod_flow, mod_scheduler):
-    reg = mod_flow(mod_one_conf)
-    schd = mod_scheduler(reg)
+    id_ = mod_flow(mod_one_conf)
+    schd = mod_scheduler(id_)
     return schd
 
 
@@ -302,11 +313,8 @@ def db_select():
             stmt += f' WHERE {where_stmt}'
             stmt_args = list(where.values())
 
-        pri_dao = schd.workflow_db_mgr.get_pri_dao()
-        try:
+        with schd.workflow_db_mgr.get_pri_dao() as pri_dao:
             return list(pri_dao.connect().execute(stmt, stmt_args))
-        finally:
-            pri_dao.close()
 
     return _inner
 
@@ -334,13 +342,14 @@ def validate(run_dir):
     errors.
 
     Args:
-        reg - The flow to validate
+        id_ - The flow to validate
         kwargs - Arguments to pass to ValidateOptions
     """
-    def _validate(reg: str, **kwargs) -> None:
-        WorkflowConfig(
-            reg,
-            str(Path(run_dir, reg, 'flow.cylc')),
+    def _validate(id_: Union[str, Path], **kwargs) -> WorkflowConfig:
+        id_ = str(id_)
+        return WorkflowConfig(
+            id_,
+            str(Path(run_dir, id_, 'flow.cylc')),
             ValidateOptions(**kwargs)
         )
 
@@ -406,3 +415,61 @@ def capture_polling():
         return polled_tasks
 
     return _disable_polling
+
+
+@pytest.fixture(scope='module')
+def mod_workflow_source(mod_flow, tmp_path_factory):
+    """Create a workflow source directory.
+
+    Args:
+        cfg: Can be passed a config dictionary.
+
+    Yields:
+        Path to source directory.
+    """
+    def _inner(cfg):
+        src_dir = _make_src_flow(tmp_path_factory.getbasetemp(), cfg)
+        return src_dir
+    yield _inner
+
+
+@pytest.fixture
+def workflow_source(mod_flow, tmp_path):
+    """Create a workflow source directory.
+
+    Args:
+        cfg: Can be passed a config dictionary.
+
+    Yields:
+        Path to source directory.
+    """
+    def _inner(cfg):
+        src_dir = _make_src_flow(tmp_path, cfg)
+        return src_dir
+    yield _inner
+
+
+@pytest.fixture
+def install(test_dir, run_dir):
+    """Install a workflow from source
+
+    Args:
+        (Actually args for _inner, but what the fixture appears to take to
+        the user)
+        source: Directory containing the source.
+        **kwargs: Options for cylc install.
+
+    Returns:
+        Workflow id, including run directory.
+    """
+    def _inner(source, **kwargs):
+        opts = InstallOpts(**kwargs)
+        # Note we append the source.name to the string rather than creating
+        # a subfolder because the extra layer of directories would exceed
+        # Cylc install's default limit.
+        opts.workflow_name = (
+            f'{str(test_dir.relative_to(run_dir))}.{source.name}')
+        workflow_id, _ = cylc_install(opts, str(source))
+        workflow_id = infer_latest_run_from_id(workflow_id)
+        return workflow_id
+    yield _inner
