@@ -24,8 +24,8 @@ from cylc.flow.network.subscriber import (
 
 async def test_publisher(flow, scheduler, run, one_conf, port_range):
     """It should publish deltas when the flow starts."""
-    reg = flow(one_conf)
-    schd = scheduler(reg, paused_start=False)
+    id_ = flow(one_conf)
+    schd = scheduler(id_, paused_start=False)
     async with run(schd):
         # create a subscriber
         subscriber = WorkflowSubscriber(
