@@ -14,14 +14,18 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from typing import Callable
-import pytest
+import logging
+from typing import AsyncGenerator, Callable
 from unittest.mock import Mock
 
+import pytest
+
+from cylc.flow import CYLC_LOG
 from cylc.flow.data_store_mgr import EDGES, TASK_PROXIES
 from cylc.flow.id import Tokens
 from cylc.flow.network.resolvers import Resolvers
 from cylc.flow.scheduler import Scheduler
+from cylc.flow.workflow_status import StopMode
 
 
 @pytest.fixture
@@ -51,9 +55,9 @@ async def mock_flow(
     mod_flow: Callable[..., str],
     mod_scheduler: Callable[..., Scheduler],
     mod_start,
-) -> Scheduler:
+) -> AsyncGenerator[Scheduler, None]:
     ret = Mock()
-    ret.reg = mod_flow({
+    ret.id_ = mod_flow({
         'scheduler': {
             'allow implicit tasks': True
         },
@@ -66,7 +70,7 @@ async def mock_flow(
         }
     })
 
-    ret.schd = mod_scheduler(ret.reg, paused_start=True)
+    ret.schd = mod_scheduler(ret.id_, paused_start=True)
     async with mod_start(ret.schd):
         ret.schd.pool.release_runahead_tasks()
         ret.schd.data_store_mgr.initiate_data_model()
@@ -216,3 +220,41 @@ async def test_mutation_mapper(mock_flow):
     assert response is None
     with pytest.raises(ValueError):
         await mock_flow.resolvers._mutation_mapper('non_exist', {}, meta)
+
+
+@pytest.mark.asyncio
+async def test_stop(
+    one: Scheduler, run: Callable, log_filter: Callable,
+):
+    """Test the stop resolver."""
+    async with run(one) as log:
+        resolvers = Resolvers(
+            one.data_store_mgr,
+            schd=one
+        )
+        resolvers.stop(StopMode.REQUEST_CLEAN)
+        await one.process_command_queue()
+        assert log_filter(
+            log, level=logging.INFO, contains="Command actioned: stop"
+        )
+        assert one.stop_mode == StopMode.REQUEST_CLEAN
+
+
+async def test_command_logging(mock_flow, caplog):
+    """It should log the command, with user name if not owner."""
+    caplog.set_level(logging.INFO, logger=CYLC_LOG)
+    owner = mock_flow.owner
+    other = f"not-{mock_flow.owner}"
+
+    command = "stop"
+    mock_flow.resolvers._log_command(command, owner)
+    assert caplog.records[-1].msg == f"[command] {command}"
+    mock_flow.resolvers._log_command(command, other)
+    msg1 = f"[command] {command} (issued by {other})"
+    assert caplog.records[-1].msg == msg1
+
+    command = "put_messages"
+    mock_flow.resolvers._log_command(command, owner)
+    assert caplog.records[-1].msg == msg1  # (prev message, i.e. not logged).
+    mock_flow.resolvers._log_command(command, other)
+    assert caplog.records[-1].msg == f"[command] {command} (issued by {other})"
