@@ -170,10 +170,10 @@ async def test_format_tree(flows, run_dir, ses_test_dir, mod_test_dir):
     """It should dump results in an ascii tree format."""
     # one stopped flow
     opts = ScanOptions(states='running', format='tree')
-    flows = []
-    await main(opts, write=flows.append, scan_dir=mod_test_dir)
-    assert len(flows) == 1
-    lines = flows[0].splitlines()
+    workflows = []
+    await main(opts, write=workflows.append, scan_dir=mod_test_dir)
+    assert len(workflows) == 1
+    lines = workflows[0].splitlines()
     # this flow is hierarchically registered in the run dir already
     # it should be registered as <session test dir>/<module test dir>/<name>
     assert ses_test_dir.name in lines[0]
@@ -185,10 +185,10 @@ async def test_format_rich(flows, mod_test_dir):
     """It should print results in a long human-friendly format."""
     # one stopped flow (--colour-blind)
     opts = ScanOptions(states='running', format='rich', colour_blind=True)
-    flows = []
-    await main(opts, write=flows.append, scan_dir=mod_test_dir)
-    assert len(flows) == 1
-    lines = flows[0].splitlines()
+    workflows = []
+    await main(opts, write=workflows.append, scan_dir=mod_test_dir)
+    assert len(workflows) == 1
+    lines = workflows[0].splitlines()
 
     # test that the multi-line description was output correctly
     # with trailing lines indented correctly
@@ -226,10 +226,12 @@ async def test_format_rich(flows, mod_test_dir):
 
     # one stopped flow (--colour=always)
     opts = ScanOptions(states='running', format='rich')
-    flows = []
-    await main(opts, write=flows.append, scan_dir=mod_test_dir, color='always')
-    assert len(flows) == 1
-    lines = flows[0].splitlines()
+    workflows = []
+    await main(
+        opts, write=workflows.append, scan_dir=mod_test_dir, color='always'
+    )
+    assert len(workflows) == 1
+    lines = workflows[0].splitlines()
 
     # test that the state totals show one task running (colour mode)
     for line in lines:
@@ -249,9 +251,9 @@ async def test_scan_cleans_stuck_contact_files(
 ):
     """Ensure scan tidies up contact files from crashed flows."""
     # create a flow
-    reg = flow(one_conf, name='-crashed-')
-    schd = scheduler(reg)
-    srv_dir = Path(run_dir, reg, WorkflowFiles.Service.DIRNAME)
+    id_ = flow(one_conf, name='-crashed-')
+    schd = scheduler(id_)
+    srv_dir = Path(run_dir, id_, WorkflowFiles.Service.DIRNAME)
     tmp_dir = test_dir / 'srv'
     cont = srv_dir / WorkflowFiles.Service.CONTACT
 
@@ -263,13 +265,13 @@ async def test_scan_cleans_stuck_contact_files(
     rmtree(tmp_dir)
 
     # the old contact file check uses the CLI command that the flow was run
-    # with to check that whether the flow is running. Because this is an
+    # with to check whether the flow is running. Because this is an
     # integration test the process is the pytest process and it is still
     # running so we need to change the command so that Cylc sees the flow as
     # having crashed
-    contact_info = load_contact_file(reg)
+    contact_info = load_contact_file(id_)
     contact_info[ContactFileFields.COMMAND] += 'xyz'
-    dump_contact_file(reg, contact_info)
+    dump_contact_file(id_, contact_info)
 
     # make sure this flow shows for a regular filesystem-only scan
     opts = ScanOptions(states='running,paused', format='name')
@@ -289,3 +291,48 @@ async def test_scan_cleans_stuck_contact_files(
 
     # the contact file should have been removed by the scan
     assert not cont.exists()
+
+
+async def test_scan_fail_well_when_client_unreachable(
+    start,
+    scheduler,
+    flow,
+    one_conf,
+    run_dir,
+    test_dir,
+    caplog,
+):
+    """It handles WorkflowRuntimeClient.async_request raising a WorkflowStopped
+    elegently.
+    """
+    # create a flow
+    id_ = flow(one_conf, name='-crashed-')
+    schd = scheduler(id_)
+    srv_dir = Path(run_dir, id_, WorkflowFiles.Service.DIRNAME)
+    tmp_dir = test_dir / 'srv'
+
+    # run the flow, copy the contact, stop the flow, copy back the contact
+    async with start(schd):
+        copytree(srv_dir, tmp_dir)
+    rmtree(srv_dir)
+    copytree(tmp_dir, srv_dir)
+    rmtree(tmp_dir)
+
+    # the old contact file check uses the CLI command that the flow was run
+    # with to check whether the flow is running. Because this is an
+    # integration test the process is the pytest process and it is still
+    # running so we need to change the command so that Cylc sees the flow as
+    # having crashed
+    contact_info = load_contact_file(id_)
+    contact_info[ContactFileFields.COMMAND] += 'xyz'
+    dump_contact_file(id_, contact_info)
+
+    # Run Cylc Scan
+    opts = ScanOptions(states='all', format='rich', ping=True)
+    flows = []
+    await main(opts, write=flows.append, scan_dir=test_dir)
+
+    # Check that the records contain a message but not an error
+    rec = caplog.records[-1]
+    assert not rec.exc_text
+    assert 'Workflow not running' in rec.msg
