@@ -28,11 +28,13 @@ from cylc.flow.cycling.loader import (
     INTEGER_CYCLING_TYPE
 )
 from cylc.flow.data_store_mgr import DataStoreMgr
+from cylc.flow.install import (
+    link_runN,
+    unlink_runN,
+)
 from cylc.flow.scheduler import Scheduler
 from cylc.flow.workflow_files import (
     WorkflowFiles,
-    link_runN,
-    unlink_runN,
 )
 from cylc.flow.xtrigger_mgr import XtriggerManager
 
@@ -68,7 +70,7 @@ def _tmp_run_dir(tmp_path: Path, monkeypatch: pytest.MonkeyPatch):
     Adds the runN symlink automatically if the workflow ID ends with /run__.
 
     Args:
-        reg: Workflow name.
+        id_: Workflow name.
         installed: If True, make it look like the workflow was installed
             using cylc install (creates _cylc-install dir).
         named: If True and installed is True, the _cylc-install dir will
@@ -81,7 +83,7 @@ def _tmp_run_dir(tmp_path: Path, monkeypatch: pytest.MonkeyPatch):
         cylc_run_dir = tmp_run_dir()
     """
     def __tmp_run_dir(
-        reg: Optional[str] = None,
+        id_: Optional[str] = None,
         installed: bool = False,
         named: bool = False
     ) -> Path:
@@ -90,8 +92,8 @@ def _tmp_run_dir(tmp_path: Path, monkeypatch: pytest.MonkeyPatch):
         cylc_run_dir = tmp_path / 'cylc-run'
         cylc_run_dir.mkdir(exist_ok=True)
         monkeypatch.setattr('cylc.flow.pathutil._CYLC_RUN_DIR', cylc_run_dir)
-        if reg:
-            run_dir = cylc_run_dir.joinpath(reg)
+        if id_:
+            run_dir = cylc_run_dir.joinpath(id_)
             run_dir.mkdir(parents=True, exist_ok=True)
             (run_dir / WorkflowFiles.FLOW_FILE).touch(exist_ok=True)
             (run_dir / WorkflowFiles.Service.DIRNAME).mkdir(exist_ok=True)
@@ -100,8 +102,8 @@ def _tmp_run_dir(tmp_path: Path, monkeypatch: pytest.MonkeyPatch):
                 link_runN(run_dir)
             if installed:
                 if named:
-                    if len(Path(reg).parts) < 2:
-                        raise ValueError("Named run requires two-level reg")
+                    if len(Path(id_).parts) < 2:
+                        raise ValueError("Named run requires two-level id_")
                     (run_dir.parent / WorkflowFiles.Install.DIRNAME).mkdir(
                         exist_ok=True)
                 else:
@@ -123,7 +125,7 @@ def mod_tmp_run_dir(tmp_path_factory: pytest.TempPathFactory):
     """Module-scoped version of tmp_run_dir()"""
     tmp_path = tmp_path_factory.getbasetemp()
     with pytest.MonkeyPatch.context() as mp:
-        return _tmp_run_dir(tmp_path, mp)
+        yield _tmp_run_dir(tmp_path, mp)
 
 
 def _tmp_src_dir(tmp_path: Path):
@@ -180,7 +182,10 @@ def set_cycling_type(monkeypatch: pytest.MonkeyPatch):
         monkeypatch.setattr(
             'cylc.flow.cycling.loader.DefaultCycler', _DefaultCycler)
         if ctype == ISO8601_CYCLING_TYPE:
-            iso8601_init(time_zone=time_zone, custom_dump_format=dump_format)
+            monkeypatch.setattr(
+                'cylc.flow.cycling.iso8601.WorkflowSpecifics',
+                iso8601_init(time_zone=time_zone, custom_dump_format=dump_format)
+            )
     return _set_cycling_type
 
 
@@ -199,3 +204,11 @@ def xtrigger_mgr() -> XtriggerManager:
             create_autospec(Scheduler, workflow=workflow_name, owner=user)
         )
     )
+
+
+@pytest.fixture()
+def prevent_symlinking(monkeypatch: pytest.MonkeyPatch):
+    monkeypatch.setattr(
+        'cylc.flow.pathutil.make_symlink_dir',
+        lambda *_, **__: {}
+    )
