@@ -14,17 +14,26 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import tempfile
+from tempfile import NamedTemporaryFile
+from contextlib import suppress
 
+import os
 import pytest
+from pytest import param
+import sqlite3
+from types import SimpleNamespace
 
+from cylc.flow import __version__ as cylc_version
 from cylc.flow.parsec.exceptions import (
     FileParseError,
     IncludeFileNotFoundError,
     Jinja2Error,
+    ParsecError,
 )
 from cylc.flow.parsec.OrderedDict import OrderedDictWithDefaults
 from cylc.flow.parsec.fileparse import (
+    _prepend_old_templatevars,
+    _get_fpath_for_source,
     addict,
     addsect,
     multiline,
@@ -270,7 +279,7 @@ def test_multiline():
 
 
 def test_read_and_proc_no_template_engine():
-    with tempfile.NamedTemporaryFile() as tf:
+    with NamedTemporaryFile() as tf:
         fpath = tf.name
         template_vars = None
         viewcfg = {
@@ -297,7 +306,7 @@ def test_read_and_proc_no_template_engine():
 
 
 def test_inline():
-    with tempfile.NamedTemporaryFile() as tf:
+    with NamedTemporaryFile() as tf:
         fpath = tf.name
         template_vars = None
         viewcfg = {
@@ -305,7 +314,7 @@ def test_inline():
             'contin': False, 'inline': True,
             'mark': None, 'single': None, 'label': None
         }
-        with tempfile.NamedTemporaryFile() as include_file:
+        with NamedTemporaryFile() as include_file:
             include_file.write("c=d".encode())
             include_file.flush()
             tf.write(("a=b\n%include \"{0}\""
@@ -317,7 +326,7 @@ def test_inline():
 
 
 def test_inline_error():
-    with tempfile.NamedTemporaryFile() as tf:
+    with NamedTemporaryFile() as tf:
         fpath = tf.name
         template_vars = None
         viewcfg = {
@@ -334,7 +343,7 @@ def test_inline_error():
 
 
 def test_read_and_proc_jinja2():
-    with tempfile.NamedTemporaryFile() as tf:
+    with NamedTemporaryFile() as tf:
         fpath = tf.name
         template_vars = {
             'name': 'Cylc'
@@ -350,8 +359,44 @@ def test_read_and_proc_jinja2():
         assert r == ['a=Cylc']
 
 
+def test_read_and_proc_cwd(tmp_path):
+    """The template processor should be able to read workflow files.
+
+    This relies on moving to the config dir during file parsing.
+    """
+
+    sdir = tmp_path / "sub"
+    sdir.mkdir()
+
+    for sub in ["a", "b", "c"]:
+        (sdir / sub).touch()
+
+    viewcfg = {
+        'empy': False,
+        'jinja2': True,
+        'contin': False,
+        'inline': False
+    }
+
+    tmpf = tmp_path / "a.conf"
+
+    with open(tmpf, 'w') as tf:
+        tf.write(
+            '#!Jinja2'
+            '\n{% from "os" import listdir %}'
+            '\n{% for f in listdir("sub") %}'
+            '\n{{f}}'
+            '\n{% endfor %}'
+        )
+
+    with open(tmpf, 'r') as tf:
+        r = read_and_proc(fpath=tf.name, viewcfg=viewcfg)
+
+    assert sorted(r) == ['a', 'b', 'c']
+
+
 def test_read_and_proc_jinja2_error():
-    with tempfile.NamedTemporaryFile() as tf:
+    with NamedTemporaryFile() as tf:
         fpath = tf.name
         template_vars = {
             'name': 'Cylc'
@@ -372,7 +417,7 @@ def test_read_and_proc_jinja2_error():
 
 
 def test_read_and_proc_jinja2_error_missing_shebang():
-    with tempfile.NamedTemporaryFile() as tf:
+    with NamedTemporaryFile() as tf:
         fpath = tf.name
         template_vars = {
             'name': 'Cylc'
@@ -392,112 +437,107 @@ def test_read_and_proc_jinja2_error_missing_shebang():
 # --- originally we had a test for empy here, moved to test_empysupport
 
 def test_parse_keys_only_singleline():
-    with tempfile.NamedTemporaryFile() as of:
-        with tempfile.NamedTemporaryFile() as tf:
-            fpath = tf.name
-            template_vars = {
-                'name': 'Cylc'
-            }
-            tf.write("#!jinja2\na={{ name }}\n".encode())
-            tf.flush()
-            r = parse(fpath=fpath, output_fname=of.name,
-                      template_vars=template_vars)
-            expected = OrderedDictWithDefaults()
-            expected['a'] = 'Cylc'
-            assert r == expected
-            of.flush()
-            output_file_contents = of.read().decode()
-            assert output_file_contents == 'a=Cylc\n'
+    with NamedTemporaryFile() as of, NamedTemporaryFile() as tf:
+        fpath = tf.name
+        template_vars = {
+            'name': 'Cylc'
+        }
+        tf.write("#!jinja2\na={{ name }}\n".encode())
+        tf.flush()
+        r = parse(fpath=fpath, output_fname=of.name,
+                  template_vars=template_vars)
+        expected = OrderedDictWithDefaults()
+        expected['a'] = 'Cylc'
+        assert r == expected
+        of.flush()
+        output_file_contents = of.read().decode()
+        assert output_file_contents == 'a=Cylc\n'
 
 
 def test_parse_keys_only_multiline():
-    with tempfile.NamedTemporaryFile() as of:
-        with tempfile.NamedTemporaryFile() as tf:
-            fpath = tf.name
-            template_vars = {
-                'name': 'Cylc'
-            }
-            tf.write(
-                "#!jinja2\na='''value is \\\n{{ name }}'''\n".encode())
-            tf.flush()
-            r = parse(fpath=fpath, output_fname=of.name,
-                      template_vars=template_vars)
-            expected = OrderedDictWithDefaults()
-            expected['a'] = "'''value is Cylc'''"
-            assert r == expected
+    with NamedTemporaryFile() as of, NamedTemporaryFile() as tf:
+        fpath = tf.name
+        template_vars = {
+            'name': 'Cylc'
+        }
+        tf.write(
+            "#!jinja2\na='''value is \\\n{{ name }}'''\n".encode())
+        tf.flush()
+        r = parse(fpath=fpath, output_fname=of.name,
+                  template_vars=template_vars)
+        expected = OrderedDictWithDefaults()
+        expected['a'] = "'''value is Cylc'''"
+        assert r == expected
 
 
 def test_parse_invalid_line():
-    with tempfile.NamedTemporaryFile() as of:
-        with tempfile.NamedTemporaryFile() as tf:
-            fpath = tf.name
-            template_vars = {
-                'name': 'Cylc'
-            }
-            tf.write("#!jinja2\n{{ name }}\n".encode())
-            tf.flush()
-            with pytest.raises(FileParseError) as cm:
-                parse(fpath=fpath, output_fname=of.name,
-                      template_vars=template_vars)
-            exc = cm.value
-            assert exc.reason == 'Invalid line'
-            assert exc.line_num == 1
-            assert exc.line == 'Cylc'
+    with NamedTemporaryFile() as of, NamedTemporaryFile() as tf:
+        fpath = tf.name
+        template_vars = {
+            'name': 'Cylc'
+        }
+        tf.write("#!jinja2\n{{ name }}\n".encode())
+        tf.flush()
+        with pytest.raises(FileParseError) as cm:
+            parse(fpath=fpath, output_fname=of.name,
+                  template_vars=template_vars)
+        exc = cm.value
+        assert exc.reason == 'Invalid line'
+        assert exc.line_num == 1
+        assert exc.line == 'Cylc'
 
 
 def test_parse_comments():
-    with tempfile.NamedTemporaryFile() as of:
-        with tempfile.NamedTemporaryFile() as tf:
-            fpath = tf.name
-            template_vars = {
-                'name': 'Cylc'
-            }
-            tf.write("#!jinja2\na={{ name }}\n# comment!".encode())
-            tf.flush()
-            r = parse(fpath=fpath, output_fname=of.name,
-                      template_vars=template_vars)
-            expected = OrderedDictWithDefaults()
-            expected['a'] = 'Cylc'
-            assert r == expected
-            of.flush()
-            output_file_contents = of.read().decode()
-            assert output_file_contents == 'a=Cylc\n# comment!\n'
+    with NamedTemporaryFile() as of, NamedTemporaryFile() as tf:
+        fpath = tf.name
+        template_vars = {
+            'name': 'Cylc'
+        }
+        tf.write("#!jinja2\na={{ name }}\n# comment!".encode())
+        tf.flush()
+        r = parse(fpath=fpath, output_fname=of.name,
+                  template_vars=template_vars)
+        expected = OrderedDictWithDefaults()
+        expected['a'] = 'Cylc'
+        assert r == expected
+        of.flush()
+        output_file_contents = of.read().decode()
+        assert output_file_contents == 'a=Cylc\n# comment!\n'
 
 
 def test_parse_with_sections():
-    with tempfile.NamedTemporaryFile() as of:
-        with tempfile.NamedTemporaryFile() as tf:
-            fpath = tf.name
-            template_vars = {
-                'name': 'Cylc'
-            }
-            tf.write(("#!jinja2\n[section1]\n"
-                      "a={{ name }}\n# comment!\n"
-                      "[[subsection1]]\n"
-                      "[[subsection2]]\n"
-                      "[section2]").encode())
-            tf.flush()
-            r = parse(fpath=fpath, output_fname=of.name,
-                      template_vars=template_vars)
-            expected = OrderedDictWithDefaults()
-            expected['section1'] = OrderedDictWithDefaults()
-            expected['section1']['a'] = 'Cylc'
-            expected['section1']['subsection1'] = OrderedDictWithDefaults()
-            expected['section1']['subsection2'] = OrderedDictWithDefaults()
-            expected['section2'] = OrderedDictWithDefaults()
-            assert r == expected
-            of.flush()
-            output_file_contents = of.read().decode()
-            assert output_file_contents == (
-                '[section1]\na=Cylc\n# comment!\n'
-                '[[subsection1]]\n'
-                '[[subsection2]]\n'
-                '[section2]\n'
-            )
+    with NamedTemporaryFile() as of, NamedTemporaryFile() as tf:
+        fpath = tf.name
+        template_vars = {
+            'name': 'Cylc'
+        }
+        tf.write(("#!jinja2\n[section1]\n"
+                  "a={{ name }}\n# comment!\n"
+                  "[[subsection1]]\n"
+                  "[[subsection2]]\n"
+                  "[section2]").encode())
+        tf.flush()
+        r = parse(fpath=fpath, output_fname=of.name,
+                  template_vars=template_vars)
+        expected = OrderedDictWithDefaults()
+        expected['section1'] = OrderedDictWithDefaults()
+        expected['section1']['a'] = 'Cylc'
+        expected['section1']['subsection1'] = OrderedDictWithDefaults()
+        expected['section1']['subsection2'] = OrderedDictWithDefaults()
+        expected['section2'] = OrderedDictWithDefaults()
+        assert r == expected
+        of.flush()
+        output_file_contents = of.read().decode()
+        assert output_file_contents == (
+            '[section1]\na=Cylc\n# comment!\n'
+            '[[subsection1]]\n'
+            '[[subsection2]]\n'
+            '[section2]\n'
+        )
 
 
 def test_parse_with_sections_missing_bracket():
-    with tempfile.NamedTemporaryFile() as tf:
+    with NamedTemporaryFile() as tf:
         fpath = tf.name
         template_vars = {
             'name': 'Cylc'
@@ -514,27 +554,26 @@ def test_parse_with_sections_missing_bracket():
 
 
 def test_parse_with_sections_error_wrong_level():
-    with tempfile.NamedTemporaryFile() as of:
-        with tempfile.NamedTemporaryFile() as tf:
-            fpath = tf.name
-            template_vars = {
-                'name': 'Cylc'
-            }
-            tf.write(("#!jinja2\n[section1]\n"
-                      "a={{ name }}\n# comment!\n"
-                      "[[[subsection1]]]\n")  # expected [[]] instead!
-                     .encode())
-            tf.flush()
-            with pytest.raises(FileParseError) as cm:
-                parse(fpath=fpath, output_fname=of.name,
-                      template_vars=template_vars)
-            exc = cm.value
-            assert exc.line_num == 4
-            assert exc.line == '[[[subsection1]]]'
+    with NamedTemporaryFile() as of, NamedTemporaryFile() as tf:
+        fpath = tf.name
+        template_vars = {
+            'name': 'Cylc'
+        }
+        tf.write(("#!jinja2\n[section1]\n"
+                  "a={{ name }}\n# comment!\n"
+                  "[[[subsection1]]]\n")  # expected [[]] instead!
+                 .encode())
+        tf.flush()
+        with pytest.raises(FileParseError) as cm:
+            parse(fpath=fpath, output_fname=of.name,
+                  template_vars=template_vars)
+        exc = cm.value
+        assert exc.line_num == 4
+        assert exc.line == '[[[subsection1]]]'
 
 
 def test_unclosed_multiline():
-    with tempfile.NamedTemporaryFile() as tf:
+    with NamedTemporaryFile() as tf:
         fpath = tf.name
         template_vars = {
             'name': 'Cylc'
@@ -599,3 +638,119 @@ def test_unclosed_multiline():
 def test_merge_template_vars(caplog, expect, native_tvars, plugin_result, log):
     assert merge_template_vars(native_tvars, plugin_result) == expect
     assert [r.msg for r in caplog.records] == log
+
+
+@pytest.fixture
+def _mock_old_template_vars_db(tmp_path):
+    def _inner(create_srclink=True):
+        # Create a fake workflow dir:
+        (tmp_path / 'flow.cylc').touch()
+        (tmp_path / 'log').mkdir()
+        db_path = tmp_path / 'log/db'
+        db_path.touch()
+
+        # Set up a fake workflow database:
+        conn = sqlite3.connect(str(db_path))
+        conn.execute(
+            "CREATE TABLE workflow_template_vars"
+            "(key TEXT, value TEXT, PRIMARY KEY(key)) ;"
+        )
+        conn.execute(
+            "INSERT INTO workflow_template_vars VALUES"
+            "    ('Marius', '\"Consul\"')"
+        )
+        conn.execute(
+            "CREATE TABLE workflow_params"
+            "(key TEXT, value TEXT, PRIMARY KEY(key)) ;"
+        )
+        conn.execute(
+            "INSERT INTO workflow_params VALUES"
+            f"    ('cylc_version', '{cylc_version}')"
+        )
+        conn.commit()
+        conn.close()
+
+        # Simulate being an installed rundir by creating a sourcelink:
+        if create_srclink:
+            src = (tmp_path / 'src')
+            src.mkdir(exist_ok=True)
+            link = tmp_path.parent / '_cylc-install/source'
+            link.parent.mkdir(exist_ok=True)
+            with suppress(FileExistsError):
+                # We don't mind the link persisting.
+                os.symlink(src, link)
+        return tmp_path / 'flow.cylc'
+    yield _inner
+
+
+@pytest.mark.parametrize(
+    'expect, tvars',
+    [
+        param(
+            {'Marius': 'Consul', 'Julius': 'Pontifex'}, {'Julius': 'Pontifex'},
+            id='It adds a new key at the end'
+        ),
+        param(
+            {'Marius': 'Tribune'}, {'Marius': 'Tribune'},
+            id='It overrides an existing key'
+        ),
+    ]
+)
+def test__prepend_old_templatevars(_mock_old_template_vars_db, expect, tvars):
+    # Create a target for a source symlink
+    result = _prepend_old_templatevars(
+        _mock_old_template_vars_db(), tvars)
+    assert result == expect
+
+
+def test_get_fpath_for_source(tmp_path):
+    # Create rundir and srcdir:
+    srcdir = tmp_path / 'source'
+    rundir = tmp_path / 'run'
+    srcdir.mkdir()
+    rundir.mkdir()
+    (rundir / 'flow.cylc').touch()
+    (srcdir / 'flow.cylc').touch()
+
+    # Mock Options object:
+    opts = SimpleNamespace()
+
+    # It raises an error if source is not linked:
+    with pytest.raises(
+        ParsecError, match=f'Cannot validate {rundir} against source:'
+    ):
+        opts.against_source = True
+        _get_fpath_for_source(rundir / 'flow.cylc', opts)
+
+    # Create symlinks:
+    link = rundir / '_cylc-install/source'
+    link.parent.mkdir(exist_ok=True)
+    os.symlink(srcdir, link)
+
+    # It does nothing if opts.against_source is False:
+    opts.against_source = False
+    assert _get_fpath_for_source(
+        rundir / 'flow.cylc', opts) == rundir / 'flow.cylc'
+
+    # It gets source dir if opts.against_source is True:
+    opts.against_source = True
+    assert _get_fpath_for_source(
+        rundir / 'flow.cylc', opts) == str(srcdir / 'flow.cylc')
+
+def test_user_has_no_cwd(tmp_path):
+    """Test we can parse a config file even if cwd does not exist."""
+    cwd = tmp_path/"cwd"
+    os.mkdir(cwd)
+    os.chdir(cwd)
+    os.rmdir(cwd)
+    # (I am now located in a non-existent directory. Outrageous!)
+    with NamedTemporaryFile() as tf:
+        fpath = tf.name
+        tf.write(('''
+        [scheduling]
+            [[graph]]
+                R1 = "foo"
+        ''').encode())
+        tf.flush()
+        # Should not raise FileNotFoundError from os.getcwd():
+        parse(fpath=fpath, output_fname="")
