@@ -14,13 +14,15 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import os
 from optparse import Values
-from typing import Any, Callable, Dict, Optional, Tuple, Type
+from typing import Any, Callable, Dict, List, Optional, Tuple, Type
 from pathlib import Path
 import pytest
 import logging
 from types import SimpleNamespace
 from unittest.mock import Mock
+from contextlib import suppress
 
 from cylc.flow import CYLC_LOG
 from cylc.flow.config import WorkflowConfig
@@ -48,22 +50,31 @@ from cylc.flow.cycling.iso8601 import ISO8601Point
 Fixture = Any
 
 
-@pytest.fixture
-def tmp_flow_config(tmp_run_dir: Callable):
+def _tmp_flow_config(tmp_run_dir: Callable):
     """Create a temporary flow config file for use in init'ing WorkflowConfig.
 
     Args:
-        reg: Workflow name.
+        id_: Workflow name.
         config: The flow file content.
 
     Returns the path to the flow file.
     """
-    def _tmp_flow_config(reg: str, config: str) -> Path:
-        run_dir: Path = tmp_run_dir(reg)
+    def __tmp_flow_config(id_: str, config: str) -> Path:
+        run_dir: Path = tmp_run_dir(id_)
         flow_file = run_dir / WorkflowFiles.FLOW_FILE
         flow_file.write_text(config)
         return flow_file
-    return _tmp_flow_config
+    return __tmp_flow_config
+
+
+@pytest.fixture
+def tmp_flow_config(tmp_run_dir: Callable):
+    return _tmp_flow_config(tmp_run_dir)
+
+
+@pytest.fixture(scope='module')
+def mod_tmp_flow_config(mod_tmp_run_dir: Callable):
+    return _tmp_flow_config(mod_tmp_run_dir)
 
 
 class TestWorkflowConfig:
@@ -228,8 +239,8 @@ def test_family_inheritance_and_quotes(
                 hosts = localhost
         '''
     )
-    reg = 'test'
-    file_path = tmp_flow_config(reg, f'''
+    id_ = 'test'
+    file_path = tmp_flow_config(id_, f'''
         [scheduler]
             allow implicit tasks = True
         [task parameters]
@@ -248,7 +259,7 @@ def test_family_inheritance_and_quotes(
                 inherit = 'MAINFAM<major, minor>', {fam_txt}
     ''')
     config = WorkflowConfig(
-        reg, file_path, template_vars={}, options=Values()
+        id_, file_path, template_vars={}, options=Values()
     )
     assert ('goodbye_0_major1_minor10' in
             config.runtime['descendants']['MAINFAM_major1_minor10'])
@@ -330,6 +341,16 @@ def test_family_inheritance_and_quotes(
             (WorkflowConfigError, "does not meet the constraints"),
             id="Violated constraints"
         ),
+        pytest.param(
+            ISO8601_CYCLING_TYPE,
+            {
+                'initial cycle point': 'a',
+            },
+            None,
+            None,
+            (WorkflowConfigError, 'Invalid ISO 8601 date representation: a'),
+            id="invalid"
+        ),
     ]
 )
 def test_process_icp(
@@ -769,8 +790,8 @@ def test_stopcp_after_fcp(
     """Test that setting a stop after cycle point that is beyond the final
     cycle point is handled correctly."""
     caplog.set_level(logging.WARNING, CYLC_LOG)
-    reg = 'cassini'
-    flow_file: Path = tmp_flow_config(reg, f"""
+    id_ = 'cassini'
+    flow_file: Path = tmp_flow_config(id_, f"""
     [scheduler]
         allow implicit tasks = True
     [scheduling]
@@ -780,7 +801,7 @@ def test_stopcp_after_fcp(
         [[graph]]
             P1Y = huygens
     """)
-    cfg = WorkflowConfig(reg, flow_file, options=RunOptions(**opts))
+    cfg = WorkflowConfig(id_, flow_file, options=RunOptions(**opts))
     msg = "will have no effect as it is after the final cycle point"
     if warning_expected:
         assert msg in caplog.text
@@ -981,8 +1002,8 @@ def test_cycle_point_tz(caplog, monkeypatch):
 
 
 def test_rsync_includes_will_not_accept_sub_directories(tmp_flow_config):
-    reg = 'rsynctest'
-    flow_file = tmp_flow_config(reg, """
+    id_ = 'rsynctest'
+    flow_file = tmp_flow_config(id_, """
     [scheduling]
         initial cycle point = 2020-01-01
         [[dependencies]]
@@ -993,15 +1014,15 @@ def test_rsync_includes_will_not_accept_sub_directories(tmp_flow_config):
 
     with pytest.raises(WorkflowConfigError) as exc:
         WorkflowConfig(
-            workflow=reg, fpath=flow_file, options=Values()
+            workflow=id_, fpath=flow_file, options=Values()
         )
     assert "Directories can only be from the top level" in str(exc.value)
 
 
 def test_valid_rsync_includes_returns_correct_list(tmp_flow_config):
     """Test that the rsync includes in the correct """
-    reg = 'rsynctest'
-    flow_file = tmp_flow_config(reg, """
+    id_ = 'rsynctest'
+    flow_file = tmp_flow_config(id_, """
     [scheduling]
         initial cycle point = 2020-01-01
         [[dependencies]]
@@ -1012,7 +1033,7 @@ def test_valid_rsync_includes_returns_correct_list(tmp_flow_config):
     """)
 
     config = WorkflowConfig(
-        workflow=reg, fpath=flow_file, options=Values()
+        workflow=id_, fpath=flow_file, options=Values()
     )
 
     rsync_includes = WorkflowConfig.get_validated_rsync_includes(config)
@@ -1067,8 +1088,8 @@ def test_check_circular(opt, monkeypatch, caplog, tmp_flow_config):
     if opt:
         setattr(options, opt, True)
 
-    reg = 'circular'
-    flow_file = tmp_flow_config(reg, """
+    id_ = 'circular'
+    flow_file = tmp_flow_config(id_, """
     [scheduling]
         cycling mode = integer
         [[graph]]
@@ -1080,7 +1101,7 @@ def test_check_circular(opt, monkeypatch, caplog, tmp_flow_config):
 
     def WorkflowConfig__assert_err_raised():
         with pytest.raises(WorkflowConfigError) as exc:
-            WorkflowConfig(workflow=reg, fpath=flow_file, options=options)
+            WorkflowConfig(workflow=id_, fpath=flow_file, options=options)
         assert "circular edges detected" in str(exc.value)
 
     # ----- The actual test -----
@@ -1099,8 +1120,8 @@ def test_check_circular(opt, monkeypatch, caplog, tmp_flow_config):
 
 def test_undefined_custom_output(tmp_flow_config: Callable):
     """Test error on undefined custom output referenced in graph."""
-    reg = 'custom_out1'
-    flow_file = tmp_flow_config(reg, """
+    id_ = 'custom_out1'
+    flow_file = tmp_flow_config(id_, """
     [scheduling]
         [[graph]]
             R1 = "foo:x => bar"
@@ -1109,14 +1130,14 @@ def test_undefined_custom_output(tmp_flow_config: Callable):
     """)
 
     with pytest.raises(WorkflowConfigError) as cm:
-        WorkflowConfig(workflow=reg, fpath=flow_file, options=Values())
+        WorkflowConfig(workflow=id_, fpath=flow_file, options=Values())
     assert "Undefined custom output" in str(cm.value)
 
 
 def test_invalid_custom_output_msg(tmp_flow_config: Callable):
     """Test invalid output message (colon not allowed)."""
-    reg = 'invalid_output'
-    flow_file = tmp_flow_config(reg, """
+    id_ = 'invalid_output'
+    flow_file = tmp_flow_config(id_, """
     [scheduling]
         [[graph]]
             R1 = "foo:x => bar"
@@ -1129,9 +1150,9 @@ def test_invalid_custom_output_msg(tmp_flow_config: Callable):
 
     with pytest.raises(WorkflowConfigError) as cm:
         WorkflowConfig(
-            workflow=reg, fpath=flow_file, options=Values())
+            workflow=id_, fpath=flow_file, options=Values())
     assert (
-        'Invalid message trigger "[runtime][foo][outputs]x = '
+        'Invalid task message "[runtime][foo][outputs]x = '
         'the quick: brown fox"'
     ) in str(cm.value)
 
@@ -1144,8 +1165,8 @@ def test_c7_back_compat_optional_outputs(tmp_flow_config, monkeypatch):
 
     """
     monkeypatch.setattr('cylc.flow.flags.cylc7_back_compat', True)
-    reg = 'custom_out2'
-    flow_file = tmp_flow_config(reg, '''
+    id_ = 'custom_out2'
+    flow_file = tmp_flow_config(id_, '''
     [scheduling]
         [[graph]]
             R1 = """
@@ -1160,7 +1181,7 @@ def test_c7_back_compat_optional_outputs(tmp_flow_config, monkeypatch):
                 x = x
     ''')
 
-    cfg = WorkflowConfig(workflow=reg, fpath=flow_file, options=None)
+    cfg = WorkflowConfig(workflow=id_, fpath=flow_file, options=None)
 
     for taskdef in cfg.taskdefs.values():
         for output, (_, required) in taskdef.outputs.items():
@@ -1180,8 +1201,8 @@ def test_c7_back_compat_optional_outputs(tmp_flow_config, monkeypatch):
 )
 def test_implicit_success_required(tmp_flow_config, graph):
     """Check foo:succeed is required if success/fail not used in the graph."""
-    reg = 'blargh'
-    flow_file = tmp_flow_config(reg, f"""
+    id_ = 'blargh'
+    flow_file = tmp_flow_config(id_, f"""
     [scheduling]
         [[graph]]
             R1 = {graph}
@@ -1191,7 +1212,7 @@ def test_implicit_success_required(tmp_flow_config, graph):
            [[[outputs]]]
                x = "the quick brown fox"
     """)
-    cfg = WorkflowConfig(workflow=reg, fpath=flow_file, options=None)
+    cfg = WorkflowConfig(workflow=id_, fpath=flow_file, options=None)
     assert cfg.taskdefs['foo'].outputs[TASK_OUTPUT_SUCCEEDED][1]
 
 
@@ -1204,8 +1225,8 @@ def test_implicit_success_required(tmp_flow_config, graph):
 )
 def test_success_after_optional_submit(tmp_flow_config, graph):
     """Check foo:succeed is not required if foo:submit is optional."""
-    reg = 'blargh'
-    flow_file = tmp_flow_config(reg, f"""
+    id_ = 'blargh'
+    flow_file = tmp_flow_config(id_, f"""
     [scheduling]
         [[graph]]
             R1 = {graph}
@@ -1213,7 +1234,7 @@ def test_success_after_optional_submit(tmp_flow_config, graph):
         [[bar]]
         [[foo]]
     """)
-    cfg = WorkflowConfig(workflow=reg, fpath=flow_file, options=None)
+    cfg = WorkflowConfig(workflow=id_, fpath=flow_file, options=None)
     assert not cfg.taskdefs['foo'].outputs[TASK_OUTPUT_SUCCEEDED][1]
 
 
@@ -1270,8 +1291,8 @@ def test_implicit_tasks(
             implicit tasks in the err msg.
     """
     # Setup
-    reg = 'rincewind'
-    flow_file: Path = tmp_flow_config(reg, f"""
+    id_ = 'rincewind'
+    flow_file: Path = tmp_flow_config(id_, f"""
     [scheduler]
         {
             f'allow implicit tasks = {allow_implicit_tasks}'
@@ -1291,7 +1312,7 @@ def test_implicit_tasks(
         expected_exc = WorkflowConfigError
     extra_msg_expected &= (allow_implicit_tasks is None)
     # Test
-    args: dict = {'workflow': reg, 'fpath': flow_file, 'options': None}
+    args: dict = {'workflow': id_, 'fpath': flow_file, 'options': None}
     expected_msg = r"implicit tasks detected.*"
     if expected_exc:
         with pytest.raises(expected_exc, match=expected_msg) as excinfo:
@@ -1374,8 +1395,8 @@ def test_zero_interval(
 ):
     """Test that a zero-duration recurrence with >1 repetition gets an
     appropriate warning."""
-    reg = 'ordinary'
-    flow_file: Path = tmp_flow_config(reg, f"""
+    id_ = 'ordinary'
+    flow_file: Path = tmp_flow_config(id_, f"""
     [scheduler]
         UTC mode = True
         allow implicit tasks = True
@@ -1385,7 +1406,7 @@ def test_zero_interval(
         [[graph]]
             {recurrence} = slidescape36
     """)
-    WorkflowConfig(reg, flow_file, options=opts)
+    WorkflowConfig(id_, flow_file, options=opts)
     logged = log_filter(
         caplog,
         level=logging.WARNING,
@@ -1422,8 +1443,8 @@ def test_chain_expr(
 
     Note the order matters when "nominal" units (years, months) are used.
     """
-    reg = 'osgiliath'
-    flow_file: Path = tmp_flow_config(reg, f"""
+    id_ = 'osgiliath'
+    flow_file: Path = tmp_flow_config(id_, f"""
         [scheduler]
             UTC mode = True
             allow implicit tasks = True
@@ -1433,7 +1454,7 @@ def test_chain_expr(
             [[graph]]
                 P1D = faramir
     """)
-    cfg = WorkflowConfig(reg, flow_file, options=ValidateOptions())
+    cfg = WorkflowConfig(id_, flow_file, options=ValidateOptions())
     assert cfg.final_point == ISO8601Point(expected_fcp).standardise()
 
 
@@ -1481,3 +1502,169 @@ def test_check_for_owner(runtime_cfg):
     else:
         # Assert function doesn't raise if no owner set:
         assert WorkflowConfig.check_for_owner(runtime_cfg) is None
+
+
+@pytest.fixture(scope='module')
+def awe_config(mod_tmp_flow_config: Callable) -> WorkflowConfig:
+    """Return a workflow config object."""
+    id_ = 'awe'
+    flow_file = mod_tmp_flow_config(id_, '''
+        [scheduling]
+            cycling mode = integer
+            [[graph]]
+                P1 = ordinary & sterling
+                R1/2 = fra_mauro
+        [runtime]
+            [[USA, MOON]]
+            [[ordinary, sterling]]
+                inherit = USA
+            [[fra_mauro]]
+                inherit = MOON
+    ''')
+    return WorkflowConfig(
+        workflow=id_, fpath=flow_file, options=ValidateOptions()
+    )
+
+
+@pytest.mark.parametrize(
+    'name, expected',
+    [
+        pytest.param(
+            'ordinary', ['ordinary'], id="task name"
+        ),
+        pytest.param(
+            'USA', ['ordinary', 'sterling'], id="family name"
+        ),
+        pytest.param(
+            'fra*', ['fra_mauro'], id="glob task name"
+        ),
+        pytest.param(
+            'U*', ['ordinary', 'sterling'], id="glob family name"
+        ),
+        pytest.param(
+            '*', ['ordinary', 'sterling', 'fra_mauro'], id="glob everything"
+        ),
+        pytest.param(
+            'butte', [], id="no match"
+        ),
+    ]
+)
+def test_find_taskdefs(
+    name: str, expected: List[str], awe_config: WorkflowConfig
+):
+    assert sorted(
+        t.name for t in awe_config.find_taskdefs(name)
+    ) == sorted(expected)
+
+
+def test__warn_if_queues_have_implicit_tasks(caplog):
+    """It Warns that queues imply tasks undefined in runtime.
+    """
+    config = {
+        'scheduling': {'queues': {
+            'q1': {'members': ['foo']},
+            'q2': {'members': ['bar', 'baz']}
+        }},
+        'runtime': {}
+    }
+    taskdefs = {}
+    max_warning_lines = 2
+    WorkflowConfig._warn_if_queues_have_implicit_tasks(
+        config, taskdefs, max_warning_lines)
+    result = caplog.records[0].message
+    assert "'foo' in queue 'q1'" in result
+    assert "'bar' in queue 'q2'" in result
+    assert "'baz'" not in result
+    assert f"showing first {max_warning_lines}" in result
+
+
+@pytest.mark.parametrize(
+    'installed, run_dir, cylc_vars',
+    [
+        pytest.param(
+            False,  # not installed (parsing a source dir)
+            None,  # no run directory passed to config object by scheduler
+            {
+                'CYLC_WORKFLOW_NAME': True,  # expected environment variables
+                'CYLC_WORKFLOW_ID': False,
+                'CYLC_WORKFLOW_RUN_DIR': False,
+                'CYLC_WORKFLOW_WORK_DIR': False,
+                'CYLC_WORKFLOW_SHARE_DIR': False,
+                'CYLC_WORKFLOW_LOG_DIR': False,
+            },
+            id="source-dir"
+        ),
+        pytest.param(
+            True,
+            None,
+            {
+                'CYLC_WORKFLOW_NAME': True,
+                'CYLC_WORKFLOW_ID': True,
+                'CYLC_WORKFLOW_RUN_DIR': True,
+                'CYLC_WORKFLOW_WORK_DIR': False,
+                'CYLC_WORKFLOW_SHARE_DIR': False,
+                'CYLC_WORKFLOW_LOG_DIR': False,
+            },
+            id="run-dir"
+        ),
+        pytest.param(
+            True,
+            "/some/path",
+            {
+                'CYLC_WORKFLOW_NAME': True,
+                'CYLC_WORKFLOW_ID': True,
+                'CYLC_WORKFLOW_RUN_DIR': True,
+                'CYLC_WORKFLOW_WORK_DIR': True,
+                'CYLC_WORKFLOW_SHARE_DIR': True,
+                'CYLC_WORKFLOW_LOG_DIR': True,
+            },
+            id="run-dir-from-scheduler"
+        ),
+    ]
+)
+def test_cylc_env_at_parsing(
+    tmp_path: Path,
+    monkeypatch: pytest.MonkeyPatch,
+    installed,
+    run_dir,
+    cylc_vars
+):
+    """Check that CYLC_ environment vars exported during config file parsing
+       are appropriate to the workflow context (source, installed, or running).
+    """
+
+    # Purge environment from previous tests.
+    for key in cylc_vars.keys():
+        with suppress(KeyError):
+            del os.environ[key]
+
+    flow_file = tmp_path / WorkflowFiles.FLOW_FILE
+    flow_config = """
+    [scheduler]
+        allow implicit tasks = True
+    [scheduling]
+        [[graph]]
+            R1 = 'foo'
+    """
+
+    flow_file.write_text(flow_config)
+
+    # Make it look as if path is relative to cylc-run (i.e. installed).
+    monkeypatch.setattr(
+        'cylc.flow.config.is_relative_to',
+        lambda _a, _b: installed
+    )
+
+    # Parse the workflow config then check the environment.
+    WorkflowConfig(
+        workflow="name", fpath=flow_file, options=Mock(spec=[]),
+        run_dir=run_dir
+    )
+
+    cylc_env = [k for k in os.environ.keys() if k.startswith('CYLC_')]
+
+    for var, expected in cylc_vars.items():
+        if expected:
+            assert var in cylc_env
+        else:
+            assert var not in cylc_env
