@@ -320,8 +320,8 @@ def test_incorrect_environment_variables_raise_error(
     with pytest.raises(WorkflowFilesError) as excinfo:
         make_localhost_symlinks('rund', 'test_workflow')
     assert (
-        "'$doh/cylc-run/test_workflow' contains an invalid "
-        "environment variable"
+        "Can't symlink to $doh/cylc-run/test_workflow\n"
+        "Undefined variables, check global config: $doh"
     ) in str(excinfo.value)
 
 
@@ -438,21 +438,21 @@ def test_remove_empty_parents(tmp_path: Path):
     """Test that _remove_empty_parents() doesn't remove parents containing a
     sibling."""
     # -- Setup --
-    reg = 'foo/bar/baz/qux'
-    path = tmp_path.joinpath(reg)
+    id_ = 'foo/bar/baz/qux'
+    path = tmp_path.joinpath(id_)
     tmp_path.joinpath('foo/bar/baz').mkdir(parents=True)
     # Note qux does not exist, but that shouldn't matter
     sibling_reg = 'foo/darmok'
     sibling_path = tmp_path.joinpath(sibling_reg)
     sibling_path.mkdir()
     # -- Test --
-    remove_empty_parents(path, reg)
+    remove_empty_parents(path, id_)
     assert tmp_path.joinpath('foo/bar').exists() is False
     assert tmp_path.joinpath('foo').exists() is True
     # Check it skips non-existent dirs, and stops at the right place too
     tmp_path.joinpath('foo/bar').mkdir()
     sibling_path.rmdir()
-    remove_empty_parents(path, reg)
+    remove_empty_parents(path, id_)
     assert tmp_path.joinpath('foo').exists() is False
     assert tmp_path.exists() is True
 
@@ -529,7 +529,8 @@ def test_parse_rm_dirs__bad(dirs: List[str], err_msg: str):
             1000, ['run20', 'run400', 'run999'], False,
             id='1000th run (from filenames)'
         ),
-        param(6, ['run1', 'run5'], False,
+        param(
+            6, ['run1', 'run5'], False,
             id='Non-sequential (from filenames)'),
         param(2, ['run1'], True, id='2nd run (from symlink)'),
         param(100, ['run1', 'run99'], True, id='100th run (from symlink)'),
@@ -549,7 +550,8 @@ def test_get_next_rundir_number(tmp_path, expect, files, runN):
     (
         param('my_workflow1', 'my_workflow1', False, id='--no-run-name'),
         param('my_workflow2', 'my_workflow2/run22', False, id='installed'),
-        param('my_workflow3', 'my_workflow3/foo', False, id='--run-name="foo"'),
+        param(
+            'my_workflow3', 'my_workflow3/foo', False, id='--run-name="foo"'),
         param('my_workflow4', 'my_workflow4', True, id='not installed'),
     )
 )
