@@ -26,8 +26,8 @@ def test_run_cmd_stdin_str():
         capture_process=True
     )
     assert [s.strip() for s in proc.communicate()] == [
-        b'1bar2',
-        b''
+        '1bar2',
+        ''
     ]
 
 
@@ -43,8 +43,8 @@ def test_run_cmd_stdin_file(tmp_path):
         capture_process=True
     )
     assert [s.strip() for s in proc.communicate()] == [
-        b'1bar2',
-        b''
+        '1bar2',
+        ''
     ]
 
 
@@ -63,11 +63,26 @@ def test_construct_rsync_over_ssh_cmd():
         }
     )
     assert host == 'miklegard'
-    assert ' '.join(cmd) == (
-        'rsync command --delete --rsh=strange_ssh --include=/.service/ '
-        '--include=/.service/server.key -a --checksum '
-        '--out-format=%o %n%L --no-t --exclude=log --exclude=share '
-        '--exclude=work --include=/app/*** --include=/bin/*** '
-        '--include=/etc/*** --include=/lib/*** --exclude=* '
-        '/foo/ miklegard:/bar/'
-    )
+    assert cmd == [
+        'rsync',
+        'command',
+        '--delete',
+        '--rsh=strange_ssh',
+        '--include=/.service/',
+        '--include=/.service/server.key',
+        '-a',
+        '--checksum',
+        '--out-format=%o %n%L',
+        '--no-t',
+        '--exclude=log',
+        '--exclude=share',
+        '--exclude=work',
+        '--include=/ana/***',
+        '--include=/app/***',
+        '--include=/bin/***',
+        '--include=/etc/***',
+        '--include=/lib/***',
+        '--exclude=*',
+        '/foo/',
+        'miklegard:/bar/',
+    ]
