@@ -16,13 +16,16 @@
 
 import contextlib
 import os
+from pathlib import Path
 import sqlite3
 import unittest
 from unittest import mock
 from tempfile import mktemp
+from types import SimpleNamespace
 
 import pytest
 
+from cylc.flow.exceptions import PlatformLookupError
 from cylc.flow.rundb import CylcWorkflowDAO
 
 
@@ -97,56 +100,25 @@ def create_temp_db():
     conn.close()  # doesn't raise error on re-invocation
 
 
-def test_remove_columns():
-    """Test workaround for dropping columns in sqlite3."""
-    with create_temp_db() as (temp_db, conn):
-        conn.execute(
-            r'''
-                CREATE TABLE foo (
-                    bar,
-                    baz,
-                    pub
-                )
-            '''
-        )
-        conn.execute(
-            r'''
-                INSERT INTO foo
-                VALUES (?,?,?)
-            ''',
-            ['BAR', 'BAZ', 'PUB']
-        )
-        conn.commit()
-        conn.close()
-
-        dao = CylcWorkflowDAO(temp_db)
-        dao.remove_columns('foo', ['bar', 'baz'])
-
-        conn = dao.connect()
-        data = list(conn.execute(r'SELECT * from foo'))
-        assert data == [('PUB',)]
-
-
-def test_operational_error(monkeypatch, tmp_path, caplog):
+def test_operational_error(tmp_path, caplog):
     """Test logging on operational error."""
     # create a db object
     db_file = tmp_path / 'db'
-    dao = CylcWorkflowDAO(db_file)
+    with CylcWorkflowDAO(db_file) as dao:
+        # stage some stuff
+        dao.add_delete_item(CylcWorkflowDAO.TABLE_TASK_JOBS)
+        dao.add_insert_item(CylcWorkflowDAO.TABLE_TASK_JOBS, ['pub'])
+        dao.add_update_item(CylcWorkflowDAO.TABLE_TASK_JOBS, ['pub'])
 
-    # stage some stuff
-    dao.add_delete_item(CylcWorkflowDAO.TABLE_TASK_JOBS)
-    dao.add_insert_item(CylcWorkflowDAO.TABLE_TASK_JOBS, ['pub'])
-    dao.add_update_item(CylcWorkflowDAO.TABLE_TASK_JOBS, ['pub'])
+        # connect the to DB
+        dao.connect()
 
-    # connect the to DB
-    dao.connect()
+        # then delete the file - this will result in an OperationalError
+        db_file.unlink()
 
-    # then delete the file - this will result in an OperationalError
-    db_file.unlink()
-
-    # execute & commit the staged items
-    with pytest.raises(sqlite3.OperationalError):
-        dao.execute_queued_items()
+        # execute & commit the staged items
+        with pytest.raises(sqlite3.OperationalError):
+            dao.execute_queued_items()
 
     # ensure that the failed transaction is logged for debug purposes
     assert len(caplog.messages) == 1
@@ -155,3 +127,51 @@ def test_operational_error(monkeypatch, tmp_path, caplog):
     assert 'DELETE FROM task_jobs' in message
     assert 'INSERT OR REPLACE INTO task_jobs' in message
     assert 'UPDATE task_jobs' in message
+
+
+def test_table_creation(tmp_path: Path):
+    """Test tables are NOT created by default."""
+    db_file = tmp_path / 'db'
+    stmt = "SELECT name FROM sqlite_master WHERE type='table'"
+    with CylcWorkflowDAO(db_file) as dao:
+        tables = list(dao.connect().execute(stmt))
+    assert tables == []
+    with CylcWorkflowDAO(db_file, create_tables=True) as dao:
+        tables = [i[0] for i in dao.connect().execute(stmt)]
+    assert CylcWorkflowDAO.TABLE_WORKFLOW_PARAMS in tables
+
+
+def test_context_manager_exit(
+    monkeypatch: pytest.MonkeyPatch, tmp_path: Path
+):
+    """Test connection is closed even if an exception occurs somewhere."""
+    db_file = tmp_path / 'db'
+    mock_close = mock.Mock()
+    with monkeypatch.context() as mp:
+        mp.setattr(CylcWorkflowDAO, 'close', mock_close)
+        with CylcWorkflowDAO(db_file) as dao, pytest.raises(RuntimeError):
+            mock_close.assert_not_called()
+            raise RuntimeError('mock err')
+        mock_close.assert_called_once()
+    # Close connection for real:
+    dao.close()
+
+
+def test_select_task_pool_for_restart_if_not_platforms(tmp_path):
+    """Returns error if platform error or errors raised by callback.
+    """
+    # Setup a fake callback function which returns the fake "platform_name":
+    def callback(index, row):
+        return ''.join(row)
+
+    db_file = tmp_path / 'db'
+    dao = CylcWorkflowDAO(db_file, create_tables=True)
+    # Fiddle the connect method to return a list of fake "platform_names":
+    dao.connect = lambda: SimpleNamespace(execute=lambda _: ['foo', 'bar'])
+
+    # Assert that an error is raised and that it mentions both fake platforms:
+    with pytest.raises(
+        PlatformLookupError,
+        match='not defined.*\n.*foo.*\n.*bar'
+    ):
+        dao.select_task_pool_for_restart(callback)
