@@ -20,6 +20,7 @@ import pytest
 from cylc.flow import CYLC_LOG
 from cylc.flow.cycling.iso8601 import ISO8601Point, ISO8601Sequence, init
 from cylc.flow.exceptions import XtriggerConfigError
+from cylc.flow.id import Tokens
 from cylc.flow.subprocctx import SubFuncContext
 from cylc.flow.task_proxy import TaskProxy
 from cylc.flow.taskdef import TaskDef
@@ -158,7 +159,7 @@ def test_housekeeping_with_xtrigger_satisfied(xtrigger_mgr):
     sequence = ISO8601Sequence('P1D', '2019')
     tdef.xtrig_labels[sequence] = ["get_name"]
     start_point = ISO8601Point('2019')
-    itask = TaskProxy(tdef, start_point)
+    itask = TaskProxy(Tokens('~user/workflow'), tdef, start_point)
     # pretend the function has been activated
     xtrigger_mgr.active.append(xtrig.get_signature())
     xtrigger_mgr.callback(xtrig)
@@ -205,7 +206,7 @@ def test__call_xtriggers_async(xtrigger_mgr):
     init()
     start_point = ISO8601Point('2019')
     # create task proxy
-    itask = TaskProxy(tdef, start_point)
+    itask = TaskProxy(Tokens('~user/workflow'), tdef, start_point)
 
     # we start with no satisfied xtriggers, and nothing active
     assert len(xtrigger_mgr.sat_xtrig) == 0
@@ -306,7 +307,7 @@ def test_check_xtriggers(xtrigger_mgr):
     sequence = ISO8601Sequence('P1D', '2019')
     tdef1.xtrig_labels[sequence] = ["get_name"]
     start_point = ISO8601Point('2019')
-    itask1 = TaskProxy(tdef1, start_point)
+    itask1 = TaskProxy(Tokens('~user/workflow'), tdef1, start_point)
     itask1.state.xtriggers["get_name"] = False  # satisfied?
 
     # add a clock xtrigger
@@ -330,7 +331,7 @@ def test_check_xtriggers(xtrigger_mgr):
     init()
     start_point = ISO8601Point('20000101T0000+05')
     # create task proxy
-    TaskProxy(tdef2, start_point)
+    TaskProxy(Tokens('~user/workflow'), tdef2, start_point)
 
     xtrigger_mgr.check_xtriggers(itask1, lambda foo: None)
     # won't be satisfied, as it is async, we are are not calling callback
